@@ -35,9 +35,10 @@
 #include "filterparameter.h"
 #include "GLLogStream.h"
 #include "meshmodel.h"
-#include "mlparameter.h"
+#include "scriptinterface.h"
 #include "xmlfilterinfo.h"
 
+
 class QWidget;
 class QGLWidget;
 class QString;
@@ -47,6 +48,7 @@ class QTreeWidgetItem;
 class MeshModel;
 class RenderMode;
 class GLArea;
+class GLAreaReg;
 class QScriptEngine;
 
 
@@ -58,7 +60,7 @@ class MainWindowInterface
 {
 public:
   virtual void executeFilter(QAction *, RichParameterSet &, bool /* isPreview */){}
-  virtual void executeFilter(MeshLabXMLFilterContainer*, FilterEnv& , bool /* isPreview */){}
+  virtual void executeFilter(MeshLabXMLFilterContainer*, EnvWrap& , bool /* isPreview */){}
 };
 
 /** \brief The MeshLabInterface class is the base of all the plugin interfaces.
@@ -92,6 +94,7 @@ public:
 	// NEVER EVER use a msgbox to say something to the user.
   void Log(const char * f, ... );
   void Log(int Level, const char * f, ... ) ;
+  void RealTimeLog(QString Id, const char * f, ... ) ;
 };
 
 class MeshCommonInterface : public MeshLabInterface
@@ -129,18 +132,6 @@ public:
    If a filter wants to save some permanent stuff should set the permanent default values.
 */
 	virtual void initGlobalParameterSet(QAction * /*format*/, RichParameterSet & /*globalparam*/) {}
-
-	// The longer string describing each filtering action. 
-	// This string is printed in the top of the parameter window 
-	// so it should be at least one or two paragraphs long.
-	// you can use simple html formatting tags (like <br> <b> and <i>) to improve readability.
-	// This string is used in the About plugin dialog and by meshlabserver to create the filter list page.
-	virtual QString filterInfo(QAction* action) const =0;
-
-	// The very short string (a few words) describing each filtering action 
-	// This string is used also to define the menu entry
-  virtual QString filterName(FilterIDType ) const =0;
-
 };
 /** \brief The MeshIOInterface is the base class for all the single mesh loading plugins.
   */
@@ -161,20 +152,7 @@ public:
 	virtual QList<Format> importFormats() const = 0;
 	virtual QList<Format> exportFormats() const = 0;
 
-	
-	// The longer string describing each filtering action. 
-	// This string is printed in the top of the parameter window 
-	// so it should be at least one or two paragraphs long.
-	// you can use simple html formatting tags (like <br> <b> and <i>) to improve readability.
-	// This string is used in the About plugin dialog and by meshlabserver to create the filter list page.
-	QString filterInfo(QAction* /*action*/) const {return QString();};
-
-	// The very short string (a few words) describing each filtering action 
-	// This string is used also to define the menu entry
-  QString filterName(FilterIDType) const {return QString();};
-
-	
-	// This function is called to initialize the list of additional parameters that a OPENING filter could require 
+  // This function is called to initialize the list of additional parameters that a OPENING filter could require
 	// it is called by the framework BEFORE the actual mesh loading to perform to determine how parse the input file
 	// The instanced parameters are then passed to the open at the loading time.
 	// Typical example of use to decide what subportion of a mesh you have to load.
@@ -208,7 +186,7 @@ public:
       const RichParameterSet & par,	/// The parameters that have been set up in the initPreOpenParameter()
       vcg::CallBackPos *cb=0,					/// standard callback for reporting progress in the loading
       QWidget *parent=0)=0;						/// you should not use this...
-    
+
   virtual bool save(
       const QString &format, // the extension of the format e.g. "PLY"
 			const QString &fileName,
@@ -222,6 +200,7 @@ public:
 	/// io plugins should avoid using QMessageBox for reporting errors. 
 	/// Failure should put some meaningful information inside the errorMessage string.
 	virtual QString &errorMsg() {return this->errorMessage;}
+	void clearErrorString() {errorMessage.clear();}
 	
 	// this string is used to pass back to the framework error messages in case of failure of a filter apply.
 	// NEVER EVER use a msgbox to say something to the user.
@@ -252,6 +231,7 @@ public:
         Smoothing        =0x00040, /*!<  Stuff that does not change the topology, but just the vertex positions*/
         Quality          =0x00080,
         Layer            =0x00100, /*!<  Layers, attributes */
+        RasterLayer      =0x20000, /*!<  Raster Layers, attributes */
         Normal           =0x00200, /*!<  Normal, Curvature, orientation (rotations and transformations fall here)*/
         Sampling         =0x00400,
         Texture          =0x00800,
@@ -263,11 +243,18 @@ public:
     };
 	
 	
+	
 	MeshFilterInterface() : MeshCommonInterface() 
 	{
 	}
 	virtual ~MeshFilterInterface() {}
 
+
+  /** The very short string (a few words) describing each filtering action
+  // This string is used also to define the menu entry
+  */
+  virtual QString filterName(FilterIDType ) const =0;
+
   /** The long, formatted string describing each filtering action.
 	// This string is printed in the top of the parameter window 
   // so it should be at least one or two paragraphs long. The more the better.
@@ -316,7 +303,7 @@ public:
 	// It is widely used by the meshlab's preview system.
 	//TO BE REPLACED WITH = 0
   */
-	virtual int postCondition( QAction* ) const {return MeshModel::MM_UNKNOWN;};
+  virtual int postCondition( QAction* ) const {return MeshModel::MM_UNKNOWN;}
 
   /** \brief applies the selected filter with the already stabilished parameters
   * This function is called by the framework after getting values for the parameters specified in the \ref InitParameterSet
@@ -350,9 +337,8 @@ public:
     * Failing filters should put some meaningful information inside the errorMessage string and return false with the \ref applyFilter
     */
 	const QString &errorMsg() {return this->errorMessage;}
-	virtual QString filterInfo(QAction *a) const {return this->filterInfo(ID(a));};
-	virtual QString filterName(QAction *a) const {return this->filterName(ID(a));};
-  virtual QString filterName(FilterIDType /*filterID*/) const =0;// const {return MeshLabInterface::filterName(filterID);};
+  virtual QString filterInfo(QAction *a) const {return this->filterInfo(ID(a));}
+  virtual QString filterName(QAction *a) const {return this->filterName(ID(a));}
   virtual QString filterScriptFunctionName(FilterIDType /*filterID*/) {return "";}
 
 
@@ -405,6 +391,10 @@ public:
 	      BE CAREFUL! this function does NOT change in anyway the state of the MeshModel!!!! **/
 	  int previewOnCreatedAttributes(QAction* act,const MeshModel& mm);
 	QString generatedScriptCode;
+
+	/** If you need to init your QGLContext in order to use GPU redefine this function. */
+	virtual bool initGLContext() {return true;}
+
 	QGLContext* glContext;
 protected:
     // Each plugins exposes a set of filtering possibilities. 
@@ -446,24 +436,11 @@ public:
 	MeshRenderInterface() :MeshCommonInterface() {}
     virtual ~MeshRenderInterface() {}
 		
-  virtual void Init(QAction * /*mode*/, MeshDocument &/*m*/, RenderMode &/*rm*/, QGLWidget * /*parent*/){};
+  virtual void Init(QAction * /*mode*/, MeshDocument &/*m*/, RenderMode &/*rm*/, QGLWidget * /*parent*/){}
 	virtual void Render(QAction * /*mode*/, MeshDocument &/*md*/, RenderMode &/*rm*/, QGLWidget * /*parent*/) = 0;
-	virtual void Finalize(QAction * /*mode*/, MeshDocument &/*m*/, GLArea * /*parent*/){};
+  virtual void Finalize(QAction * /*mode*/, MeshDocument */*m*/, GLArea * /*parent*/){}
 	virtual bool isSupported() = 0;
 	virtual QList<QAction *> actions() = 0;
-
-	// The longer string describing each filtering action. 
-	// This string is printed in the top of the parameter window 
-	// so it should be at least one or two paragraphs long.
-	// you can use simple html formatting tags (like <br> <b> and <i>) to improve readability.
-	// This string is used in the About plugin dialog and by meshlabserver to create the filter list page.
-	QString filterInfo(QAction* /*action*/) const {return QString();};
-
-	// The very short string (a few words) describing each filtering action 
-	// This string is used also to define the menu entry
-	QString filterName(FilterIDType /*action*/) const {return QString();};
-
-
 };
 /**
   MeshDecorateInterface is the base class of all <b> decorators </b>
@@ -504,17 +481,24 @@ public:
 
   MeshDecorateInterface(): MeshCommonInterface() {}
   virtual ~MeshDecorateInterface() {}
+  /** The very short string (a few words) describing each filtering action
+  // This string is used also to define the menu entry
+  */
+  virtual QString decorationName(FilterIDType ) const =0;
+  virtual QString decorationInfo(FilterIDType ) const =0;
+  virtual QString decorationInfo(QAction *a) const {return decorationInfo(ID(a));}
+
 
   virtual bool startDecorate(QAction * /*mode*/, MeshDocument &/*m*/, RichParameterSet * /*param*/, GLArea * /*parent*/) =0;
   virtual void decorate(QAction * /*mode*/,  MeshDocument &/*m*/, RichParameterSet *, GLArea * /*parent*/, QPainter */*p*/) = 0;
-  virtual void endDecorate(QAction * /*mode*/,   MeshDocument &/*m*/, RichParameterSet *, GLArea * /*parent*/){};
+  virtual void endDecorate(QAction * /*mode*/,   MeshDocument &/*m*/, RichParameterSet *, GLArea * /*parent*/){}
 
   /** \brief tests if a decoration is applicable to a mesh.
   For istance curvature cannot be shown on a mesh without curvature.
   On failure (returning false) the function fills the MissingItems list with strings describing the missing items.
   It is invoked only for decoration of \i PerMesh class;
   */
-  virtual bool isDecorationApplicable(QAction */*action*/, const MeshModel& /*m*/, QString&/*MissingItems*/) const {return true;};
+  virtual bool isDecorationApplicable(QAction */*action*/, const MeshModel& /*m*/, QString&/*MissingItems*/) const {return true;}
 
   virtual int getDecorationClass(QAction */*action*/) const {return Generic;}
 
@@ -526,7 +510,7 @@ protected:
 	virtual FilterIDType ID(QAction *a) const
 	{
 		foreach( FilterIDType tt, types())
-			if( a->text() == this->filterName(tt) ) return tt;
+      if( a->text() == this->decorationName(tt) ) return tt;
 		qDebug("unable to find the id corresponding to action  '%s'",qPrintable(a->text()));
 		assert(0);
 		return -1;
@@ -549,19 +533,8 @@ public:
 	//should return a sentence describing what the editing tool does
 	static const QString Info();
 
-	// The longer string describing each filtering action. 
-	// This string is printed in the top of the parameter window 
-	// so it should be at least one or two paragraphs long.
-	// you can use simple html formatting tags (like <br> <b> and <i>) to improve readability.
-	// This string is used in the About plugin dialog and by meshlabserver to create the filter list page.
-	QString filterInfo(QAction* /*action*/) const {return QString();};
-
-	// The very short string (a few words) describing each filtering action 
-	// This string is used also to define the menu entry
-	QString filterName(FilterIDType ) const {return QString();};
-
-	// Called when the user press the first time the button 
-	virtual bool StartEdit(MeshModel &/*m*/, GLArea * /*parent*/){return true;};
+  // Called when the user press the first time the button
+    virtual bool StartEdit(MeshModel &/*m*/, GLArea * /*parent*/){return true;}
 	virtual bool StartEdit(MeshDocument &md, GLArea *parent)
 	{
 		//assert(NULL != md.mm());
@@ -570,14 +543,22 @@ public:
 		else return false;
 	}
 	// Called when the user press the second time the button 
-	virtual void EndEdit(MeshModel &/*m*/, GLArea * /*parent*/){};
+	virtual void EndEdit(MeshModel &/*m*/, GLArea * /*parent*/){}
     
+
+	// There are two classes of editing tools, the one that works on a single layer at a time
+	// and the ones that works on all layers and have to manage in a correct way the action of changing the current layer.
+	// For the edit tools that works ona single layer changing the layer means the restart of the edit tool.
+	virtual bool isSingleMeshEdit() const { return true; }
+
 	// Called when the user changes the selected layer
 	//by default it calls end edit with the layer that was selected and start with the new layer that is
 	//selected.  This ensures that plugins who dont support layers do not get sent pointers to meshes
-	//they are not expecting
+	//they are not expecting.
+	// If your editing plugins is not singleMesh you MUST reimplement this to correctly handle the change of layer.
 	virtual void LayerChanged(MeshDocument &md, MeshModel &oldMeshModel, GLArea *parent)
 	{
+		assert(this->isSingleMeshEdit());
 		EndEdit(oldMeshModel, parent);
 		StartEdit(md, parent);
 	}
@@ -588,9 +569,9 @@ public:
 	virtual void mousePressEvent  (QMouseEvent *event, MeshModel &/*m*/, GLArea * )=0;
 	virtual void mouseMoveEvent   (QMouseEvent *event, MeshModel &/*m*/, GLArea * )=0;
 	virtual void mouseReleaseEvent(QMouseEvent *event, MeshModel &/*m*/, GLArea * )=0;
-	//virtual void wheelEvent     (QWheelEvent*e, MeshModel &/*m*/, GLArea * );
-  virtual void keyReleaseEvent  (QKeyEvent *, MeshModel &/*m*/, GLArea *){}
+	  virtual void keyReleaseEvent  (QKeyEvent *, MeshModel &/*m*/, GLArea *){}
   virtual void keyPressEvent    (QKeyEvent *, MeshModel &/*m*/, GLArea *){}
+  virtual void wheelEvent(QWheelEvent*, MeshModel &/*m*/, GLArea * ){}
   virtual void tabletEvent(QTabletEvent * e, MeshModel &/*m*/, GLArea *){e->ignore();}
 };
 
@@ -617,24 +598,49 @@ public:
 
 };
 
+
+
 /**************************************************************************************************************************************************************/
 /*The new class of filter defined through XML file*/
 
-class MeshLabFilterInterface : public MeshLabInterface
+typedef bool SignalCallBack();
+
+class MeshLabFilterInterface : public QObject, public MeshLabInterface
 {
-private:
-	static QMap<QString,MeshModel::MeshElement> convertingMap;
-	static void initConvertingMap();
-	static bool init;
+	Q_OBJECT
 public:
 	MeshLabFilterInterface();
 	virtual ~MeshLabFilterInterface() {}
 
 	QGLContext* glContext;
+
+	static void initConvertingMap(QMap<QString,MeshModel::MeshElement>& convertingMap);
+	static void initConvertingCategoryMap(QMap<QString,MeshFilterInterface::FilterClass>& convertingMap);
 	static bool arePreCondsValid(const int filterPreConds,const MeshModel& m, QStringList &MissingItems);
 	static int convertStringListToMeshElementEnum(const QStringList& stringListEnum);
+	static int convertStringListToCategoryEnum(const QStringList& stringListEnum);
+	virtual bool applyFilter(const QString& filterName,MeshDocument& md,EnvWrap& env, vcg::CallBackPos* cb) =0;
+	const QString &errorMsg() {return this->errorMessage;}
+public slots:
+	inline void setInterrupt(const bool& inter) {intteruptreq = inter;};
 
-	virtual bool applyFilter(const QString& filterName,MeshDocument& md,const FilterEnv& env, vcg::CallBackPos *cb) =0;
+protected:
+	////This function has two different aims: 
+	////1) should be invoked by filters in order to request a redraw of a subset of meshes and/or rasters inside the MeshDocument. 
+	////2) like a synchronization point where the filter can safely stop is execution.
+	////if filter has not a pending interrupt request a render state update request will be sent to the framework.
+	////return value: true if the request has been sent, false otherwise (filter has an interrupt request).
+
+	//bool sendUpdateRequest(const MeshDocument& md,);
+
+	//QList<int> meshestobeupdated;
+	//int meshmaskattributestobeupdated;
+	//QList<int> rasterstobeupdated;
+	//int rastermaskattributestobeupdated;
+
+	// this string is used to pass back to the framework error messages in case of failure of a filter apply.
+	QString errorMessage;
+	bool intteruptreq;
 };
 
 Q_DECLARE_INTERFACE(MeshIOInterface,						"vcg.meshlab.MeshIOInterface/1.0")
