@@ -28,6 +28,8 @@
 #include "meshmodel.h"
 #include <wrap/gl/math.h>
 #include "scriptinterface.h"
+#include <vcg/complex/append.h>
+
 
 using namespace vcg;
 
@@ -42,6 +44,8 @@ MeshDocument::~MeshDocument()
 {
   foreach(MeshModel *mmp, meshList)
     delete mmp;
+  foreach(RasterModel* rmp,rasterList)
+	  delete rmp;
 }
 
 //returns the mesh ata given position in the list
@@ -90,17 +94,34 @@ QList<TagBase *> MeshDocument::getMeshTags(int meshId)
 
 void MeshDocument::setCurrentMesh( int i)
 {
-  foreach(MeshModel *mmp, meshList)
+	if(i<0)
+	{
+		currentMesh=0;
+		return;
+	}
+
+	foreach(MeshModel *mmp, meshList)
+	{
+		if(mmp->id() == i)
+		{
+			currentMesh = mmp;
+			emit currentMeshChanged(i);
+			return;
+		}
+	}
+	assert(0);
+	return;
+}
+
+//returns the raster at a given position in the list
+RasterModel *MeshDocument::getRaster(int i)
+{
+  foreach(RasterModel *rmp, rasterList)
   {
-    if(mmp->id() == i)
-    {
-      currentMesh = mmp;
-      emit currentMeshChanged(i);
-      return;
-    }
+    if(rmp->id() == i) return rmp;
   }
-  assert(0);
-  return;
+  //assert(0);
+  return 0;
 }
 
 //if i is <0 it means that no currentRaster is set
@@ -127,24 +148,42 @@ void MeshDocument::setCurrentRaster( int i)
 template <class LayerElement>
 QString NameDisambiguator(QList<LayerElement*> &elemList, QString meshLabel )
 {
-  QFileInfo info(meshLabel);
-  QString newName=info.fileName();
+  QString newName=meshLabel;
   typename QList<LayerElement*>::iterator mmi;
+
   for(mmi=elemList.begin(); mmi!=elemList.end(); ++mmi)
   {
-    if((*mmi)->label() == newName)
+    if((*mmi)->label() == newName) // if duplicated name found
     {
       QFileInfo fi((*mmi)->label());
       QString baseName = fi.baseName(); //  all characters in the file up to the first '.' Eg "/tmp/archive.tar.gz" -> "archive"
-      int lastNum = baseName.right(1).toInt();
-      if( baseName.right(2).toInt() >= 10)
-        lastNum = baseName.right(2).toInt();
-      if(lastNum)
-        newName = baseName.left(baseName.length()-(lastNum<10?1:2))+QString::number(lastNum+1);
+      QString suffix = fi.suffix();
+      bool ok;
+ 
+      // if name ends with a number between parenthesis (XXX), 
+      // it was himself a duplicated name, and we need to 
+      // just increase the number between parenthesis 
+      int numDisamb;
+      int startDisamb;
+      int endDisamb;
+
+      startDisamb = baseName.lastIndexOf("(");
+      endDisamb   = baseName.lastIndexOf(")");
+      if((startDisamb!=-1)&&(endDisamb!=-1))
+        numDisamb = (baseName.mid((startDisamb+1),(endDisamb-startDisamb-1))).toInt(&ok);
       else
-        newName = baseName+"_1";
-      if (info.suffix() != QString(""))
-      newName = newName + "." + info.suffix();
+        numDisamb = 0;
+      
+      if(startDisamb!=-1)
+        newName = baseName.left(startDisamb)+ "(" + QString::number(numDisamb+1) + ")";
+      else
+        newName = baseName + "(" + QString::number(numDisamb+1) + ")";
+
+      if (suffix != QString(""))
+      newName = newName + "." + suffix;
+
+      // now recurse to see if the new name is free
+      newName = NameDisambiguator(elemList, newName);
     }
   }
   return newName;
@@ -168,7 +207,6 @@ MeshModel * MeshDocument::addNewMesh(QString fullPath, QString label, bool setAs
 
   MeshModel *newMesh = new MeshModel(this,qPrintable(fullPath),newlabel);
   meshList.push_back(newMesh);
-
   emit meshSetChanged();
 
   if(setAsCurrent)
@@ -178,14 +216,16 @@ MeshModel * MeshDocument::addNewMesh(QString fullPath, QString label, bool setAs
 
 bool MeshDocument::delMesh(MeshModel *mmToDel)
 {
-	if(meshList.size()==1) return false;
-  if(!meshList.removeOne(mmToDel)) return false;
-  if(currentMesh == mmToDel)
-        setCurrentMesh(this->meshList.at(0)->id());
+	if(!meshList.removeOne(mmToDel)) 
+		return false;
+	if((currentMesh == mmToDel) && (meshList.size() != 0))
+		setCurrentMesh(this->meshList.at(0)->id());
+	else if (meshList.size() == 0)
+			setCurrentMesh(-1);
 
-  delete mmToDel;
+	delete mmToDel;
 
-  emit meshSetChanged();
+	emit meshSetChanged();
 	return true;
 }
 
@@ -224,8 +264,12 @@ bool MeshDocument::delRaster(RasterModel *rasterToDel)
 	}
 
 	if(currentRaster == rasterToDel)
-		setCurrentRaster(-1);
-
+	{
+		if (rasterList.size() > 0)
+			setCurrentRaster(rasterList.at(0)->id());
+		else
+			setCurrentRaster(-1);
+	}
 	emit rasterSetChanged();
 
 	return true;
@@ -258,96 +302,93 @@ bool MeshDocument::hasBeenModified()
 	return false;
 }
 
-MeshModel::MeshModel(MeshDocument *_parent, QString fullFileName, QString labelName) 
+void MeshDocument::updateRenderStateMeshes(const QList<int>& mm,const int meshupdatemask)
 {
-	meshModified() = false;
-  parent=_parent;
-	glw.m=&cm;
-	_id=parent->newMeshId();
-	// These data are always active on the mesh
-	currentDataMask = MM_NONE;
-	currentDataMask |= MM_VERTCOORD | MM_VERTNORMAL | MM_VERTFLAG ;
-	currentDataMask |= MM_FACEVERT  | MM_FACENORMAL | MM_FACEFLAG ;
-
-	visible=true;
-	cm.Tr.SetIdentity();
-	cm.sfn=0;
-	cm.svn=0;
-
-  if(!fullFileName.isEmpty())   this->fullPathFileName=fullFileName;
-
-  if(!labelName.isEmpty())     this->_label=labelName;
+	static QTime currTime;
+	if(currTime.elapsed()< 100) 
+		return;
+	for (QList<int>::const_iterator mit = mm.begin();mit != mm.end();++mit)
+	{
+		MeshModel* mesh = getMesh(*mit);
+		if (mesh != NULL)
+			renderState().update(mesh->id(),mesh->cm,meshupdatemask);
+	}
+	if ((mm.size() > 0) && (meshupdatemask != MeshModel::MM_NONE))
+		emit documentUpdated();
+	currTime.start();
 }
 
-
-bool MeshModel::Render(GLW::DrawMode _dm, GLW::ColorMode _cm, GLW::TextureMode _tm)
-  {
-      // Needed to be defined here for splatrender as long there is no "MeshlabCore" library.
-      using namespace vcg;
-      glPushMatrix();
-      glMultMatrix(cm.Tr);
-      if( (_cm == GLW::CMPerFace)  && (!tri::HasPerFaceColor(cm)) ) _cm=GLW::CMNone;
-      if( (_tm == GLW::TMPerWedge )&& (!tri::HasPerWedgeTexCoord(cm)) ) _tm=GLW::TMNone;
-      if( (_tm == GLW::TMPerWedgeMulti )&& (!tri::HasPerWedgeTexCoord(cm)) ) _tm=GLW::TMNone;
-      glw.Draw(_dm,_cm,_tm);
-      glPopMatrix();
-      return true;
-  }
-
-bool MeshModel::RenderSelectedFace()
+void MeshDocument::updateRenderStateRasters(const QList<int>& rm,const int rasterupdatemask)
 {
-  glPushAttrib(GL_ALL_ATTRIB_BITS);
-  glEnable(GL_POLYGON_OFFSET_FILL);
-  glDisable(GL_LIGHTING);
-  glDisable(GL_TEXTURE_2D);
-  glEnable(GL_BLEND);
-  glDepthMask(GL_FALSE);
-  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
-  glColor4f(1.0f,0.0,0.0,.3f);
-  glPolygonOffset(-1.0, -1);
-  CMeshO::FaceIterator fi;
-	glPushMatrix();
-	glMultMatrix(cm.Tr);glBegin(GL_TRIANGLES);
-	cm.sfn=0;
-	for(fi=cm.face.begin();fi!=cm.face.end();++fi)
-    if(!(*fi).IsD() && (*fi).IsS())
-    {
-  		glVertex((*fi).cP(0));
-  		glVertex((*fi).cP(1));
-  		glVertex((*fi).cP(2));
-			++cm.sfn;
-    }
-  glEnd();
-	glPopMatrix();
-	glPopAttrib();
-  return true;
+	static QTime currTime;
+	if(currTime.elapsed()< 100) 
+		return;
+	for (QList<int>::const_iterator rit = rm.begin();rit != rm.end();++rit)
+	{
+		RasterModel* raster = getRaster(*rit);
+		if (raster != NULL)
+			renderState().update(raster->id(),*raster,rasterupdatemask);
+	}
+	if ((rm.size() > 0) && (rasterupdatemask != RasterModel::RM_NONE))
+		emit documentUpdated();
+	currTime.start();
 }
-bool MeshModel::RenderSelectedVert()
+
+void MeshDocument::updateRenderState(const QList<int>& mm,const int meshupdatemask,const QList<int>& rm,const int rasterupdatemask)
 {
-  glPushAttrib(GL_ALL_ATTRIB_BITS);
-  glDisable(GL_LIGHTING);
-  glDisable(GL_TEXTURE_2D);
-  glEnable(GL_BLEND);
-  glDepthMask(GL_FALSE);
-  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
-  glColor4f(1.0f,0.0,0.0,.3f);
-  glDepthRange(0.00,0.999);
-  glPointSize(3.0);
-  glPushMatrix();
-  glMultMatrix(cm.Tr);
-  glBegin(GL_POINTS);
+	static QTime currTime;
+	if(currTime.elapsed()< 100) 
+		return;
+	for (QList<int>::const_iterator mit = mm.begin();mit != mm.end();++mit)
+	{
+		MeshModel* mesh = getMesh(*mit);
+		if (mesh != NULL)
+			renderState().update(mesh->id(),mesh->cm,meshupdatemask);
+	}
+    for (QList<int>::const_iterator rit = rm.begin();rit != rm.end();++rit)
+	{
+		RasterModel* raster = getRaster(*rit);
+		if (raster != NULL)
+			renderState().update(raster->id(),*raster,rasterupdatemask);
+	}
+	if (((mm.size() > 0) && (meshupdatemask != MeshModel::MM_NONE)) || (rm.size() > 0 && (rasterupdatemask != RasterModel::RM_NONE)))
+		emit documentUpdated();
+	currTime.start();
+}
+
+void MeshModel::Clear()
+{
+  meshModified() = false;
+  glw.m=&cm;
+  // These data are always active on the mesh
+  currentDataMask = MM_NONE;
+  currentDataMask |= MM_VERTCOORD | MM_VERTNORMAL | MM_VERTFLAG ;
+  currentDataMask |= MM_FACEVERT  | MM_FACENORMAL | MM_FACEFLAG ;
+
+  visible=true;
+  cm.Tr.SetIdentity();
+  cm.sfn=0;
   cm.svn=0;
-  CMeshO::VertexIterator vi;
-  for(vi=cm.vert.begin();vi!=cm.vert.end();++vi)
-    if(!(*vi).IsD() && (*vi).IsS())
-    {
-      glVertex((*vi).cP());
-      ++cm.svn;
-    }
-  glEnd();
-  glPopMatrix();
-  glPopAttrib();
-  return true;
+}
+
+void MeshModel::UpdateBoxAndNormals()
+{
+  tri::UpdateBounding<CMeshO>::Box(cm);
+  if(cm.fn>0) {
+    tri::UpdateNormals<CMeshO>::PerFaceNormalized(cm);
+    tri::UpdateNormals<CMeshO>::PerVertexAngleWeighted(cm);
+  }
+}
+
+MeshModel::MeshModel(MeshDocument *_parent, QString fullFileName, QString labelName)
+:MeshLabRenderMesh()
+{
+
+  Clear();
+  parent=_parent;
+  _id=parent->newMeshId();
+  if(!fullFileName.isEmpty())   this->fullPathFileName=fullFileName;
+  if(!labelName.isEmpty())     this->_label=labelName;
 }
 
 QString MeshModel::relativePathName() const
@@ -398,27 +439,66 @@ int MeshModel::io2mm(int single_iobit)
 	} ;
 }
 
-Plane::Plane(RasterModel *_parent, const QString pathName, const QString _semantic){
-	parent = _parent;
+Plane::Plane(const Plane& pl)
+{
+	semantic = pl.semantic;
+	fullPathFileName = pl.fullPathFileName;
+	image = QImage(pl.image);
+}
+
+Plane::Plane(const QString pathName, const QString _semantic)
+{
 	semantic =_semantic;
 	fullPathFileName = pathName;
 
 	image = QImage(pathName);
 }
 
-RasterModel::RasterModel(MeshDocument *parent, QString _rasterName) {
+RasterModel::RasterModel(MeshDocument *parent, QString _rasterName)
+: MeshLabRenderRaster() 
+{
   _id=parent->newRasterId(); 
   par = parent;
   this->_label= _rasterName;
-	visible=false;
+  visible=true;
 }
 
-void RasterModel::addPlane(Plane *plane)
+RasterModel::RasterModel()
+: MeshLabRenderRaster() 
+{
+
+}
+
+
+MeshLabRenderRaster::MeshLabRenderRaster()
+{
+
+}
+
+MeshLabRenderRaster::MeshLabRenderRaster( const MeshLabRenderRaster& rm )
+:shot(rm.shot),planeList()
+{
+	for(QList<Plane*>::const_iterator it = rm.planeList.begin();it != rm.planeList.end();++it)	
+	{
+		planeList.push_back(new Plane(**it));
+		if (rm.currentPlane == *it)
+			currentPlane = planeList[planeList.size() - 1];
+	}
+}
+
+void MeshLabRenderRaster::addPlane(Plane *plane)
 {
 	planeList.append(plane);
 	currentPlane = plane;
 }
 
+MeshLabRenderRaster::~MeshLabRenderRaster()
+{
+	currentPlane = NULL;
+	for(int ii = 0;ii < planeList.size();++ii)
+		delete planeList[ii];
+}
+
 void MeshModelState::create(int _mask, MeshModel* _m)
 {
     m=_m;
@@ -485,7 +565,8 @@ void MeshModelState::create(int _mask, MeshModel* _m)
 
 bool MeshModelState::apply(MeshModel *_m)
 {
-  if(_m != m) return false;
+  if(_m != m) 
+	  return false;
     if(changeMask & MeshModel::MM_VERTCOLOR)
     {
         if(vertColor.size() != m->cm.vert.size()) return false;
@@ -594,14 +675,14 @@ void MeshModel::updateDataMask(int neededDataMask)
   if( ( (neededDataMask & MM_VERTRADIUS)!=0)		&& !hasDataMask(MM_VERTRADIUS))			cm.vert.EnableRadius();
   if( ( (neededDataMask & MM_VERTTEXCOORD)!=0)  && !hasDataMask(MM_VERTTEXCOORD))		cm.vert.EnableTexCoord();
 
-  if(  ( (neededDataMask & MM_FACEFLAGBORDER) && !hasDataMask(MM_FACEFLAGBORDER) ) ||
-       ( (neededDataMask & MM_VERTFLAGBORDER) && !hasDataMask(MM_VERTFLAGBORDER) )    )
-  {
-    if( (currentDataMask & MM_FACEFACETOPO) || (neededDataMask & MM_FACEFACETOPO))
-         tri::UpdateFlags<CMeshO>::FaceBorderFromFF(cm);
-    else tri::UpdateFlags<CMeshO>::FaceBorderFromNone(cm);
-    tri::UpdateFlags<CMeshO>::VertexBorderFromFace(cm);
-  }
+//  if(  ( (neededDataMask & MM_FACEFLAGBORDER) && !hasDataMask(MM_FACEFLAGBORDER) ) ||
+//       ( (neededDataMask & MM_VERTFLAGBORDER) && !hasDataMask(MM_VERTFLAGBORDER) )    )
+//  {
+//    if( (currentDataMask & MM_FACEFACETOPO) || (neededDataMask & MM_FACEFACETOPO))
+//         tri::UpdateFlags<CMeshO>::FaceBorderFromFF(cm);
+//    else tri::UpdateFlags<CMeshO>::FaceBorderFromNone(cm);
+//    tri::UpdateFlags<CMeshO>::VertexBorderFromFace(cm);
+//  }
 
   currentDataMask |= neededDataMask;
  }
@@ -627,10 +708,14 @@ void MeshModel::clearDataMask(int unneededDataMask)
 
 void MeshModel::Enable(int openingFileMask)
 {
-  if( openingFileMask & tri::io::Mask::IOM_VERTTEXCOORD ) updateDataMask(MM_VERTTEXCOORD);
-  if( openingFileMask & tri::io::Mask::IOM_WEDGTEXCOORD ) updateDataMask(MM_WEDGTEXCOORD);
-  if( openingFileMask & tri::io::Mask::IOM_VERTCOLOR    ) updateDataMask(MM_VERTCOLOR);
-  if( openingFileMask & tri::io::Mask::IOM_FACECOLOR    ) updateDataMask(MM_FACECOLOR);
+  if( openingFileMask & tri::io::Mask::IOM_VERTTEXCOORD ) 
+	  updateDataMask(MM_VERTTEXCOORD);
+  if( openingFileMask & tri::io::Mask::IOM_WEDGTEXCOORD ) 
+	  updateDataMask(MM_WEDGTEXCOORD);
+  if( openingFileMask & tri::io::Mask::IOM_VERTCOLOR    ) 
+	  updateDataMask(MM_VERTCOLOR);
+  if( openingFileMask & tri::io::Mask::IOM_FACECOLOR    ) 
+	  updateDataMask(MM_FACECOLOR);
   if( openingFileMask & tri::io::Mask::IOM_VERTRADIUS   ) updateDataMask(MM_VERTRADIUS);
   if( openingFileMask & tri::io::Mask::IOM_CAMERA       ) updateDataMask(MM_CAMERA);
   if( openingFileMask & tri::io::Mask::IOM_VERTQUALITY  ) updateDataMask(MM_VERTQUALITY);
@@ -641,4 +726,447 @@ void MeshModel::Enable(int openingFileMask)
 bool& MeshModel::meshModified()
 {
 	return this->modified;
-}
\ No newline at end of file
+}
+
+MeshLabRenderMesh::MeshLabRenderMesh()
+:glw(),cm()
+{
+}
+
+MeshLabRenderMesh::MeshLabRenderMesh(CMeshO& mesh )
+:glw(),cm()
+{
+	vcg::tri::Append<CMeshO,CMeshO>::MeshCopy(cm,mesh);
+	//cm.Tr = mesh.Tr;
+	cm.Tr.SetIdentity();
+	cm.sfn = mesh.sfn;
+	cm.svn = mesh.svn;
+	glw.m = &cm; 
+}
+
+bool MeshLabRenderMesh::render(vcg::GLW::DrawMode dm,vcg::GLW::ColorMode colm,vcg::GLW::TextureMode tm )
+{
+	if (glw.m != NULL)
+	{
+		glPushMatrix();
+		glMultMatrix(glw.m->Tr);
+		if( (colm == vcg::GLW::CMPerFace)  && (!vcg::tri::HasPerFaceColor(*glw.m)) ) 
+			colm=vcg::GLW::CMNone;
+		if( (tm == vcg::GLW::TMPerWedge )&& (!vcg::tri::HasPerWedgeTexCoord(*glw.m)) ) 
+			tm=vcg::GLW::TMNone;
+		if( (tm == vcg::GLW::TMPerWedgeMulti )&& (!vcg::tri::HasPerWedgeTexCoord(*glw.m)) ) 
+			tm=vcg::GLW::TMNone;
+		glw.Draw(dm,colm,tm);
+		glPopMatrix();
+		return true;
+	}
+	return false;
+}
+
+bool MeshLabRenderMesh::renderSelectedFace()
+{
+	if (glw.m != NULL)
+	{
+		glPushAttrib(GL_ALL_ATTRIB_BITS);
+		glEnable(GL_POLYGON_OFFSET_FILL);
+		glDisable(GL_LIGHTING);
+		glDisable(GL_TEXTURE_2D);
+		glEnable(GL_BLEND);
+		glDepthMask(GL_FALSE);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
+		glColor4f(1.0f,0.0,0.0,.3f);
+		glPolygonOffset(-1.0, -1);
+		CMeshO::FaceIterator fi;
+		glPushMatrix();
+		glMultMatrix(glw.m->Tr);
+		glBegin(GL_TRIANGLES);
+		glw.m->sfn=0;
+		for(fi=glw.m->face.begin();fi!=glw.m->face.end();++fi)
+		{
+			if(!(*fi).IsD() && (*fi).IsS())
+			{
+				glVertex((*fi).cP(0));
+				glVertex((*fi).cP(1));
+				glVertex((*fi).cP(2));
+				++glw.m->sfn;
+			}
+		}
+		glEnd();
+		glPopMatrix();
+		glPopAttrib();
+		return true;
+	}
+	return false;
+}
+
+bool MeshLabRenderMesh::renderSelectedVert()
+{
+	if (glw.m != NULL)
+	{
+		glPushAttrib(GL_ALL_ATTRIB_BITS);
+		glDisable(GL_LIGHTING);
+		glDisable(GL_TEXTURE_2D);
+		glEnable(GL_BLEND);
+		glDepthMask(GL_FALSE);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
+		glColor4f(1.0f,0.0,0.0,.3f);
+		glDepthRange(0.00,0.999);
+		glPointSize(3.0);
+		glPushMatrix();
+		glMultMatrix(glw.m->Tr);
+		glBegin(GL_POINTS);
+		glw.m->svn=0;
+		CMeshO::VertexIterator vi;
+		for(vi=glw.m->vert.begin();vi!=glw.m->vert.end();++vi)
+		{
+			if(!(*vi).IsD() && (*vi).IsS())
+			{
+				glVertex((*vi).cP());
+				++glw.m->svn;
+			}
+		}
+		glEnd();
+		glPopMatrix();
+		glPopAttrib();
+		return true;
+	}
+	return false;
+}
+
+MeshLabRenderMesh::~MeshLabRenderMesh()
+{
+	glw.m = NULL;
+	cm.Clear();
+	CMeshO::VertContainer tempVert;
+	CMeshO::FaceContainer tempFace;
+	cm.vert.swap(tempVert);
+	cm.face.swap(tempFace);
+}
+
+MeshLabRenderState::MeshLabRenderState()
+:_meshmap(),_meshmut(QReadWriteLock::Recursive),_rastermut(QReadWriteLock::Recursive)
+{
+
+}
+
+MeshLabRenderState::~MeshLabRenderState()
+{
+	clearState();
+}
+
+bool MeshLabRenderState::update(const int id,CMeshO& mm,const int updateattributesmask)
+{
+	if (updateattributesmask == MeshModel::MM_NONE)
+		return false;
+	lockRenderState(MESH,WRITE);
+	QMap<int,MeshLabRenderMesh*>::iterator it = _meshmap.find(id);
+	if (it != _meshmap.end())
+	{
+		MeshLabRenderMesh* rm = *it;
+
+		if (!(updateattributesmask & MeshModel::MM_VERTCOLOR) &
+			!(updateattributesmask & MeshModel::MM_VERTCOORD) &
+			!(updateattributesmask & MeshModel::MM_VERTQUALITY) &
+			!(updateattributesmask & MeshModel::MM_VERTNORMAL) &
+			!(updateattributesmask & MeshModel::MM_FACEFLAGSELECT) &
+			!(updateattributesmask & MeshModel::MM_VERTFLAGSELECT) &
+			!(updateattributesmask & MeshModel::MM_TRANSFMATRIX) &
+			!(updateattributesmask & MeshModel::MM_CAMERA))
+		{
+			remove(it);
+			_meshmap[id] = new MeshLabRenderMesh(mm);
+			unlockRenderState(MESH);
+			return true;
+		}
+
+		if (updateattributesmask & MeshModel::MM_VERTCOLOR)
+		{		
+			if(mm.vert.size() != (rm->cm.vert.size())) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			else
+			{
+				CMeshO::VertexIterator rmvi = rm->cm.vert.begin();
+				for(CMeshO::ConstVertexIterator mmvi = mm.vert.begin(); mmvi != mm.vert.end(); ++mmvi, ++rmvi)
+					if(!(*mmvi).IsD()) 
+						(*rmvi).C()=(*mmvi).cC();
+			}
+		}
+
+		if (updateattributesmask & MeshModel::MM_VERTCOORD)
+		{
+			if(mm.vert.size() != (rm->cm.vert.size())) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			else
+			{
+				CMeshO::VertexIterator rmvi = rm->cm.vert.begin();
+				for(CMeshO::ConstVertexIterator mmvi = mm.vert.begin(); mmvi != mm.vert.end(); ++mmvi, ++rmvi)
+					if(!(*mmvi).IsD()) 
+						(*rmvi).P()=(*mmvi).cP();
+			}
+		}
+
+		if (updateattributesmask & MeshModel::MM_VERTQUALITY)
+		{
+			if(mm.vert.size() != (rm->cm.vert.size())) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			else
+			{
+				CMeshO::VertexIterator rmvi = rm->cm.vert.begin();
+				for(CMeshO::ConstVertexIterator mmvi = mm.vert.begin(); mmvi != mm.vert.end(); ++mmvi, ++rmvi)
+					if(!(*mmvi).IsD()) 
+						(*rmvi).Q()=(*mmvi).cQ();
+			}
+		}
+
+		if	(updateattributesmask & MeshModel::MM_VERTNORMAL)
+		{
+			if(mm.vert.size() != (rm->cm.vert.size())) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			else
+			{
+				CMeshO::VertexIterator rmvi = rm->cm.vert.begin();
+				for(CMeshO::ConstVertexIterator mmvi = mm.vert.begin(); mmvi != mm.vert.end(); ++mmvi, ++rmvi)
+					if(!(*mmvi).IsD()) 
+						(*rmvi).N()=(*mmvi).cN();
+			}
+		}
+		
+		if(updateattributesmask & MeshModel::MM_FACEFLAGSELECT)
+		{
+			if(mm.face.size() != rm->cm.face.size()) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			CMeshO::FaceIterator rmfi = rm->cm.face.begin();
+			for(CMeshO::ConstFaceIterator mmfi = mm.face.begin(); mmfi != mm.face.end(); ++mmfi, ++rmfi)
+			{
+				if ((!(*mmfi).IsD()) && ((*mmfi).IsS()))
+					(*rmfi).SetS();
+				else
+					if (!(*mmfi).IsS())
+						(*rmfi).ClearS();
+			}
+		}
+
+		if(updateattributesmask & MeshModel::MM_VERTFLAGSELECT)
+		{
+			if(mm.vert.size() != (rm->cm.vert.size())) 
+			{
+				unlockRenderState(MESH);
+				return false;
+			}
+			else
+			{
+				CMeshO::VertexIterator rmvi = rm->cm.vert.begin();
+				for(CMeshO::ConstVertexIterator mmvi = mm.vert.begin(); mmvi != mm.vert.end(); ++mmvi, ++rmvi)
+				{
+					if ((!(*mmvi).IsD()) && ((*mmvi).IsS()))
+						(*rmvi).SetS();
+					else
+						if (!(*mmvi).IsS())
+							(*rmvi).ClearS();
+				}
+			}
+		}
+
+		if(updateattributesmask & MeshModel::MM_TRANSFMATRIX)
+			rm->cm.Tr = mm.Tr;
+		if(updateattributesmask & MeshModel::MM_CAMERA)
+			rm->cm.shot = mm.shot;
+		unlockRenderState(MESH);
+		return true;
+	}
+	unlockRenderState(MESH);
+	return false;
+}
+
+void MeshLabRenderState::add(const int id,CMeshO& mm )
+{
+	lockRenderState(MESH,WRITE);
+	if (!_meshmap.contains(id))
+	{
+		_meshmap[id] = new MeshLabRenderMesh(mm);
+	}
+	unlockRenderState(MESH);
+}
+
+QMap<int,MeshLabRenderMesh*>::iterator MeshLabRenderState::remove(QMap<int,MeshLabRenderMesh*>::iterator it )
+{	
+	lockRenderState(MESH,WRITE);
+	if (it != _meshmap.end())
+	{
+		MeshLabRenderMesh* tmp = it.value();
+		delete tmp;
+		QMap<int,MeshLabRenderMesh*>::iterator tmpit = _meshmap.erase(it);
+		unlockRenderState(MESH);
+		return tmpit;
+	}
+	unlockRenderState(MESH);
+	return _meshmap.end();
+}
+
+void MeshLabRenderState::clearState()
+{
+	lockRenderState(MESH,WRITE);
+	QMap<int,MeshLabRenderMesh*>::iterator it = _meshmap.begin();
+	while(it != _meshmap.end())
+		it = remove(it);
+	unlockRenderState(MESH);
+	lockRenderState(RASTER,WRITE);
+	QMap<int,MeshLabRenderRaster*>::iterator itr = _rastermap.begin();
+	while(itr != _rastermap.end())
+		itr = remove(itr);
+	unlockRenderState(RASTER);
+}
+
+void MeshLabRenderState::copyBack( const int /*id*/,CMeshO& /*mm*/ ) const
+{
+//	mm.Clear();
+//	mm.vert.swap(CMeshO::VertContainer());
+//	mm.face.swap(CMeshO::FaceContainer());
+//	vcg::tri::Append<CMeshO,CMeshO>::MeshCopy(mm,_rendermap[id]->cm);
+}
+
+void MeshLabRenderState::render( const int id,vcg::GLW::DrawMode dm,vcg::GLW::ColorMode cm,vcg::GLW::TextureMode tm  )
+{
+	lockRenderState(MESH,READ);
+	_meshmap[id]->render(dm,cm,tm);
+	unlockRenderState(MESH);
+}
+
+void MeshLabRenderState::render(vcg::GLW::DrawMode dm,vcg::GLW::ColorMode cm,vcg::GLW::TextureMode tm  )
+{
+	lockRenderState(MESH,READ);
+	for(QMap<int,MeshLabRenderMesh*>::iterator it = _meshmap.begin();it != _meshmap.end();++it)
+		(*it)->render(dm,cm,tm);
+	unlockRenderState(MESH);
+}
+
+bool MeshLabRenderState::isEntityInRenderingState( const int id,const MESHLAB_RENDER_ENTITY ent)
+{
+	bool found = false;
+	switch(ent)
+	{
+		case (MESH):
+		{
+			lockRenderState(MESH,READ);
+			found = _meshmap.contains(id);
+			unlockRenderState(MESH);
+		}
+
+		case (RASTER):
+		{
+			lockRenderState(RASTER,READ);
+			found = _meshmap.contains(id);
+			unlockRenderState(RASTER);
+		}	
+	}
+	return found;
+}
+
+void MeshLabRenderState::add( const int id,const MeshLabRenderRaster& rm )
+{
+	lockRenderState(RASTER,WRITE);
+	if (!_rastermap.contains(id))
+	{
+		_rastermap[id] = new MeshLabRenderRaster(rm);
+	}
+	unlockRenderState(RASTER);
+}
+
+bool MeshLabRenderState::update( const int id,const MeshLabRenderRaster& rm,const int updateattributesmask)
+{
+	if (updateattributesmask & RasterModel::RM_NONE)
+		return false;
+	lockRenderState(RASTER,WRITE);
+	QMap<int,MeshLabRenderRaster*>::iterator it = _rastermap.find(id);
+	if (it != _rastermap.end())
+	{
+		MeshLabRenderRaster* rrst = *it;
+		remove(it);
+		_rastermap[id] = new MeshLabRenderRaster(rm);
+		unlockRenderState(RASTER);
+		return true;
+	}
+	unlockRenderState(RASTER);
+	return false;
+}
+
+QMap<int,MeshLabRenderRaster*>::iterator MeshLabRenderState::remove( QMap<int,MeshLabRenderRaster*>::iterator it )
+{
+	lockRenderState(RASTER,WRITE);
+	if (it != _rastermap.end())
+	{
+		MeshLabRenderRaster* tmp = it.value();
+		delete tmp;
+		QMap<int,MeshLabRenderRaster*>::iterator tmpit = _rastermap.erase(it);
+		unlockRenderState(RASTER);
+		return tmpit;
+	}
+	unlockRenderState(RASTER);
+	return _rastermap.end();
+}
+
+void MeshLabRenderState::lockRenderState( const MESHLAB_RENDER_ENTITY ent,const MESHLAB_RENDER_STATE_ACTION act )
+{
+	switch(ent)
+	{
+		case (MESH):
+		{
+			lockReadOrWrite(_meshmut,act);
+			break;
+		}
+		case (RASTER):
+		{
+			lockReadOrWrite(_rastermut,act);
+			break;
+		}
+	}
+}
+
+void MeshLabRenderState::unlockRenderState( const MESHLAB_RENDER_ENTITY ent )
+{
+	switch(ent)
+	{
+		case (MESH):
+		{
+			_meshmut.unlock();
+			break;
+		}
+		case (RASTER):
+		{
+			_rastermut.unlock();
+			break;
+		}
+	}
+}
+
+void MeshLabRenderState::lockReadOrWrite( QReadWriteLock& mutex,const MESHLAB_RENDER_STATE_ACTION act )
+{
+	switch(act)
+	{
+		case (READ):
+		{
+			mutex.lockForRead();
+			break;
+		}
+		case (WRITE):
+		{
+			mutex.lockForWrite();
+			break;
+		}
+	}
+}
