@@ -44,6 +44,7 @@ LayerDialog::LayerDialog(QWidget *parent )    : QDockWidget(parent)
 	setVisible(false);
 	LayerDialog::ui->setupUi(this);
 	mw=qobject_cast<MainWindow *>(parent);
+	this->layout();
 
 	tagMenu = new QMenu(this);
 	removeTagAct = new QAction(tr("&Remove Tag"),this);
@@ -63,15 +64,16 @@ LayerDialog::LayerDialog(QWidget *parent )    : QDockWidget(parent)
 	// The following connection is used to associate the click with the switch between raster and mesh view. 
   connect(ui->rasterTreeWidget, SIGNAL(itemClicked(QTreeWidgetItem * , int  )) , this,  SLOT(rasterItemClicked(QTreeWidgetItem * , int ) ) );
 
-  connect(ui->addButton, SIGNAL(clicked()), mw, SLOT(importMesh()) );
-	connect(ui->deleteButton, SIGNAL(clicked()), mw, SLOT(delCurrentMesh()) );
+  /*connect(ui->addButton, SIGNAL(clicked()), mw, SLOT(importMesh()) );
+	connect(ui->deleteButton, SIGNAL(clicked()), mw, SLOT(delCurrentMesh()) );*/
 
 	this->setContextMenuPolicy(Qt::CustomContextMenu);
 	ui->meshTreeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
+	ui->rasterTreeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
 
-	connect(ui->meshTreeWidget, SIGNAL(customContextMenuRequested(const QPoint&)),
-            this, SLOT(showContextMenu(const QPoint&)));
-	connect(ui->menuButton, SIGNAL(clicked()), this, SLOT(showLayerMenu()));
+	connect(ui->meshTreeWidget, SIGNAL(customContextMenuRequested(const QPoint&)),this, SLOT(showContextMenu(const QPoint&)));
+	connect(ui->rasterTreeWidget, SIGNAL(customContextMenuRequested(const QPoint&)),this, SLOT(showContextMenu(const QPoint&)));
+	//connect(ui->menuButton, SIGNAL(clicked()), this, SLOT(showLayerMenu()));
 	//connect(mw,SIGNAL(selectedDecoration(GLArea*,QAction*)),this,SLOT(addParamsToDecorationDialog(GLArea*,QAction*)));
 	//connect(mw,SIGNAL(unSelectedDecoration(GLArea*,QAction*)),this,SLOT(removeParamsFromDecorationDialog(GLArea*,QAction*)));
 }
@@ -147,31 +149,24 @@ void LayerDialog::rasterItemClicked (QTreeWidgetItem * item , int col)
 		{
 		case 0 :
 			{
-				//the user has clicked on one of the eyes
-        MeshDocument  *md= mw->meshDoc();
-
-				//Only one raster could be visible
-				foreach(RasterModel *rm, md->rasterList)
-					if(rm->id()!= clickedId)
-						rm->visible=false;
-
 				if(rItem->r->visible){
 					rItem->r->visible = false;
-					mw->GLA()->setIsRaster(false);
 				}
 				else{
 					rItem->r->visible = true;
-					mw->GLA()->setIsRaster(true);
-					mw->GLA()->loadRaster(clickedId);
 				}
-
 				//Update current GLArea visibility 
 				mw->GLA()->updateRasterSetVisibilities( );
 			}
     case 1 :
     case 2 :
     case 3 :
-      mw->meshDoc()->setCurrentRaster(clickedId);
+          if(mw->meshDoc()->rm()->id() != clickedId || mw->GLA()->isRaster() )
+          {
+            mw->meshDoc()->setCurrentRaster(clickedId);
+            if(mw->GLA()->isRaster())
+              mw->GLA()->loadRaster(clickedId);
+          }
       break;
 
 		}
@@ -185,45 +180,68 @@ void LayerDialog::showEvent ( QShowEvent * /* event*/ )
 	updateTable();
 }
 
-void LayerDialog::showLayerMenu()
-{
-	foreach (QWidget *widget, QApplication::topLevelWidgets()) {
-		MainWindow* mainwindow = dynamic_cast<MainWindow*>(widget);
-		if (mainwindow)
-		{
-			mainwindow->layerMenu()->popup(ui->menuButton->mapToGlobal(QPoint(10,10)));
-			return;
-		}
-	}
-}
-
 void LayerDialog::showContextMenu(const QPoint& pos)
 {
-	// switch layer
-	MeshTreeWidgetItem *mItem = dynamic_cast<MeshTreeWidgetItem *>(ui->meshTreeWidget->itemAt(pos.x(),pos.y()));
-	QTreeWidgetItem *tItem = dynamic_cast<QTreeWidgetItem *>(ui->meshTreeWidget->itemAt(pos.x(),pos.y()));
-	if(mItem){ 
-		if (mItem->m)
-      mw->meshDoc()->setCurrentMesh(mItem->m->id());
+	QObject* sigsender = sender();
+	if (sigsender == ui->meshTreeWidget)
+	{
+		MeshTreeWidgetItem   *mItem = dynamic_cast<MeshTreeWidgetItem   *>(ui->meshTreeWidget->itemAt(pos.x(),pos.y()));
+		QTreeWidgetItem     *qtItem = dynamic_cast<QTreeWidgetItem *>     (ui->meshTreeWidget->itemAt(pos.x(),pos.y()));
 
-		foreach (QWidget *widget, QApplication::topLevelWidgets()) {
-			MainWindow* mainwindow = dynamic_cast<MainWindow*>(widget);
-			if (mainwindow)
+		if(mItem)
+		{ 
+			if (mItem->m) mw->meshDoc()->setCurrentMesh(mItem->m->id());
+
+			foreach (QWidget *widget, QApplication::topLevelWidgets()) 
 			{
-				mainwindow->layerMenu()->popup(ui->meshTreeWidget->mapToGlobal(pos));
-				return;
+				MainWindow* mainwindow = dynamic_cast<MainWindow*>(widget);
+				if (mainwindow)
+				{
+					mainwindow->meshLayerMenu()->popup(ui->meshTreeWidget->mapToGlobal(pos));
+					return;
+				}
+			}
+		}
+		else 
+		{
+			if(qtItem)
+			{
+				bool ok;
+				int idToRemove = qtItem->text(2).toInt(&ok);
+				if(ok)
+				{
+					removeTagAct->setData(idToRemove);
+					tagMenu->popup(mapToGlobal(pos));
+				}
 			}
 		}
 	}
-	else if(tItem){
-		bool ok;
-		int idToRemove = tItem->text(2).toInt(&ok);
-		if(ok){
-			removeTagAct->setData(idToRemove);
-			tagMenu->popup(mapToGlobal(pos));
+	// switch layer
+	else
+	{
+		if (sigsender == ui->rasterTreeWidget)
+		{
+			RasterTreeWidgetItem *rItem = dynamic_cast<RasterTreeWidgetItem *>(ui->rasterTreeWidget->itemAt(pos.x(),pos.y()));
+	
+			if (rItem)
+			{ 
+				if (rItem->r)
+				{
+					mw->meshDoc()->setCurrentRaster(rItem->r->id());
+
+					foreach (QWidget *widget, QApplication::topLevelWidgets()) 
+					{
+						MainWindow* mainwindow = dynamic_cast<MainWindow*>(widget);
+						if (mainwindow)
+						{
+							mainwindow->rasterLayerMenu()->popup(ui->rasterTreeWidget->mapToGlobal(pos));
+							return;
+						}
+					}
+				}
+			}
 		}
 	}
-	else return; // user clicked on other info 
 }
 
 void LayerDialog::removeTag()
@@ -245,49 +263,57 @@ void LayerDialog::updateLog(GLLogStream &log)
 	QString preFilter  = "<font face=\"courier\" size=2 color=\"black\">" ;
 
 	QString post   = "</font>";
-
+    QString logText;
 	foreach(logElem, logStringList){
-		QString logText = logElem.second;
-		if(logElem.first == GLLogStream::SYSTEM)  logText = preSystem + logText + post;
-		if(logElem.first == GLLogStream::WARNING) logText = preWarn + logText + post;
-		if(logElem.first == GLLogStream::FILTER)  logText = preFilter + logText + post;
-		ui->logPlainTextEdit->appendHtml(logText);
+		 logText += logElem.second;
+		if(logElem.first == GLLogStream::SYSTEM)  
+			logText = preSystem + logText + post;
+		if(logElem.first == GLLogStream::WARNING) 
+			logText = preWarn + logText + post;
+		if(logElem.first == GLLogStream::FILTER)  
+			logText = preFilter + logText + post;
+		logText += "<BR>";
 	}
+	ui->logPlainTextEdit->appendHtml(logText);
 }
 
 void LayerDialog::updateTable()
 {
 	//TODO:Check if the current viewer is a GLArea
 	if(!isVisible()) return;
-  if(isVisible() && !mw->GLA())
+	if(isVisible() && !mw->GLA())
 	{
 		setVisible(false);
 		//The layer dialog cannot be opened unless a new document is opened
 		return;
 	}
-  MeshDocument *md=mw->meshDoc();
+	MeshDocument *md=mw->meshDoc();
 	this->setWindowTitle(md->docLabel());
 
 	ui->meshTreeWidget->clear();
 	ui->meshTreeWidget->setColumnCount(4);
-  ui->meshTreeWidget->setColumnWidth(0,40);
-  ui->meshTreeWidget->setColumnWidth(1,40);
-  //ui->meshTreeWidget->setColumnWidth(2,40);
+	ui->meshTreeWidget->setColumnWidth(0,40);
+	ui->meshTreeWidget->setColumnWidth(1,40);
+	//ui->meshTreeWidget->setColumnWidth(2,40);
 	ui->meshTreeWidget->header()->hide();
 	foreach(MeshModel* mmd, md->meshList)
 	{
-		//Restore mesh visibility according to the current visibility map
-		//very good to keep viewer state consistent
-		if( mw->GLA()->meshVisibilityMap.contains(mmd->id()))
-      mmd->visible = mw->GLA()->meshVisibilityMap.value(mmd->id());
-    else
-    {
-      mw->GLA()->meshVisibilityMap[mmd->id()]=true;
-      mmd->visible=true;
-    }
+	  //Restore mesh visibility according to the current visibility map
+	  //very good to keep viewer state consistent
+	  if( mw->GLA()->meshVisibilityMap.contains(mmd->id()))
+		mmd->visible = mw->GLA()->meshVisibilityMap.value(mmd->id());
+	  else
+	  {
+		mw->GLA()->meshVisibilityMap[mmd->id()]=true;
+		mmd->visible=true;
+	  }
 
 		MeshTreeWidgetItem *item = new MeshTreeWidgetItem(mmd);
 		if(mmd== mw->GLA()->mm()) {
+			item->setBackground(1,QBrush(Qt::yellow));
+			item->setForeground(1,QBrush(Qt::blue));
+			item->setBackground(2,QBrush(Qt::yellow));
+			item->setForeground(2,QBrush(Qt::blue));
 			item->setBackground(3,QBrush(Qt::yellow));
 			item->setForeground(3,QBrush(Qt::blue));
 		}
@@ -303,11 +329,16 @@ void LayerDialog::updateTable()
 		foreach( TagBase *tag, tags)
 			addTreeWidgetItem(item, tag, *md, mmd);
 	}
-
+		
   for(int i=0; i< ui->meshTreeWidget->columnCount(); i++)
 		ui->meshTreeWidget->resizeColumnToContents(i);
 
 	//RasterTreewWidget
+
+	if (md->rasterList.size() > 0)
+		ui->rasterTreeWidget->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
+	else
+		ui->rasterTreeWidget->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Ignored);
 	ui->rasterTreeWidget->clear();
 	ui->rasterTreeWidget->setColumnCount(4);
 	ui->rasterTreeWidget->setColumnWidth(0,40);
@@ -323,9 +354,13 @@ void LayerDialog::updateTable()
 			rmd->visible =mw->GLA()->rasterVisibilityMap.value(rmd->id());
 
 		RasterTreeWidgetItem *item = new RasterTreeWidgetItem(rmd);
-    if(rmd== mw->meshDoc()->rm()) {
+		if(rmd== mw->meshDoc()->rm()) {
+			item->setBackground(1,QBrush(Qt::yellow));
+			item->setForeground(1,QBrush(Qt::blue));
 			item->setBackground(2,QBrush(Qt::yellow));
 			item->setForeground(2,QBrush(Qt::blue));
+			item->setBackground(3,QBrush(Qt::yellow));
+			item->setForeground(3,QBrush(Qt::blue));
 		}
 		ui->rasterTreeWidget->addTopLevelItem(item);
 
@@ -335,8 +370,6 @@ void LayerDialog::updateTable()
 
 	for(int i=2; i< ui->rasterTreeWidget->columnCount(); i++)
 		ui->rasterTreeWidget->resizeColumnToContents(i);
-
-
 }
 
 //Reconstruct the correct layout of the treewidget after updating the main table. It is necessary to keep the changing 
@@ -370,7 +403,7 @@ void LayerDialog::adaptLayout(QTreeWidgetItem * item)
 }
 
 //Add default annotations for each mesh about faces and vertices number
-void LayerDialog::addDefaultNotes(QTreeWidgetItem * parent, const MeshModel *meshModel)
+void LayerDialog::addDefaultNotes(QTreeWidgetItem * parent, MeshModel *meshModel)
 {
   QTreeWidgetItem *fileItem = new QTreeWidgetItem();
   fileItem->setText(2, QString("File"));
@@ -385,11 +418,59 @@ void LayerDialog::addDefaultNotes(QTreeWidgetItem * parent, const MeshModel *mes
   parent->addChild(faceItem);
   updateColumnNumber(faceItem);
 
+  if(meshModel->cm.en>0){
+    QTreeWidgetItem *edgeItem = new QTreeWidgetItem();
+    edgeItem->setText(2, QString("Edges"));
+    edgeItem->setText(3, QString::number(meshModel->cm.en));
+    parent->addChild(edgeItem);
+    updateColumnNumber(edgeItem);
+  }
   QTreeWidgetItem *vertItem = new QTreeWidgetItem();
   vertItem->setText(2, QString("Vertices"));
   vertItem->setText(3, QString::number(meshModel->cm.vn));
   parent->addChild(vertItem);
   updateColumnNumber(vertItem);
+
+  std::vector<std::string> AttribNameVector;
+  vcg::tri::Allocator<CMeshO>::GetAllPerVertexAttribute< float >(meshModel->cm,AttribNameVector);
+  for(int i = 0; i < (int) AttribNameVector.size(); i++)
+  {
+      QTreeWidgetItem *vertItem = new QTreeWidgetItem();
+      vertItem->setText(2, QString("Vert Attr."));
+      vertItem->setText(3, "float "+QString(AttribNameVector[i].c_str()));
+      parent->addChild(vertItem);
+      updateColumnNumber(vertItem);
+  }
+  AttribNameVector.clear();
+  vcg::tri::Allocator<CMeshO>::GetAllPerVertexAttribute< vcg::Point3f >(meshModel->cm,AttribNameVector);
+  for(int i = 0; i < (int) AttribNameVector.size(); i++)
+  {
+      QTreeWidgetItem *vertItem = new QTreeWidgetItem();
+      vertItem->setText(2, QString("Vert Attr."));
+      vertItem->setText(3, "Point3f "+QString(AttribNameVector[i].c_str()));
+      parent->addChild(vertItem);
+      updateColumnNumber(vertItem);
+  }
+  vcg::tri::Allocator<CMeshO>::GetAllPerFaceAttribute< float >(meshModel->cm,AttribNameVector);
+  for(int i = 0; i < (int) AttribNameVector.size(); i++)
+  {
+      QTreeWidgetItem *vertItem = new QTreeWidgetItem();
+      vertItem->setText(2, QString("Face Attr."));
+      vertItem->setText(3, "float "+QString(AttribNameVector[i].c_str()));
+      parent->addChild(vertItem);
+      updateColumnNumber(vertItem);
+  }
+  AttribNameVector.clear();
+  vcg::tri::Allocator<CMeshO>::GetAllPerFaceAttribute< vcg::Point3f >(meshModel->cm,AttribNameVector);
+  for(int i = 0; i < (int) AttribNameVector.size(); i++)
+  {
+      QTreeWidgetItem *vertItem = new QTreeWidgetItem();
+      vertItem->setText(2, QString("Face Attr."));
+      vertItem->setText(3, "Point3f "+QString(AttribNameVector[i].c_str()));
+      parent->addChild(vertItem);
+      updateColumnNumber(vertItem);
+  }
+
 }
 
 //Add a new item (not a MeshTreeWidgetItem but a tag item) to the treeWidget
@@ -486,32 +567,29 @@ void LayerDialog::updateDecoratorParsView()
 
 MeshTreeWidgetItem::MeshTreeWidgetItem(MeshModel *meshModel)
 {
-	if(meshModel->visible)  setIcon(0,QIcon(":/images/layer_eye_open.png"));
-                     else setIcon(0,QIcon(":/images/layer_eye_close.png"));
-
-//	setIcon(1,QIcon(":/images/layer_edit_unlocked.png"));
-
+  if(meshModel->visible) setIcon(0,QIcon(":/images/layer_eye_open.png"));
+                    else setIcon(0,QIcon(":/images/layer_eye_close.png"));
   setText(1, QString::number(meshModel->id()));
 
   QString meshName = meshModel->label();
   if (meshModel->meshModified())
-	  meshName += " *";
+    meshName += " *";
   setText(2, meshName);
 
-	m=meshModel;
+  this->m=meshModel;
 }
 
 RasterTreeWidgetItem::RasterTreeWidgetItem(RasterModel *rasterModel)
 {
-	if(rasterModel->visible)  setIcon(0,QIcon(":/images/layer_eye_open.png"));
-	else setIcon(0,QIcon(":/images/layer_eye_close.png"));
+  if(rasterModel->visible) setIcon(0,QIcon(":/images/ok.png"));
+                      else setIcon(0,QIcon(":/images/stop.png"));
 
-	setText(1, QString::number(rasterModel->id()));
+  setText(1, QString::number(rasterModel->id()));
 
   QString rasterName = rasterModel->label();
-	setText(2, rasterName);	
+  setText(2, rasterName);
 
-	r=rasterModel;
+  this->r=rasterModel;
 }
 
 DecoratorParamsTreeWidget::DecoratorParamsTreeWidget(QAction* act,MainWindow *mw,QWidget* parent)
@@ -533,24 +611,19 @@ DecoratorParamsTreeWidget::DecoratorParamsTreeWidget(QAction* act,MainWindow *mw
 			}
 			
 			dialoglayout = new QGridLayout(parent);
-
 			frame = new StdParFrame(parent,mw->GLA());
-      frame->loadFrameContent(tmpSet,mw->meshDoc());
+			frame->loadFrameContent(tmpSet,mw->meshDoc());
 			savebut = new QPushButton("Save",parent);
 			resetbut = new QPushButton("Reset",parent);
-			//applybut = new QPushButton("Apply",parent);
 			loadbut = new QPushButton("Load",parent);
-
-			dialoglayout->addWidget(savebut,1,0);
-			dialoglayout->addWidget(resetbut,1,1);
-			dialoglayout->addWidget(loadbut,1,2);
-			//dialoglayout->addWidget(applybut,1,3);
-			dialoglayout->addWidget(frame,0,0,1,3);
+			int countel = frame->stdfieldwidgets.size();
+			dialoglayout->addWidget(savebut,countel,0);
+			dialoglayout->addWidget(resetbut,countel,1);
+			dialoglayout->addWidget(loadbut,countel,2);
+			dialoglayout->addWidget(frame,0,0,countel,3);
 			this->setLayout(dialoglayout);
-			//connect(applybut,SIGNAL(clicked()),this,SLOT(apply()));
-			//for(int ii = 0;ii < frame->stdfieldwidgets.size();++ii)
+			int si = dialoglayout->rowCount();
 			connect(frame,SIGNAL(parameterChanged()),this,SLOT(apply()));
-
 			connect(resetbut,SIGNAL(clicked()),this,SLOT(reset()));
 			connect(savebut,SIGNAL(clicked()),this,SLOT(save()));
 			connect(loadbut,SIGNAL(clicked()),this,SLOT(load()));
