@@ -24,6 +24,7 @@
 
 #include "../common/interfaces.h"
 #include "../common/xmlfilterinfo.h"
+#include "../common/searcher.h"
 
 #include <QtGui>
 #include <QToolBar>
@@ -42,12 +43,15 @@
 QProgressBar *MainWindow::qb;
 
 MainWindow::MainWindow()
+:wama()
 {
 	//workspace = new QWorkspace(this);
 	mdiarea = new QMdiArea(this);
 	layerDialog = new LayerDialog(this);
 	layerDialog->setAllowedAreas (    Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
 	addDockWidget(Qt::RightDockWidgetArea,layerDialog);
+
+
 	//setCentralWidget(workspace);
 	setCentralWidget(mdiarea);
 	windowMapper = new QSignalMapper(this);
@@ -57,22 +61,23 @@ MainWindow::MainWindow()
 	connect(mdiarea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateMenus()));
 	connect(mdiarea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateWindowMenu()));
 	connect(mdiarea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateStdDialog()));
+	connect(mdiarea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateXMLStdDialog()));
 	connect(mdiarea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateDocumentScriptBindings()));
-
 	httpReq=new QHttp(this);
 	//connect(httpReq, SIGNAL(requestFinished(int,bool)), this, SLOT(connectionFinished(int,bool)));
 	connect(httpReq, SIGNAL(done(bool)), this, SLOT(connectionDone(bool)));
 
-        QIcon icon;
-        icon.addPixmap(QPixmap(":images/eye48.png"));
-        setWindowIcon(icon);
+	QIcon icon;
+	icon.addPixmap(QPixmap(":images/eye48.png"));
+	setWindowIcon(icon);
 
-        PM.loadPlugins(defaultGlobalParams);
+	PM.loadPlugins(defaultGlobalParams);
 	// Now load from the registry the settings and  merge the hardwired values got from the PM.loadPlugins with the ones found in the registry.
 	loadMeshLabSettings();
 	createActions();
 	createToolBars();
 	createMenus();
+	initSearchEngine();
 	stddialog = 0;
 	xmldialog = 0;
 	setAcceptDrops(true);
@@ -85,8 +90,15 @@ MainWindow::MainWindow()
 	qb->setMinimum(0);
 	//qb->reset();
 	statusBar()->addPermanentWidget(qb,0);
-	updateMenus();
-  newProject();
+	//updateMenus();
+	newProject();
+	//PM should be initialized before passing it to PluginGeneratorGUI
+	plugingui = new PluginGeneratorGUI(PM,this);
+	plugingui->setAllowedAreas (    Qt::LeftDockWidgetArea | Qt::BottomDockWidgetArea | Qt::RightDockWidgetArea);
+	addDockWidget(Qt::LeftDockWidgetArea,plugingui);
+	connect(plugingui,SIGNAL(scriptCodeExecuted(const QScriptValue&,const int,const QString&)),this,SLOT(scriptCodeExecuted(const QScriptValue&,const int,const QString&)));
+	connect(plugingui,SIGNAL(insertXMLPluginRequested(const QString&,const QString& )),this,SLOT(loadAndInsertXMLPlugin(const QString&,const QString&)));
+	connect(plugingui,SIGNAL(historyRequest()),this,SLOT(sendHistory()));
 	//QWidget* wid = reinterpret_cast<QWidget*>(ar->parent());
 	//wid->showMaximized();
 	//ar->update();
@@ -94,22 +106,27 @@ MainWindow::MainWindow()
 	//qb->setAutoClose(true);
 	//qb->setMinimumDuration(0);
 	//qb->reset();
-
+	connect(GLA(), SIGNAL(updateLayerTable()), layerDialog, SLOT(updateTable()));
 }
 
 void MainWindow::createActions()
 {
+	searchShortCut = new QShortcut(QKeySequence(Qt::CTRL+Qt::Key_F), this);
+	searchShortCut->setContext(Qt::ApplicationShortcut);
 	//////////////Action Menu File ////////////////////////////////////////////////////////////////////////////
-  newProjectAct = new QAction(QIcon(":/images/new_project.png"),tr("New Empty Project..."), this);
-  newProjectAct->setShortcutContext(Qt::ApplicationShortcut);
-  newProjectAct->setShortcut(Qt::CTRL+Qt::Key_N);
-  connect(newProjectAct, SIGNAL(triggered()), this, SLOT(newProject()));
+	newProjectAct = new QAction(QIcon(":/images/new_project.png"),tr("New Empty Project..."), this);
+	newProjectAct->setShortcutContext(Qt::ApplicationShortcut);
+	newProjectAct->setShortcut(Qt::CTRL+Qt::Key_N);
+	connect(newProjectAct, SIGNAL(triggered()), this, SLOT(newProject()));
 
-  openProjectAct = new QAction(QIcon(":/images/open_project.png"),tr("&Open project..."), this);
+	openProjectAct = new QAction(QIcon(":/images/open_project.png"),tr("&Open project..."), this);
 	openProjectAct->setShortcutContext(Qt::ApplicationShortcut);
 	openProjectAct->setShortcut(Qt::CTRL+Qt::Key_O);
 	connect(openProjectAct, SIGNAL(triggered()), this, SLOT(openProject()));
 
+	appendProjectAct = new QAction(tr("Append project to current..."), this);
+	connect(appendProjectAct, SIGNAL(triggered()), this, SLOT(appendProject()));
+
 	saveProjectAct = new QAction(QIcon(":/images/save.png"),tr("&Save Project"), this);
 	saveProjectAct->setShortcutContext(Qt::ApplicationShortcut);
 	saveProjectAct->setShortcut(Qt::CTRL+Qt::Key_S);
@@ -123,26 +140,31 @@ void MainWindow::createActions()
 	//closeAct->setShortcut(Qt::CTRL+Qt::Key_C);
 	connect(closeProjectAct, SIGNAL(triggered()),mdiarea, SLOT(closeActiveSubWindow()));
 	//connect(closeProjectAct, SIGNAL(triggered()),this, SLOT(closeProjectWindow()));
-  importMeshAct = new QAction(QIcon(":/images/import_mesh.png"),tr("&Import Mesh..."), this);
-  importMeshAct->setShortcutContext(Qt::ApplicationShortcut);
-  importMeshAct->setShortcut(Qt::CTRL+Qt::Key_I);
-  connect(importMeshAct, SIGNAL(triggered()), this, SLOT(importMesh()));
+	importMeshAct = new QAction(QIcon(":/images/import_mesh.png"),tr("&Import Mesh..."), this);
+	importMeshAct->setShortcutContext(Qt::ApplicationShortcut);
+	importMeshAct->setShortcut(Qt::CTRL+Qt::Key_I);
+	connect(importMeshAct, SIGNAL(triggered()), this, SLOT(importMesh()));
 
-  exportMeshAct = new QAction(QIcon(":/images/save.png"),tr("&Export Mesh..."), this);
-  exportMeshAct->setShortcutContext(Qt::ApplicationShortcut);
-  exportMeshAct->setShortcut(Qt::CTRL+Qt::Key_E);
-  connect(exportMeshAct, SIGNAL(triggered()), this, SLOT(save()));
+	exportMeshAct = new QAction(QIcon(":/images/save.png"),tr("&Export Mesh..."), this);
+	exportMeshAct->setShortcutContext(Qt::ApplicationShortcut);
+	exportMeshAct->setShortcut(Qt::CTRL+Qt::Key_E);
+	connect(exportMeshAct, SIGNAL(triggered()), this, SLOT(save()));
 
-  exportMeshAsAct = new QAction(QIcon(":/images/save.png"),tr("&Export Mesh As..."), this);
-  connect(exportMeshAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
+	exportMeshAsAct = new QAction(QIcon(":/images/save.png"),tr("&Export Mesh As..."), this);
+	connect(exportMeshAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
 
   reloadMeshAct = new QAction(QIcon(":/images/reload.png"),tr("&Reload"), this);
   reloadMeshAct->setShortcutContext(Qt::ApplicationShortcut);
   reloadMeshAct->setShortcut(Qt::CTRL+Qt::Key_R);
   connect(reloadMeshAct, SIGNAL(triggered()), this, SLOT(reload()));
 
-  importRasterAct = new QAction(QIcon(":/images/open.png"),tr("Import Raster..."), this);
-  connect(importRasterAct, SIGNAL(triggered()), this, SLOT(importRaster()));
+  reloadAllMeshAct = new QAction(tr("&Reload All"), this);
+  reloadAllMeshAct->setShortcutContext(Qt::ApplicationShortcut);
+  reloadAllMeshAct->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_R);
+  connect(reloadAllMeshAct, SIGNAL(triggered()), this, SLOT(reloadAllMesh()));
+
+	importRasterAct = new QAction(QIcon(":/images/open.png"),tr("Import Raster..."), this);
+	connect(importRasterAct, SIGNAL(triggered()), this, SLOT(importRaster()));
 
 	saveSnapshotAct = new QAction(QIcon(":/images/snapshot.png"),tr("Save snapsho&t"), this);
 	connect(saveSnapshotAct, SIGNAL(triggered()), this, SLOT(saveSnapshot()));
@@ -156,10 +178,10 @@ void MainWindow::createActions()
 		recentFileActs[i] = new QAction(this);
 		recentFileActs[i]->setVisible(true);
 		recentFileActs[i]->setEnabled(false);
-    recentFileActs[i]->setShortcut(QKeySequence(Qt::ALT + Qt::Key_1+i));
+		recentFileActs[i]->setShortcut(QKeySequence(Qt::ALT + Qt::Key_1+i));
 		connect(recentProjActs[i],SIGNAL(triggered()),this,SLOT(openRecentProj()));
-    connect(recentFileActs[i], SIGNAL(triggered()),this, SLOT(openRecentMesh()));
-		
+		connect(recentFileActs[i], SIGNAL(triggered()),this, SLOT(openRecentMesh()));
+
 	}
 
 	exitAct = new QAction(tr("E&xit"), this);
@@ -215,7 +237,7 @@ void MainWindow::createActions()
 	setFancyLightingAct   = new QAction(tr("&Fancy Lighting"),this);
 	setFancyLightingAct->setCheckable(true);
 	setFancyLightingAct->setShortcutContext(Qt::ApplicationShortcut);
-	setFancyLightingAct->setShortcut(Qt::CTRL+Qt::Key_F);
+	setFancyLightingAct->setShortcut(Qt::CTRL+Qt::Key_Y);
 	connect(setFancyLightingAct, SIGNAL(triggered()), this, SLOT(setFancyLighting()));
 
 	backFaceCullAct 	  = new QAction(tr("BackFace &Culling"),this);
@@ -275,6 +297,12 @@ void MainWindow::createActions()
 	connect(showLayerDlgAct, SIGNAL(triggered()), this, SLOT(showLayerDlg()));
 
 
+	showRasterAct =  new QAction (QIcon(":/images/view_raster.png"),tr("Show Current Raster Mode"), this);
+	showRasterAct->setCheckable(true);
+	showRasterAct->setChecked(true);
+	connect(showRasterAct, SIGNAL(triggered()), this, SLOT(showRaster()));
+
+
 	//////////////Action Menu EDIT /////////////////////////////////////////////////////////////////////////
 	suspendEditModeAct = new QAction (QIcon(":/images/no_edit.png"),tr("Not editing"), this);
 	suspendEditModeAct->setShortcut(Qt::Key_Escape);
@@ -318,6 +346,14 @@ void MainWindow::createActions()
 	viewFrontAct	  = new QAction(tr("Front"),viewFromGroupAct);
 	viewBackAct	    = new QAction(tr("Back"),viewFromGroupAct);
 
+	// keyboard shortcuts for canonical viewdirections, blender style
+	viewFrontAct->setShortcut(Qt::Key_End);
+	viewBackAct->setShortcut(Qt::CTRL + Qt::Key_End);
+	viewRightAct->setShortcut(Qt::Key_PageDown);
+	viewLeftAct->setShortcut(Qt::CTRL + Qt::Key_PageDown);
+	viewTopAct->setShortcut(Qt::Key_Home);
+	viewBottomAct->setShortcut(Qt::CTRL + Qt::Key_Home);
+
 	connect(viewFromGroupAct, SIGNAL(triggered(QAction *)), this, SLOT(viewFrom(QAction *)));
 
 	viewFromMeshAct = new QAction (tr("View from Mesh Camera"), this);
@@ -346,10 +382,6 @@ void MainWindow::createActions()
 	showFilterScriptAct->setEnabled(false);
 	connect(showFilterScriptAct, SIGNAL(triggered()), this, SLOT(showFilterScript()));
 
-	showScriptEditAct = new QAction(tr("Script Editor"),this);
-	showScriptEditAct->setEnabled(false);
-	connect(showScriptEditAct, SIGNAL(triggered()), this, SLOT(showScriptEditor()));
-
 	//////////////Action Menu Preferences /////////////////////////////////////////////////////////////////////
 	setCustomizeAct	  = new QAction(tr("&Options..."),this);
 	connect(setCustomizeAct, SIGNAL(triggered()), this, SLOT(setCustomize()));
@@ -390,19 +422,27 @@ void MainWindow::createActions()
 	connect(splitGroupAct, SIGNAL(triggered(QAction *)), this, SLOT(splitFromHandle(QAction *)));
 
 	connect(unsplitGroupAct, SIGNAL(triggered(QAction *)), this, SLOT(unsplitFromHandle(QAction *)));
+
+	//TOOL MENU
+	showFilterEditAct = new QAction(tr("XML Plugin Editor GUI"),this);
+	showFilterEditAct->setEnabled(true);
+	connect(showFilterEditAct, SIGNAL(triggered()), this, SLOT(showXMLPluginEditorGui()));
+
 }
 
 void MainWindow::createToolBars()
 {
 	mainToolBar = addToolBar(tr("Standard"));
 	mainToolBar->setIconSize(QSize(32,32));
-  mainToolBar->addAction(this->newProjectAct);
-  mainToolBar->addAction(this->openProjectAct);
-  mainToolBar->addAction(importMeshAct);
-  mainToolBar->addAction(reloadMeshAct);
-  mainToolBar->addAction(exportMeshAct);
+	mainToolBar->addAction(this->newProjectAct);
+	mainToolBar->addAction(this->openProjectAct);
+	mainToolBar->addAction(importMeshAct);
+	mainToolBar->addAction(reloadMeshAct);
+//  mainToolBar->addAction(reloadAllMeshAct);
+	mainToolBar->addAction(exportMeshAct);
 	mainToolBar->addAction(saveSnapshotAct);
 	mainToolBar->addAction(showLayerDlgAct);
+	mainToolBar->addAction(showRasterAct);
 
 	renderToolBar = addToolBar(tr("Render"));
 	//renderToolBar->setIconSize(QSize(32,32));
@@ -417,7 +457,8 @@ void MainWindow::createToolBars()
 	editToolBar->addSeparator();
 
 	filterToolBar = addToolBar(tr("Action"));
-
+	filterToolBar->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
+	filterToolBar->setIconSize(QSize(32,32));
 	foreach(MeshEditInterfaceFactory *iEditFactory,PM.meshEditFactoryPlugins())
 	{		
 		foreach(QAction* editAction, iEditFactory->actions())
@@ -428,6 +469,22 @@ void MainWindow::createToolBars()
 			} else qDebug() << "action was null";
 		}
 	}
+	
+	QWidget *spacerWidget = new QWidget();
+	spacerWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+	//spacerWidget->setVisible(true);
+	searchToolBar = addToolBar(tr("Search"));
+	searchToolBar->addWidget(spacerWidget);
+	searchToolBar->setMovable(false);
+	searchToolBar->setFloatable(false);
+	searchToolBar->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Fixed);
+	searchButton = new MyToolButton(this);
+	searchButton->setPopupMode(QToolButton::InstantPopup);
+	searchButton->setIcon(QIcon(":/images/search.png"));
+	searchToolBar->addWidget(searchButton);
+	SearchMenu* ser = new SearchMenu(wama,20,searchButton);
+	searchButton->setMenu(ser);
+	connect(searchShortCut,SIGNAL(activated()),searchButton,SLOT(showMenu()));
 }
 
 
@@ -435,25 +492,27 @@ void MainWindow::createMenus()
 {
 	//////////////////// Menu File ////////////////////////////////////////////////////////////////////////////
 	fileMenu = menuBar()->addMenu(tr("&File"));
-  fileMenu->addAction(newProjectAct);
+	fileMenu->addAction(newProjectAct);
 	fileMenu->addAction(openProjectAct);
+	fileMenu->addAction(appendProjectAct);
 	fileMenu->addAction(saveProjectAct);
 	fileMenu->addAction(closeProjectAct);
 	fileMenu->addSeparator();
 
-  fileMenu->addAction(importMeshAct);
-  fileMenu->addAction(exportMeshAct);
-  fileMenu->addAction(exportMeshAsAct);
-  fileMenu->addAction(reloadMeshAct);
+	fileMenu->addAction(importMeshAct);
+	fileMenu->addAction(exportMeshAct);
+	fileMenu->addAction(exportMeshAsAct);
+	fileMenu->addAction(reloadMeshAct);
+  fileMenu->addAction(reloadAllMeshAct);
+	fileMenu->addSeparator();
+	fileMenu->addAction(importRasterAct);
 	fileMenu->addSeparator();
-  fileMenu->addAction(importRasterAct);
-  fileMenu->addSeparator();
 
 	fileMenu->addAction(saveSnapshotAct);
 	separatorAct = fileMenu->addSeparator();
 	recentProjMenu = fileMenu->addMenu(tr("Recent Projects"));
 	recentFileMenu = fileMenu->addMenu(tr("Recent Files"));
-	
+
 
 	for (int i = 0; i < MAXRECENTFILES; ++i) 
 	{
@@ -470,10 +529,13 @@ void MainWindow::createMenus()
 
 	//////////////////// Menu Filter //////////////////////////////////////////////////////////////////////////
 	filterMenu = menuBar()->addMenu(tr("Fi&lters"));
-	filterMenu->addAction(lastFilterAct);
-	filterMenu->addAction(showFilterScriptAct);
-	filterMenu->addAction(showScriptEditAct);
-	filterMenu->addSeparator();
+	fillFilterMenu();
+	//filterMenu = menuBar()->addMenu(tr("Fi&lters"));
+	//filterMenu->addAction(lastFilterAct);
+	//filterMenu->addAction(showFilterScriptAct);
+	//filterMenu->addAction(showScriptEditAct);
+	//filterMenu->addAction(showFilterEditAct);
+	//filterMenu->addSeparator();
 
 
 	//////////////////// Menu Render //////////////////////////////////////////////////////////////////////////
@@ -526,6 +588,7 @@ void MainWindow::createMenus()
 	viewMenu		= menuBar()->addMenu(tr("&View"));
 	viewMenu->addAction(fullScreenAct);
 	viewMenu->addAction(showLayerDlgAct);
+	viewMenu->addAction(showRasterAct);
 
 	trackBallMenu = viewMenu->addMenu(tr("&Trackball"));
 	trackBallMenu->addAction(showTrackBallAct);
@@ -546,7 +609,10 @@ void MainWindow::createMenus()
 
 	//////////////////// Menu Preferences /////////////////////////////////////////////////////////////////////
 	preferencesMenu=menuBar()->addMenu(tr("&Tools"));
+	//preferencesMenu->addAction(showFilterEditAct);
+	//preferencesMenu->addSeparator();
 	preferencesMenu->addAction(setCustomizeAct);
+	
 
 	//////////////////// Menu Help ////////////////////////////////////////////////////////////////
 	helpMenu = menuBar()->addMenu(tr("&Help"));
@@ -557,7 +623,7 @@ void MainWindow::createMenus()
 	helpMenu->addAction(submitBugAct);
 	helpMenu->addAction(checkUpdatesAct);
 
-	fillFilterMenu();
+	//fillFilterMenu();
 	fillEditMenu();
 	fillRenderMenu();
 	fillDecorateMenu();
@@ -568,33 +634,87 @@ void MainWindow::createMenus()
 	unSplitMenu = handleMenu->addMenu("&Close");
 }
 
+void MainWindow::initSearchEngine()
+{
+	for(QMap<QString,QAction*>::iterator it = PM.actionFilterMap.begin();it != PM.actionFilterMap.end();++it)
+		initItemForSearching(it.value());
+
+	for(QMap<QString,MeshLabXMLFilterContainer>::iterator it = PM.stringXMLFilterMap.begin();it != PM.stringXMLFilterMap.end();++it)
+		initItemForSearching(it.value().act);
+
+	initMenuForSearching(editMenu);
+	initMenuForSearching(renderMenu);
+}
+
+void MainWindow::initMenuForSearching(QMenu* menu)
+{
+	if (menu == NULL)
+		return;
+	const QList<QAction*>& acts = menu->actions();
+	foreach(QAction* act,acts)
+	{
+		QMenu* submenu = act->menu();
+		if (!act->isSeparator() && (submenu == NULL))
+			initItemForSearching(act);
+		else if (!act->isSeparator())
+				initMenuForSearching(submenu);
+	}
+}
+
+void MainWindow::initItemForSearching(QAction* act)
+{
+	QString tx = act->text() + " " + act->toolTip();
+	wama.addWordsPerAction(*act,tx);
+}
+
 void MainWindow::fillFilterMenu()
 {
+	filterMenu->clear();
+	filterMenu->addAction(lastFilterAct);
+	filterMenu->addAction(showFilterScriptAct);
+	filterMenu->addSeparator();
+	//filterMenu->addMenu(new SearcherMenu(this,filterMenu));
+	//filterMenu->addSeparator();
 	// Connects the events of the actions within colorize to the method which shows their tooltip
 	filterMenuSelect = filterMenu->addMenu(tr("Selection"));
 	filterMenuClean  = filterMenu->addMenu(tr("Cleaning and Repairing"));
 	filterMenuCreate = filterMenu->addMenu(tr("Create New Mesh Layer"));
-	filterMenuRemeshing = filterMenu->addMenu(tr("Remeshing, simplification and reconstruction"));
+	filterMenuRemeshing = filterMenu->addMenu(tr("Remeshing, Simplification and Reconstruction"));
 	filterMenuPolygonal = filterMenu->addMenu(tr("Polygonal and Quad Mesh"));
 	filterMenuColorize = filterMenu->addMenu(tr("Color Creation and Processing"));
 	filterMenuSmoothing = filterMenu->addMenu(tr("Smoothing, Fairing and Deformation"));
-	filterMenuQuality = filterMenu->addMenu(tr("Quality Measure and computations"));
+	filterMenuQuality = filterMenu->addMenu(tr("Quality Measure and Computations"));
 	filterMenuNormal = filterMenu->addMenu(tr("Normals, Curvatures and Orientation"));
-	filterMenuLayer = filterMenu->addMenu(tr("Layer and Attribute Management"));
+	filterMenuMeshLayer   = filterMenu->addMenu(tr("Mesh Layer"));
+	filterMenuRasterLayer = filterMenu->addMenu(tr("Raster Layer"));
 	filterMenuRangeMap = filterMenu->addMenu(tr("Range Map"));
 	filterMenuPointSet = filterMenu->addMenu(tr("Point Set"));
 	filterMenuSampling = filterMenu->addMenu(tr("Sampling"));
 	filterMenuTexture = filterMenu->addMenu(tr("Texture"));
 	filterMenuCamera = filterMenu->addMenu(tr("Camera"));
 
-	connect(filterMenu, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuSelect, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuClean, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuCreate, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuRemeshing, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuPolygonal, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuColorize, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuQuality, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuNormal, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuMeshLayer,   SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuRasterLayer, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuRangeMap, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuPointSet, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuSampling, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuTexture, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
+	connect(filterMenuCamera, SIGNAL(hovered(QAction*)), this, SLOT(showTooltip(QAction*)) );
 
 	QMap<QString,MeshFilterInterface *>::iterator msi;
 	for(msi =  PM.stringFilterMap.begin(); msi != PM.stringFilterMap.end();++msi)
 	{
 		MeshFilterInterface * iFilter= msi.value();
 		QAction *filterAction = iFilter->AC((msi.key()));
-			filterAction->setToolTip(iFilter->filterInfo(filterAction));
+		filterAction->setToolTip(iFilter->filterInfo(filterAction));
 		connect(filterAction,SIGNAL(triggered()),this,SLOT(startFilter()));
 
 		int filterClass = iFilter->getClass(filterAction);
@@ -606,15 +726,16 @@ void MainWindow::fillFilterMenu()
 		if( filterClass & MeshFilterInterface::Smoothing )      filterMenuSmoothing->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::Normal )         filterMenuNormal->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::Quality )        filterMenuQuality->addAction(filterAction);
-		if( filterClass & MeshFilterInterface::Measure  )	      filterMenuQuality->addAction(filterAction);
-		if( filterClass & MeshFilterInterface::Layer )          filterMenuLayer->addAction(filterAction);
+		if( filterClass & MeshFilterInterface::Measure  )	    filterMenuQuality->addAction(filterAction);
+		if( filterClass & MeshFilterInterface::Layer )          filterMenuMeshLayer->addAction(filterAction);
+		if( filterClass & MeshFilterInterface::RasterLayer )    filterMenuRasterLayer->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::MeshCreation )   filterMenuCreate->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::RangeMap )       filterMenuRangeMap->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::PointSet )       filterMenuPointSet->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::Sampling )       filterMenuSampling->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::Texture)         filterMenuTexture->addAction(filterAction);
 		if( filterClass & MeshFilterInterface::Polygonal)       filterMenuPolygonal->addAction(filterAction);
-    if( filterClass & MeshFilterInterface::Camera)          filterMenuCamera->addAction(filterAction);
+		if( filterClass & MeshFilterInterface::Camera)          filterMenuCamera->addAction(filterAction);
 		//  MeshFilterInterface::Generic :
 		if(filterClass == 0)                                    filterMenu->addAction(filterAction);
 		if(!filterAction->icon().isNull())                      filterToolBar->addAction(filterAction);
@@ -623,9 +744,9 @@ void MainWindow::fillFilterMenu()
 	QMap<QString,MeshLabXMLFilterContainer>::iterator xmlit;
 	for(xmlit =  PM.stringXMLFilterMap.begin(); xmlit != PM.stringXMLFilterMap.end();++xmlit)
 	{
-    //MeshLabFilterInterface * iFilter= xmlit.value().filterInterface;
+		//MeshLabFilterInterface * iFilter= xmlit.value().filterInterface;
 		QAction *filterAction = xmlit.value().act;
-		XMLFilterInfo* info = xmlit.value().xmlInfo;
+		MLXMLPluginInfo* info = xmlit.value().xmlInfo;
 		QString filterName = xmlit.key();
 		try
 		{
@@ -646,15 +767,16 @@ void MainWindow::fillFilterMenu()
 				if( nameClass == QString("Normal")) filterMenuNormal->addAction(filterAction);
 				if( nameClass == QString("Quality")) filterMenuQuality->addAction(filterAction);
 				if( nameClass == QString("Measure")) filterMenuQuality->addAction(filterAction);
-				if( nameClass == QString("Layer")) filterMenuLayer->addAction(filterAction);
+				if( nameClass == QString("Layer")) filterMenuMeshLayer->addAction(filterAction);
+				if( nameClass == QString("RasterLayer")) filterMenuRasterLayer->addAction(filterAction);
 				if( nameClass == QString("MeshCreation")) filterMenuCreate->addAction(filterAction);
 				if( nameClass == QString("RangeMap")) filterMenuRangeMap->addAction(filterAction);
 				if( nameClass == QString("PointSet")) filterMenuPointSet->addAction(filterAction);
 				if( nameClass == QString("Sampling")) filterMenuSampling->addAction(filterAction);
 				if( nameClass == QString("Texture")) filterMenuTexture->addAction(filterAction);
 				if( nameClass == QString("Polygonal")) filterMenuPolygonal->addAction(filterAction);
-        if( nameClass == QString("Camera")) filterMenuCamera->addAction(filterAction);
-			  //  //  MeshFilterInterface::Generic :
+				if( nameClass == QString("Camera")) filterMenuCamera->addAction(filterAction);
+				//  //  MeshFilterInterface::Generic :
 				if(	nameClass == QString("Generic")) filterMenu->addAction(filterAction);
 				if(!filterAction->icon().isNull()) filterToolBar->addAction(filterAction);
 			}
@@ -673,7 +795,7 @@ void MainWindow::fillDecorateMenu()
 		foreach(QAction *decorateAction, iDecorate->actions())
 		{
 			connect(decorateAction,SIGNAL(triggered()),this,SLOT(applyDecorateMode()));
-			decorateAction->setToolTip(iDecorate->filterInfo(decorateAction));
+			decorateAction->setToolTip(iDecorate->decorationInfo(decorateAction));
 			renderMenu->addAction(decorateAction);
 		}
 	}
@@ -681,6 +803,10 @@ void MainWindow::fillDecorateMenu()
 
 void MainWindow::fillRenderMenu()
 {
+	QAction * qaNone = new QAction("None", this); 
+	qaNone->setCheckable(false);
+	shadersMenu->addAction(qaNone);
+	connect(qaNone,SIGNAL(triggered()),this,SLOT(applyRenderMode()));
 	foreach(MeshRenderInterface *iRender,PM.meshRenderPlugins())
 	{
 		addToMenu(iRender->actions(), shadersMenu, SLOT(applyRenderMode()));
@@ -689,7 +815,7 @@ void MainWindow::fillRenderMenu()
 
 void MainWindow::fillEditMenu()
 {
-  foreach(MeshEditInterfaceFactory *iEditFactory, PM.meshEditFactoryPlugins())
+	foreach(MeshEditInterfaceFactory *iEditFactory, PM.meshEditFactoryPlugins())
 	{		
 		foreach(QAction* editAction, iEditFactory->actions())
 		{
@@ -702,63 +828,63 @@ void MainWindow::fillEditMenu()
 
 void MainWindow::loadMeshLabSettings()
 {
-  // I have already loaded the plugins so the default parameters for the settings
-  // of the plugins are already in the <defaultGlobalParams> .
-  // we just miss the globals default of meshlab itself
-  GLArea::initGlobalParameterSet(& defaultGlobalParams);
+	// I have already loaded the plugins so the default parameters for the settings
+	// of the plugins are already in the <defaultGlobalParams> .
+	// we just miss the globals default of meshlab itself
+	GLArea::initGlobalParameterSet(& defaultGlobalParams);
 
-  QSettings settings;
-  QStringList klist = settings.allKeys();
+	QSettings settings;
+	QStringList klist = settings.allKeys();
 
-  // 1) load saved values into the <currentGlobalParams>
-  for(int ii = 0;ii < klist.size();++ii)
-  {
-    QDomDocument doc;
-    doc.setContent(settings.value(klist.at(ii)).toString());
+	// 1) load saved values into the <currentGlobalParams>
+	for(int ii = 0;ii < klist.size();++ii)
+	{
+		QDomDocument doc;
+		doc.setContent(settings.value(klist.at(ii)).toString());
 
-    QString st = settings.value(klist.at(ii)).toString();
-    QDomElement docElem = doc.firstChild().toElement();
+		QString st = settings.value(klist.at(ii)).toString();
+		QDomElement docElem = doc.firstChild().toElement();
 
-    RichParameter* rpar = NULL;
-    if(!docElem.isNull())
-    {
-      bool ret = RichParameterFactory::create(docElem,&rpar);
-      if (!ret)
-      {
-        //  qDebug("Warning Ignored parameter '%s' = '%s'. Malformed.", qPrintable(docElem.attribute("name")),qPrintable(docElem.attribute("value")));
-        continue;
-      }
-      if (!defaultGlobalParams.hasParameter(rpar->name))
-      {
-        //  qDebug("Warning Ignored parameter %s. In the saved parameters there are ones that are not in the HardWired ones. "
-        //         "It happens if you are running MeshLab with only a subset of the plugins. ",qPrintable(rpar->name));
-      }
-      else currentGlobalParams.addParam(rpar);
-    }
-  }
+		RichParameter* rpar = NULL;
+		if(!docElem.isNull())
+		{
+			bool ret = RichParameterFactory::create(docElem,&rpar);
+			if (!ret)
+			{
+				//  qDebug("Warning Ignored parameter '%s' = '%s'. Malformed.", qPrintable(docElem.attribute("name")),qPrintable(docElem.attribute("value")));
+				continue;
+			}
+			if (!defaultGlobalParams.hasParameter(rpar->name))
+			{
+				//  qDebug("Warning Ignored parameter %s. In the saved parameters there are ones that are not in the HardWired ones. "
+				//         "It happens if you are running MeshLab with only a subset of the plugins. ",qPrintable(rpar->name));
+			}
+			else currentGlobalParams.addParam(rpar);
+		}
+	}
 
-  // 2) eventually fill missing values with the hardwired defaults
-  for(int ii = 0;ii < defaultGlobalParams.paramList.size();++ii)
-  {
-    //		qDebug("Searching param[%i] %s of the default into the loaded settings. ",ii,qPrintable(defaultGlobalParams.paramList.at(ii)->name));
-    if (!currentGlobalParams.hasParameter(defaultGlobalParams.paramList.at(ii)->name))
-    {
-      qDebug("Warning! a default param was not found in the saved settings. This should happen only on the first run...");
-      RichParameterCopyConstructor v;
-      defaultGlobalParams.paramList.at(ii)->accept(v);
-      currentGlobalParams.paramList.push_back(v.lastCreated);
+	// 2) eventually fill missing values with the hardwired defaults
+	for(int ii = 0;ii < defaultGlobalParams.paramList.size();++ii)
+	{
+		//		qDebug("Searching param[%i] %s of the default into the loaded settings. ",ii,qPrintable(defaultGlobalParams.paramList.at(ii)->name));
+		if (!currentGlobalParams.hasParameter(defaultGlobalParams.paramList.at(ii)->name))
+		{
+			qDebug("Warning! a default param was not found in the saved settings. This should happen only on the first run...");
+			RichParameterCopyConstructor v;
+			defaultGlobalParams.paramList.at(ii)->accept(v);
+			currentGlobalParams.paramList.push_back(v.lastCreated);
 
-      QDomDocument doc("MeshLabSettings");
-      RichParameterXMLVisitor vxml(doc);
-      v.lastCreated->accept(vxml);
-      doc.appendChild(vxml.parElem);
-      QString docstring =  doc.toString();
-      QSettings setting;
-      setting.setValue(v.lastCreated->name,QVariant(docstring));
-    }
-  }
+			QDomDocument doc("MeshLabSettings");
+			RichParameterXMLVisitor vxml(doc);
+			v.lastCreated->accept(vxml);
+			doc.appendChild(vxml.parElem);
+			QString docstring =  doc.toString();
+			QSettings setting;
+			setting.setValue(v.lastCreated->name,QVariant(docstring));
+		}
+	}
 
-  emit dispatchCustomSettings(currentGlobalParams);
+	emit dispatchCustomSettings(currentGlobalParams);
 }
 
 void MainWindow::addToMenu(QList<QAction *> actionList, QMenu *menu, const char *slot)
@@ -799,24 +925,24 @@ void MainWindow::saveRecentFileList(const QString &fileName)
 	int lastComunicatedValue = settings.value("lastComunicatedValue",0).toInt();
 
 	if(loadedMeshCounter-lastComunicatedValue>connectionInterval && !myLocalBuf.isOpen())
-  {
+	{
 #if not defined(__DISABLE_AUTO_STATS__)
 		checkForUpdates(false);
 #endif
-    int congratsMeshCounter = settings.value("congratsMeshCounter",50).toInt();
-    if(loadedMeshCounter > congratsMeshCounter * 2 )
-			{
-        // This preference values store when you did the last request for a mail
-				settings.setValue("congratsMeshCounter",loadedMeshCounter);
+		int congratsMeshCounter = settings.value("congratsMeshCounter",50).toInt();
+		if(loadedMeshCounter > congratsMeshCounter * 2 )
+		{
+			// This preference values store when you did the last request for a mail
+			settings.setValue("congratsMeshCounter",loadedMeshCounter);
 
-				QDialog *congratsDialog = new QDialog();
-				Ui::CongratsDialog temp;
-				temp.setupUi(congratsDialog);
-        temp.buttonBox->addButton("Send Mail", QDialogButtonBox::AcceptRole);
-				congratsDialog->exec();
-				if(congratsDialog->result()==QDialog::Accepted)
-          QDesktopServices::openUrl(QUrl("mailto:p.cignoni@isti.cnr.it?cc=g.ranzuglia@isti.cnr.it&subject=[MeshLab] Reporting Info on MeshLab Usage"));
-			}
+			QDialog *congratsDialog = new QDialog();
+			Ui::CongratsDialog temp;
+			temp.setupUi(congratsDialog);
+			temp.buttonBox->addButton("Send Mail", QDialogButtonBox::AcceptRole);
+			congratsDialog->exec();
+			if(congratsDialog->result()==QDialog::Accepted)
+				QDesktopServices::openUrl(QUrl("mailto:p.cignoni@isti.cnr.it?cc=g.ranzuglia@isti.cnr.it&subject=[MeshLab] Reporting Info on MeshLab Usage"));
+		}
 	}
 }
 
@@ -859,23 +985,23 @@ void MainWindow::checkForUpdates(bool verboseFlag)
 	}
 
 #ifdef _DEBUG_PHP
-    QString BaseCommand("/~cignoni/meshlab_d.php");
+	QString BaseCommand("/~cignoni/meshlab_d.php");
 #else
-    QString BaseCommand("/~cignoni/meshlab.php");
+	QString BaseCommand("/~cignoni/meshlab.php");
 #endif
 
 #ifdef Q_WS_WIN
-    QString OS="Win";
+	QString OS="Win";
 #elif defined( Q_WS_MAC)
-    QString OS="Mac";
+	QString OS="Mac";
 #else
-    QString OS="Lin";
+	QString OS="Lin";
 #endif
-    QString message=BaseCommand+QString("?code=%1&count=%2&scount=%3&totkv=%4&ver=%5&os=%6").arg(UID).arg(loadedMeshCounter).arg(savedMeshCounter).arg(totalKV).arg(appVer()).arg(OS);
-    idHost=httpReq->setHost("vcg.isti.cnr.it"); // id == 1
-    bool ret=myLocalBuf.open(QBuffer::WriteOnly);
-    if(!ret) QMessageBox::information(this,"Meshlab",QString("Failed opening of internal buffer"));
-    idGet=httpReq->get(message,&myLocalBuf);     // id == 2
+	QString message=BaseCommand+QString("?code=%1&count=%2&scount=%3&totkv=%4&ver=%5&os=%6").arg(UID).arg(loadedMeshCounter).arg(savedMeshCounter).arg(totalKV).arg(appVer()).arg(OS);
+	idHost=httpReq->setHost("vcg.isti.cnr.it"); // id == 1
+	bool ret=myLocalBuf.open(QBuffer::WriteOnly);
+	if(!ret) QMessageBox::information(this,"Meshlab",QString("Failed opening of internal buffer"));
+	idGet=httpReq->get(message,&myLocalBuf);     // id == 2
 
 }
 
@@ -900,15 +1026,15 @@ void MainWindow::submitBug()
 	QPushButton *submitBug = mb.addButton("Submit Bug",QMessageBox::AcceptRole);
 	mb.addButton(QMessageBox::Cancel);
 	mb.setText(tr("If Meshlab closed in unexpected way (e.g. it crashed badly) and"
-						 "if you are able to repeat the bug, please consider to submit a report using the SourceForge tracking system.\n"
-						  ) );
+		"if you are able to repeat the bug, please consider to submit a report using the SourceForge tracking system.\n"
+		) );
 	mb.setInformativeText(	tr(
-	         "Hints for a good, useful bug report:\n"
-					 "- Be verbose and descriptive\n"
-					 "- Report meshlab version and OS\n"
-					 "- Describe the sequence of actions that bring you to the crash.\n"
-					 "- Consider submitting the mesh file causing a particular crash.\n"
-					 ) );
+		"Hints for a good, useful bug report:\n"
+		"- Be verbose and descriptive\n"
+		"- Report meshlab version and OS\n"
+		"- Describe the sequence of actions that bring you to the crash.\n"
+		"- Consider submitting the mesh file causing a particular crash.\n"
+		) );
 
 	mb.exec();
 
