@@ -33,7 +33,7 @@ MeshlabStdDialog::MeshlabStdDialog(QWidget *p)
 		clearValues();
 }
 
-StdParFrame::StdParFrame(QWidget *p, QWidget *curr_gla)
+StdParFrame::StdParFrame(QWidget *p, QWidget *curr_gla )
 //:QDialog(p)
 :QFrame(p)
 {
@@ -56,32 +56,33 @@ bool MeshlabStdDialog::showAutoDialog(MeshFilterInterface *mfi, MeshModel *mm, M
 
 		mfi->initParameterSet(action, *mdp, curParSet);
         curmask = mfi->postCondition(action);
-        if(curParSet.isEmpty() && !isDynamic()) return false;
+        if(curParSet.isEmpty() && !isPreviewable()) return false;
 
         createFrame();
 		loadFrameContent(mdp);
-		if(isDynamic())
+		if(isPreviewable())
 		{
 			meshState.create(curmask, curModel);
-			connect(stdParFrame,SIGNAL(dynamicFloatChanged(int)), this, SLOT(applyDynamic()));
 			connect(stdParFrame,SIGNAL(parameterChanged()), this, SLOT(applyDynamic()));
 		}
         connect(curMeshDoc, SIGNAL(currentMeshChanged(int)),this, SLOT(changeCurrentMesh(int)));
+		raise();
+		activateWindow();
         return true;
   }
 
 void MeshlabStdDialog::changeCurrentMesh(int meshInd)
 {
-    if(isDynamic())
+    if(isPreviewable())
     {
         meshState.apply(curModel);
-        curModel=curMeshDoc->meshList.at(meshInd);
+        curModel=curMeshDoc->getMesh(meshInd);
         meshState.create(curmask, curModel);
         applyDynamic();
     }
 }
 
-bool MeshlabStdDialog::isDynamic()
+bool MeshlabStdDialog::isPreviewable()
 {
     return ((curmask != MeshModel::MM_UNKNOWN) && (curmask != MeshModel::MM_NONE) && !(curmask & MeshModel::MM_VERTNUMBER) && !(curmask & MeshModel::MM_FACENUMBER));
 }
@@ -156,7 +157,7 @@ void MeshlabStdDialog::loadFrameContent(MeshDocument *mdPt)
 	QPushButton *closeButton = new QPushButton("Close", qf);
 	QPushButton *applyButton = new QPushButton("Apply", qf);
 	QPushButton *defaultButton = new QPushButton("Default", qf);
-
+	applyButton->setFocus();
 #ifdef Q_WS_MAC
 	// Hack needed on mac for correct sizes of button in the bottom of the dialog.
 	helpButton->setMinimumSize(100, 25);
@@ -165,7 +166,7 @@ void MeshlabStdDialog::loadFrameContent(MeshDocument *mdPt)
 	defaultButton->setMinimumSize(100, 25);
 #endif 	
 
-	if(isDynamic())
+	if(isPreviewable())
 		{
 			previewCB = new QCheckBox("Preview", qf);
                         previewCB->setCheckState(Qt::Unchecked);
@@ -194,21 +195,14 @@ void MeshlabStdDialog::loadFrameContent(MeshDocument *mdPt)
 	this->adjustSize();
 }
 
-//void StdParFrame::loadFrameContent(ParameterDeclarationSet &curParSet,MeshDocument *mdPt)
-void StdParFrame::loadFrameContent(RichParameterSet &curParSet,MeshDocument */*mdPt*/)
+void StdParFrame::loadFrameContent(RichParameterSet &curParSet,MeshDocument * /*_mdPt*/ )
 {
  if(layout()) delete layout();
 	QGridLayout * vLayout = new QGridLayout(this);
     vLayout->setAlignment(Qt::AlignTop);
-	//QLabel* lb = new QLabel("",this);
-	//vLayout->addWidget(lb);
-	
 	setLayout(vLayout);
-	
-	//QLabel *ql;
 
-	QString descr;
-	RichWidgetInterfaceConstructor rwc(this);
+  RichWidgetInterfaceConstructor rwc(this);
 	for(int i = 0; i < curParSet.paramList.count(); i++)
 	{
 		RichParameter* fpi=curParSet.paramList.at(i);
@@ -217,7 +211,7 @@ void StdParFrame::loadFrameContent(RichParameterSet &curParSet,MeshDocument */*m
 		stdfieldwidgets.push_back(rwc.lastCreated);
 		helpList.push_back(rwc.lastCreated->helpLab);
 	} // end for each parameter
-
+	
 	this->setMinimumSize(vLayout->sizeHint());
 	this->showNormal();
 	this->adjustSize();
@@ -286,19 +280,19 @@ void MeshlabStdDialog::applyClick()
 	QAction *q = curAction;
 	stdParFrame->readValues(curParSet);
 	
-	////int mask = 0;//curParSet.getDynamicFloatMask();
-	if(curmask)	
+	// Note that curModel CAN BE NULL (for creation filters on empty docs...)
+	if(curmask && curModel)
 		meshState.apply(curModel);
 
 	//PreView Caching: if the apply parameters are the same to those used in the preview mode
 	//we don't need to reapply the filter to the mesh
 	bool isEqual = (curParSet == prevParSet);
-    if ((isEqual) && (validcache))
+	if (curModel && (isEqual) && (validcache))
 		meshCacheState.apply(curModel);
 	else
 		curmwi->executeFilter(q, curParSet, false);
 
-	if(curmask)	
+	if(curmask && curModel)
 		meshState.create(curmask, curModel);
 	if(this->curgla) 
 		this->curgla->update();
@@ -367,7 +361,7 @@ void MeshlabStdDialog::closeEvent(QCloseEvent * /*event*/)
 MeshlabStdDialog::~MeshlabStdDialog()
 {
 	delete stdParFrame;
-	if(isDynamic()) 
+	if(isPreviewable())
 		delete previewCB;
 }
 
@@ -536,7 +530,7 @@ Point3fWidget::Point3fWidget(QWidget *p, RichPoint3f* rpf, QWidget *gla_curr): M
 			connect(getPoint3Button,SIGNAL(clicked()),this,SLOT(getPoint()));
 			connect(getPoint3Combo,SIGNAL(currentIndexChanged(int)),this,SLOT(getPoint()));
 			connect(gla_curr,SIGNAL(transmitViewDir(QString,vcg::Point3f)),this,SLOT(setValue(QString,vcg::Point3f)));
-      connect(gla_curr,SIGNAL(transmitShot(QString,vcg::Shotf)),this,SLOT(setValue(QString,vcg::Point3f)));
+      connect(gla_curr,SIGNAL(transmitShot(QString,vcg::Shotf)),this,SLOT(setShotValue(QString,vcg::Shotf)));
 			connect(gla_curr,SIGNAL(transmitSurfacePos(QString,vcg::Point3f)),this,SLOT(setValue(QString,vcg::Point3f)));
 			connect(this,SIGNAL(askViewDir(QString)),gla_curr,SLOT(sendViewDir(QString)));
       connect(this,SIGNAL(askViewPos(QString)),gla_curr,SLOT(sendMeshShot(QString)));
@@ -571,7 +565,7 @@ void Point3fWidget::setValue(QString name,Point3f newVal)
 	}
 }
 
-void Point3fWidget::setValue(QString name,Shotf newValShot)
+void Point3fWidget::setShotValue(QString name,Shotf newValShot)
 {
   Point3f p = newValShot.GetViewPoint();
   setValue(name,p);
@@ -598,6 +592,126 @@ void Point3fWidget::setWidgetValue( const Value& nv )
 	for(unsigned int ii = 0; ii < 3;++ii)
 		coordSB[ii]->setText(QString::number(nv.getPoint3f()[ii],'g',3));
 }
+
+/******************************************/
+// Matrix44fWidget Implementation
+/******************************************/
+
+
+//QHBoxLayout(NULL)
+Matrix44fWidget::Matrix44fWidget(QWidget *p, RichMatrix44f* rpf,  QWidget *gla_curr): MeshLabWidget(p,rpf)
+{
+
+	paramName = rpf->name;
+	//int row = gridLay->rowCount() - 1;
+
+	descLab = new QLabel(rpf->pd->fieldDesc,p);
+	descLab->setToolTip(rpf->pd->fieldDesc);
+	gridLay->addWidget(descLab,row,0,Qt::AlignTop);
+
+	QVBoxLayout* lay = new QVBoxLayout(p);
+	QGridLayout* lay44 = new QGridLayout(p);
+
+	
+
+	for(int i =0;i<16;++i)
+		{
+			coordSB[i]= new QLineEdit(p);
+			QFont baseFont=coordSB[i]->font();
+			if(baseFont.pixelSize() != -1) baseFont.setPixelSize(baseFont.pixelSize()*3/4);
+															  else baseFont.setPointSize(baseFont.pointSize()*3/4);
+			coordSB[i]->setFont(baseFont);
+			//coordSB[i]->setMinimumWidth(coordSB[i]->sizeHint().width()/4);
+			coordSB[i]->setMinimumWidth(0);
+			coordSB[i]->setMaximumWidth(coordSB[i]->sizeHint().width()/2);
+			//coordSB[i]->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Fixed);
+			coordSB[i]->setValidator(new QDoubleValidator(p));
+			coordSB[i]->setAlignment(Qt::AlignRight);
+			//this->addWidget(coordSB[i],1,Qt::AlignHCenter);
+			 lay44->addWidget(coordSB[i],i/4,i%4);
+
+		}
+	this->setValue(paramName,rp->val->getMatrix44f());
+
+	lay->addLayout(lay44);
+
+	QPushButton     * getMatrixButton = new QPushButton("Read from current layer");
+    lay->addWidget(getMatrixButton);
+
+	QPushButton     * pasteMatrixButton = new QPushButton("Paste from clipboard");
+	lay->addWidget(pasteMatrixButton);
+
+	gridLay->addLayout(lay,row,1,Qt::AlignTop);
+
+	connect(gla_curr,SIGNAL(transmitMatrix(QString,vcg::Matrix44f)),this,SLOT(setValue(QString,vcg::Matrix44f)));
+	connect(getMatrixButton,SIGNAL(clicked()),this,SLOT(getMatrix()));
+	connect(pasteMatrixButton,SIGNAL(clicked()),this,SLOT(pasteMatrix()));
+	connect(this,SIGNAL(askMeshMatrix(QString)),  gla_curr,SLOT(sendMeshMatrix(QString)));
+
+}
+
+
+Matrix44fWidget::~Matrix44fWidget() {}
+
+void Matrix44fWidget::setValue(QString name,Matrix44f newVal)
+{
+	if(name==paramName)
+	{
+		for(int i =0;i<16;++i)
+			coordSB[i]->setText(QString::number(newVal[i/4][i%4],'g',4));
+	}
+}
+
+
+vcg::Matrix44f Matrix44fWidget::getValue()
+{
+	float val[16];
+	for(unsigned int i  = 0; i < 16; ++i)
+		val[i] = coordSB[i]->text().toFloat();
+	return Matrix44f(val); 
+}
+
+void Matrix44fWidget::getMatrix(){
+
+	emit askMeshMatrix(QString("TransformMatrix"));
+}
+
+void Matrix44fWidget::pasteMatrix(){
+	QClipboard *clipboard = QApplication::clipboard();
+	QString shotString = clipboard->text();
+	QStringList list1 = shotString.split(" ");
+	if(list1.size() != 16) return;
+	int id = 0;
+	for(QStringList::iterator i = list1.begin(); i != list1.end(); ++i,++id){
+		bool ok = true;
+		(*i).toFloat(&ok);
+		if(!ok) return;
+	}
+	id = 0;
+	for(QStringList::iterator i = list1.begin(); i != list1.end(); ++i,++id) 
+		coordSB[id]->setText(*i) ;
+}
+
+void Matrix44fWidget::collectWidgetValue()
+{
+	vcg::Matrix44f  m;
+	for(unsigned int i  = 0; i < 16; ++i) m[i/4][i%4] = coordSB[i]->text().toFloat();
+	rp->val->set(Matrix44fValue(m));
+}
+
+void Matrix44fWidget::resetWidgetValue()
+{
+	vcg::Matrix44f  m; m.SetIdentity();
+	for(unsigned int ii = 0; ii < 16;++ii)
+		coordSB[ii]->setText(QString::number(rp->pd->defVal->getMatrix44f()[ii/4][ii%4],'g',3));
+}
+
+void Matrix44fWidget::setWidgetValue( const Value& nv )
+{
+	for(unsigned int ii = 0; ii < 16;++ii)
+		coordSB[ii]->setText(QString::number(nv.getMatrix44f()[ii/4][ii%4],'g',3));
+}
+
 /********************/
 // ShotfWidget Implementation
 
@@ -614,7 +728,7 @@ ShotfWidget::ShotfWidget(QWidget *p, RichShotf* rpf, QWidget *gla_curr): MeshLab
   QHBoxLayout* lay = new QHBoxLayout(p);
 
 
-  this->setValue(paramName,rp->val->getShotf());
+  this->setShotValue(paramName,rp->val->getShotf());
   if(gla_curr) // if we have a connection to the current glarea we can setup the additional button for getting the current view direction.
     {
       getShotButton = new QPushButton("Get shot",p);
@@ -631,7 +745,7 @@ ShotfWidget::ShotfWidget(QWidget *p, RichShotf* rpf, QWidget *gla_curr): MeshLab
       lay->addWidget(getShotCombo);
       connect(getShotCombo,SIGNAL(currentIndexChanged(int)),this,SLOT(getShot()));
       connect(getShotButton,SIGNAL(clicked()),this,SLOT(getShot()));
-      connect(gla_curr,SIGNAL(transmitShot(QString,vcg::Shotf)),this,SLOT(setValue(QString,vcg::Shotf)));
+      connect(gla_curr,SIGNAL(transmitShot(QString,vcg::Shotf)),this,SLOT(setShotValue(QString,vcg::Shotf)));
       connect(this,SIGNAL(askViewerShot(QString)),gla_curr,SLOT(sendViewerShot(QString)));
       connect(this,SIGNAL(askMeshShot(QString)),  gla_curr,SLOT(sendMeshShot(QString)));
       connect(this,SIGNAL(askRasterShot(QString)),gla_curr,SLOT(sendRasterShot(QString)));
@@ -677,7 +791,7 @@ void ShotfWidget::getShot()
 
 ShotfWidget::~ShotfWidget() {}
 
-void ShotfWidget::setValue(QString name,Shotf newVal)
+void ShotfWidget::setShotValue(QString name,Shotf newVal)
 {
   if(name==paramName)
   {
@@ -1152,7 +1266,6 @@ MeshLabWidget::MeshLabWidget( QWidget* p,RichParameter* rpar )
 		helpLab->setMaximumWidth(QWIDGETSIZE_MAX);
 		gridLay = qobject_cast<QGridLayout*>(p->layout());
 		assert(gridLay != 0);
-
 		row = gridLay->rowCount();
 		//WARNING!!!!!!!!!!!!!!!!!! HORRIBLE PATCH FOR THE BOOL WIDGET PROBLEM
 		if ((row == 1) && (rpar->val->isBool()))	
@@ -1179,6 +1292,13 @@ MeshLabWidget::~MeshLabWidget()
 	delete helpLab;
 }
 
+void MeshLabWidget::setEqualSpaceForEachColumn()
+{
+	int singlewidth = gridLay->geometry().width() / gridLay->columnCount();
+	for (int ii = 0;ii < gridLay->columnCount();++ii)
+		gridLay->setColumnMinimumWidth(ii,singlewidth);
+}
+
 //connect(qcb,SIGNAL(stateChanged(int)),this,SIGNAL(parameterChanged()));
 BoolWidget::BoolWidget( QWidget* p,RichBool* rb )
 :MeshLabWidget(p,rb)
@@ -1200,7 +1320,6 @@ BoolWidget::BoolWidget( QWidget* p,RichBool* rb )
 		gridLay->addWidget(cb,row,0,1,2,Qt::AlignTop);
 
 	connect(cb,SIGNAL(stateChanged(int)),p,SIGNAL(parameterChanged()));
-
 }
 
 void BoolWidget::collectWidgetValue()
@@ -1235,7 +1354,9 @@ LineEditWidget::LineEditWidget( QWidget* p,RichParameter* rpar )
 	lab->setToolTip(rp->pd->tooltip);
 	gridLay->addWidget(lab,row,0,Qt::AlignTop);
 	gridLay->addWidget(lned,row,1,Qt::AlignTop);
-	connect(lned,SIGNAL(editingFinished()),p,SIGNAL(parameterChanged()));
+	connect(lned,SIGNAL(editingFinished()),this,SLOT(changeChecker()));
+	connect(this,SIGNAL(lineEditChanged()),p,SIGNAL(parameterChanged()));
+	lned->setAlignment(Qt::AlignLeft);
 }
 
 LineEditWidget::~LineEditWidget()
@@ -1243,6 +1364,17 @@ LineEditWidget::~LineEditWidget()
 	delete lned;
 	delete lab;
 }
+
+void LineEditWidget::changeChecker()
+{
+  if(lned->text() != this->lastVal)
+  {
+    this->lastVal = lned->text();
+    if(!this->lastVal.isEmpty())
+      emit lineEditChanged();
+  }
+}
+
 IntWidget::IntWidget( QWidget* p,RichInt* rpar )
 :LineEditWidget(p,rpar)
 {
@@ -1267,7 +1399,6 @@ void IntWidget::setWidgetValue( const Value& nv )
 FloatWidget::FloatWidget( QWidget* p,RichFloat* rpar )
 :LineEditWidget(p,rpar)
 {
-  this->lned->setAlignment(Qt::AlignRight);
 	lned->setText(QString::number(rp->val->getFloat(),'g',3));
 }
 
@@ -1307,60 +1438,6 @@ void StringWidget::setWidgetValue( const Value& nv )
 	lned->setText(nv.getString());
 }
 
-//Matrix44fWidget::Matrix44fWidget( QWidget* p,RichMatrix44f* rpar )
-//:MeshLabWidget(p,rb)
-//{
-//}
-//
-//void Matrix44fWidget::collectWidgetValue()
-//{
-//}
-//
-//void Matrix44fWidget::resetWidgetValue()
-//{
-//}
-
-
-/*
-FloatListWidget::FloatListWidget( QWidget* p,RichMesh* rpar )
-:MeshLabWidget(p,rb)
-{
-}
-
-
-void FloatListWidget::collectWidgetValue()
-{
-}
-
-void FloatListWidget::resetWidgetValue()
-{
-}
-
-OpenFileWidget::OpenFileWidget( QWidget* p,RichMesh* rpar )
-:MeshLabWidget(p,rb)
-{
-}
-
-void OpenFileWidget::collectWidgetValue()
-{
-}
-
-void OpenFileWidget::resetWidgetValue()
-{
-}
-*/
-
-/*
-ql = new QLabel(fpi.fieldDesc,this);
-ql->setToolTip(fpi.fieldToolTip);
-
-qcbt = new QColorButton(this,QColor(fpi.fieldVal.toUInt()));
-gridLayout->addWidget(ql,i,0,Qt::AlignTop);
-gridLayout->addLayout(qcbt,i,1,Qt::AlignTop);
-
-stdfieldwidgets.push_back(qcbt);
-connect(qcbt,SIGNAL(dialogParamChanged()),this,SIGNAL(parameterChanged()));
-*/
 
 /******************************************/
 // ColorWidget Implementation
@@ -1583,25 +1660,6 @@ void OpenFileWidget::selectFile()
 	emit dialogParamChanged();
 }
 
-//void OpenFileWidget::collectWidgetValue()
-//{
-//	rp->val->set(FileValue(fl));
-//}
-//
-//void OpenFileWidget::resetWidgetValue()
-//{
-//	QString fle = rp->pd->defVal->getFileName();
-//	fl = fle;
-//	updateFileName(fle);
-//}
-//
-//
-//void OpenFileWidget::setWidgetValue(const Value& nv)
-//{
-//	QString fle = nv.getFileName();
-//	fl = fle;
-//	updateFileName(QString());
-//}
 
 OpenFileWidget::~OpenFileWidget()
 {
@@ -1627,31 +1685,3 @@ void SaveFileWidget::selectFile()
 	updateFileName(fl);
 	emit dialogParamChanged();
 }
-
-//XMLBoolWidget::XMLBoolWidget( QWidget* parent,const QStringList& xmlWidgetTag )
-//{
-//
-//}
-
-//XMLMeshLabWidget::XMLMeshLabWidget( QWidget* p )
-//:QObject(parent)
-//{
-//	helpLab = new QLabel("<small>"+rpar->pd->tooltip +"</small>",p);
-//	helpLab->setTextFormat(Qt::RichText);
-//	helpLab->setWordWrap(true);
-//	helpLab->setVisible(false);
-//	helpLab->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
-//	helpLab->setMinimumWidth(250);
-//	helpLab->setMaximumWidth(QWIDGETSIZE_MAX);
-//	gridLay = qobject_cast<QGridLayout*>(p->layout());
-//	assert(gridLay != 0);
-//	row = gridLay->rowCount();
-//	gridLay->addWidget(helpLab,row,3,1,1,Qt::AlignTop);
-//}
-
-
-//XMLMeshLabWidget::XMLMeshLabWidget( Expression* expr,QWidget* parent )
-//:exp(expr),QObject(parent)
-//{
-//
-//}
