@@ -56,6 +56,8 @@ public:
 signals:
 	void parameterChanged();
 protected:
+	void setEqualSpaceForEachColumn();
+
 	int row;
 	QGridLayout* gridLay;
 	void InitRichParameter(RichParameter* rpar);
@@ -78,11 +80,18 @@ public:
 
 class LineEditWidget : public MeshLabWidget
 {
+  Q_OBJECT
 protected:
 	QLabel* lab;
 	QLineEdit* lned;
-public:
+	QString lastVal;
 
+	private slots:
+		void changeChecker();
+	signals:
+		void lineEditChanged();
+
+public:
 	LineEditWidget(QWidget* p,RichParameter* rpar);
 	~LineEditWidget();
 	virtual void collectWidgetValue() = 0;
@@ -249,7 +258,7 @@ public:
 	public slots:
 	void  getPoint();
 	void  setValue(QString name, vcg::Point3f val);	
-  void  setValue(QString name, vcg::Shotf val);
+  void  setShotValue(QString name, vcg::Shotf val);
 	signals:
 	void askViewDir(QString);
 	void askViewPos(QString);
@@ -263,6 +272,33 @@ protected:
 	QLabel* descLab;
 };
 
+class Matrix44fWidget : public MeshLabWidget
+{
+	Q_OBJECT
+	
+public:
+  Matrix44fWidget(QWidget *p,  RichMatrix44f* rpf, QWidget *gla_curr);
+  ~Matrix44fWidget();
+	QString paramName;
+	vcg::Matrix44f getValue();
+
+	void collectWidgetValue();
+	void resetWidgetValue();
+	void setWidgetValue(const Value& nv);
+	
+	public slots:
+	void  setValue(QString name, vcg::Matrix44f val);	
+	void getMatrix();
+	void pasteMatrix();
+	signals:
+	void askMeshMatrix(QString);
+
+
+protected:
+	QLineEdit * coordSB[16];
+	QPushButton *getPoint3Button;
+	QLabel* descLab;
+};
 
 /// Widget to enter a Shot,
 /// if at the creation you provide a pointer to a GLArea (the mesh viewing window)
@@ -284,7 +320,7 @@ public:
 
   public slots:
   void  getShot();
-  void  setValue(QString name, vcg::Shotf val);
+  void  setShotValue(QString name, vcg::Shotf val);
   signals:
   void askRasterShot(QString);
   void askMeshShot(QString);
@@ -330,7 +366,6 @@ protected:
 	QLabel* fieldDesc; 
   float minVal;
   float maxVal;
-	int mask;
 private :
 	float intToFloat(int val);  
 	int floatToInt(float val);
@@ -645,7 +680,6 @@ public:
 	~StdParFrame();
 signals:
 
-		void dynamicFloatChanged(int mask);
 		void parameterChanged();
 };
 
@@ -696,7 +730,7 @@ public:
 	void loadFrameContent(MeshDocument *mdPt=0);
 
 	bool showAutoDialog(MeshFilterInterface *mfi, MeshModel *mm, MeshDocument * md, QAction *q, MainWindowInterface *mwi, QWidget *gla=0);
-	bool isDynamic();
+	bool isPreviewable();
 
 	public slots:
 		void closeClick();
@@ -727,6 +761,7 @@ public:
 	RichParameterSet curParSet;
 	RichParameterSet prevParSet;
 	bool validcache;
+
 };
 
 //QWidget* parent parameter says to the class who will destroy the MeshLabWidget object that it had created
@@ -741,7 +776,7 @@ public:
 	void visit(RichInt& pd) {lastCreated = new IntWidget(par,&pd);};
 	void visit(RichFloat& pd){lastCreated = new FloatWidget(par,&pd);};
 	void visit(RichString& pd){lastCreated = new StringWidget(par,&pd);};
-	void visit(RichMatrix44f& /*pd*/){assert(0);/*TO BE IMPLEMENTED*/ /*lastCreated = new Matrix44fWidget(par,&pd);*/};
+	void visit(RichMatrix44f&  pd ){  lastCreated = new Matrix44fWidget(par,&pd,reinterpret_cast<StdParFrame*>(par)->gla); }
 	void visit(RichPoint3f& pd){lastCreated = new Point3fWidget(par,&pd,reinterpret_cast<StdParFrame*>(par)->gla);};
   void visit(RichShotf& pd){lastCreated = new ShotfWidget(par,&pd,reinterpret_cast<StdParFrame*>(par)->gla);};
 	void visit(RichColor& pd){lastCreated = new ColorWidget(par,&pd);};
