@@ -38,7 +38,7 @@
 #include "pickpointsDialog.h"
 
 #include <vcg/space/index/grid_static_ptr.h>
-#include <vcg/complex/trimesh/closest.h>
+#include <vcg/complex/algorithms/closest.h>
 
 using namespace vcg;
 
@@ -738,7 +738,6 @@ PickedPoints * PickPointsDialog::getPickedPoints()
 }
 
 void PickPointsDialog::loadPoints(QString filename){
-	vcg::tri::UpdateFlags<CMeshO>::FaceProjection(meshModel->cm);
 	//clear the points tree and template in case it was loaded
 	clearTemplate();
 		
