@@ -35,7 +35,7 @@
 #define AMBOCC_DEFAULT_TEXTURE_SIZE 512
 #define AMBOCC_DEFAULT_NUM_VIEWS 128
 #define AMBOCC_USEGPU_BY_DEFAULT false
-#define AMBOCC_USEVBO_BY_DEFAULT false
+#define AMBOCC_USEVBO_BY_DEFAULT true
 
 using namespace std;
 using namespace vcg;
@@ -81,15 +81,15 @@ QString AmbientOcclusionPlugin::filterInfo(FilterIDType filterId) const
 {
 	switch(filterId)
 	{
-		case FP_VERT_AMBIENT_OCCLUSION :  return QString("Generates environment occlusions values for the loaded mesh"); 
-		case FP_FACE_AMBIENT_OCCLUSION :  return QString("Generates environment occlusions values for the loaded mesh"); 
+    case FP_VERT_AMBIENT_OCCLUSION :
+    case FP_FACE_AMBIENT_OCCLUSION :  return QString("Compute ambient occlusions values; it takes a number or well distributed view direction and for point of the surface it computes how many time it is visible from these directions. This number is saved into quality and automatically mapped into a gray shade. The average direction is saved into an attribute named 'BentNormal'");
 		default : assert(0); 
 	}
 
 	return QString("");
 }
 
-int AmbientOcclusionPlugin::getRequirements(QAction *action)
+int AmbientOcclusionPlugin::getRequirements(QAction */*action*/)
 {
 	//no requirements needed
 	return 0;
@@ -194,6 +194,14 @@ bool AmbientOcclusionPlugin::processGL(MeshModel &m, vector<Point3f> &posVect)
 	vcg::tri::Allocator<CMeshO>::CompactFaceVector(m.cm);
 	vcg::tri::UpdateNormals<CMeshO>::PerVertexNormalizedPerFaceNormalized(m.cm);
 
+  CMeshO::PerVertexAttributeHandle<Point3f> BN = tri::Allocator<CMeshO>::GetPerVertexAttribute<Point3f>(m.cm, "BentNormal");
+  if(!tri::Allocator<CMeshO>::IsValidHandle<Point3f>(m.cm,BN))
+  {
+        BN = tri::Allocator<CMeshO>::AddPerVertexAttribute<Point3f> (m.cm,"BentNormal");
+        std::vector<std::string> AllVertexAttribName;
+        tri::Allocator<CMeshO>::GetAllPerVertexAttribute< Point3f >(m.cm,AllVertexAttribName);
+        qDebug("Now mesh has %i attrib",AllVertexAttribName.size());
+  }
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
 
@@ -276,13 +284,16 @@ bool AmbientOcclusionPlugin::processGL(MeshModel &m, vector<Point3f> &posVect)
 		{
 			tri::UpdateColor<CMeshO>::FaceQualityGray(m.cm);
 			CMeshO::FaceIterator fi;
-			for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD()) 
+      for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi)
 					(*fi).Q()=(*fi).Q()/numViews;
 		} else {
 			tri::UpdateColor<CMeshO>::VertexQualityGray(m.cm);
 			CMeshO::VertexIterator vi;
-			for(vi=m.cm.vert.begin();vi!=m.cm.vert.end();++vi) if(!(*vi).IsD()) 
+      for(vi=m.cm.vert.begin();vi!=m.cm.vert.end();++vi)
+      {
 					(*vi).Q()=(*vi).Q()/numViews;
+          BN[vi].Normalize();
+      }
 		}
 	
 	Log(0,"Successfully calculated A.O. after %3.2f sec, %3.2f of which is due to initialization", ((float)tAll.elapsed()/1000.0f), ((float)tInitElapsed/1000.0f) );
@@ -423,9 +434,9 @@ void AmbientOcclusionPlugin::initGL(vcg::CallBackPos *cb, unsigned int numVertic
 		cb(30, "Initializing: Shaders and Textures");
 
 		if (maxTexPages == 4)
-			set_shaders("ambient_occlusion4",vs,fs,shdrID);
+      set_shaders(":/AmbientOcclusion/shaders/ambient_occlusion4",vs,fs,shdrID);
 		else
-			set_shaders("ambient_occlusion8",vs,fs,shdrID);  //geforce 8+
+      set_shaders(":/AmbientOcclusion/shaders/ambient_occlusion8",vs,fs,shdrID);  //geforce 8+
 
 
 		maxTexSize = smartTexSize;
@@ -731,6 +742,8 @@ void AmbientOcclusionPlugin::generateOcclusionSW(MeshModel &m)
 	glReadPixels(0, 0, depthTexSize, depthTexSize, GL_DEPTH_COMPONENT, GL_FLOAT, dFloat);
 
 	cameraDir.Normalize();
+  CMeshO::PerVertexAttributeHandle<Point3f> BN = tri::Allocator<CMeshO>::GetPerVertexAttribute<Point3f>(m.cm, "BentNormal");
+
 	for (int i=0; i<m.cm.vn; ++i)
 	{
 		Point3<CMeshO::ScalarType> &vp = m.cm.vert[i].P();
@@ -744,6 +757,7 @@ void AmbientOcclusionPlugin::generateOcclusionSW(MeshModel &m)
 		if (resCoords[2] <= (GLdouble)dFloat[depthTexSize*y+x])
 		{
 			 m.cm.vert[i].Q() += max(m.cm.vert[i].cN().dot(cameraDir), 0.0f);
+       BN[ m.cm.vert[i] ] += cameraDir;
 		}
 	}
 
@@ -808,30 +822,6 @@ void AmbientOcclusionPlugin::applyOcclusionHW(MeshModel &m)
 
 void AmbientOcclusionPlugin::set_shaders(char *shaderName, GLuint &v, GLuint &f, GLuint &pr)
 {	
-	QDir shadersDir = QDir(qApp->applicationDirPath());
-
-
-#if defined(Q_OS_WIN)
-	if (shadersDir.dirName() == "debug" || shadersDir.dirName() == "release" || shadersDir.dirName() == "plugins"  )
-		shadersDir.cdUp();
-#elif defined(Q_OS_MAC)
-	if (shadersDir.dirName() == "MacOS") {
-		for(int i=0;i<4;++i)
-		{
-			if(shadersDir.exists("shaders"))
-				break;
-			shadersDir.cdUp();
-		}
-	}
-#endif
-
-	bool ret=shadersDir.cd("shaders");
-	if(!ret) 
-	{
-		QMessageBox::information(0, "Ambient Occlusion Plugin","Unable to find the shaders directory.\nNo shaders will be loaded.");
-		return;
-	}
-
 	f = glCreateShader(GL_FRAGMENT_SHADER);
 	v = glCreateShader(GL_VERTEX_SHADER);
 
@@ -845,7 +835,7 @@ void AmbientOcclusionPlugin::set_shaders(char *shaderName, GLuint &v, GLuint &f,
 	
 	fileName = fileName.left(fileName.size()-1);
 	fileName.append(".vert");
-	file.setFileName(shadersDir.absoluteFilePath(fileName));
+  file.setFileName(fileName);
 	if (file.open(QIODevice::ReadOnly))
 	{
 		QTextStream ts(&file);
@@ -862,7 +852,7 @@ void AmbientOcclusionPlugin::set_shaders(char *shaderName, GLuint &v, GLuint &f,
 	fileName = fileName.left(fileName.size()-5);
 	fileName.append(nMRT);
 	fileName.append(".frag");
-	file.setFileName(shadersDir.absoluteFilePath(fileName));
+  file.setFileName(fileName);
 	if (file.open(QIODevice::ReadOnly))
 	{
 		QTextStream ts(&file);
