@@ -20,16 +20,14 @@
 * for more details.                                                         *
 *                                                                           *
 ****************************************************************************/
-#include <QtGui>
 
-#include <vcg/complex/trimesh/clean.h>
-#include <vcg/complex/trimesh/stat.h>
-#include <vcg/complex/trimesh/smooth.h>
-#include <vcg/complex/trimesh/update/flag.h>
-#include <vcg/complex/trimesh/update/curvature.h>
-#include <vcg/complex/trimesh/update/quality.h>
-#include <limits>
 #include "meshcolorize.h"
+#include <vcg/complex/algorithms/clean.h>
+#include <vcg/complex/algorithms/stat.h>
+#include <vcg/complex/algorithms/smooth.h>
+#include <vcg/complex/algorithms/update/curvature.h>
+#include <vcg/complex/algorithms/parametrization/distortion.h>
+
 
 
 using namespace std;
@@ -48,7 +46,8 @@ ExtraMeshColorizePlugin::ExtraMeshColorizePlugin() {
 		CP_VERTEX_TO_FACE <<
 		CP_FACE_TO_VERTEX <<
 		CP_TEXTURE_TO_VERTEX <<
-    CP_RANDOM_FACE;
+    CP_RANDOM_FACE <<
+    CP_RANDOM_CONNECTED_COMPONENT;
     
   FilterIDType tt;
   foreach(tt , types())
@@ -70,6 +69,7 @@ QString ExtraMeshColorizePlugin::filterName(FilterIDType c) const{
   case CP_FACE_TO_VERTEX:            return QString("Transfer Color: Face to Vertex");
   case CP_TEXTURE_TO_VERTEX:         return QString("Transfer Color: Texture to Vertex");
   case CP_RANDOM_FACE:               return QString("Random Face Color");
+  case CP_RANDOM_CONNECTED_COMPONENT:return QString("Random Component Color ");
   default: assert(0);
   }
   return QString("error!");
@@ -94,31 +94,12 @@ QString ExtraMeshColorizePlugin::filterInfo(FilterIDType filterId) const {
   case CP_FACE_TO_VERTEX:           return QString("Face to Vertex color transfer");
   case CP_TEXTURE_TO_VERTEX:        return QString("Texture to Vertex color transfer");
   case CP_COLOR_NON_TOPO_COHERENT : return QString("Color edges topologically non coherent.");
-  case CP_RANDOM_FACE:              return QString("Colorize Faces randomly. If internal edges are present they are used");
+  case CP_RANDOM_FACE:              return QString("Colorize Faces randomly. If internal edges are present they are used. Useful for quads.");
+  case CP_RANDOM_CONNECTED_COMPONENT:  return QString("Colorize each connected component randomly.");
   default: assert(0); return QString("");
   }
 }
 
-// What "new" properties the plugin requires
-int ExtraMeshColorizePlugin::getRequirements(QAction *action){
-  switch(ID(action)){
-  case CP_DISCRETE_CURVATURE:       return MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACEFLAGBORDER | MeshModel::MM_VERTCURV;
-  case CP_TRIANGLE_QUALITY:         return MeshModel::MM_FACECOLOR | MeshModel::MM_FACEQUALITY;
-  case CP_SATURATE_QUALITY:         return MeshModel::MM_VERTFACETOPO;
-  case CP_RANDOM_FACE:              return MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACECOLOR;
-  case CP_CLAMP_QUALITY:            return 0; // TODO: split clamp on vertex & faces and add requirements
-  case CP_MAP_VQUALITY_INTO_COLOR:  return MeshModel::MM_VERTCOLOR;
-  case CP_MAP_FQUALITY_INTO_COLOR:  return MeshModel::MM_FACECOLOR;
-  case CP_VERTEX_SMOOTH:            return 0; // TODO: should it be MM_NONE?
-  case CP_FACE_SMOOTH:              return MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACECOLOR ;
-  case CP_VERTEX_TO_FACE:           return MeshModel::MM_FACECOLOR;
-  case CP_FACE_TO_VERTEX:           return MeshModel::MM_VERTCOLOR;
-  case CP_TEXTURE_TO_VERTEX:        return MeshModel::MM_FACECOLOR;
-  default: assert(0); return 0;
-  }
-  return 0;
-}
-
 void ExtraMeshColorizePlugin::initParameterSet(QAction *a, MeshModel &m, RichParameterSet & par){
   // variables cannot be defined within switch statement
   QStringList metrics;
@@ -131,9 +112,12 @@ void ExtraMeshColorizePlugin::initParameterSet(QAction *a, MeshModel &m, RichPar
     par.addParam(new RichInt("iteration",1,QString("Iteration"),QString("the number ofiteration of the smoothing algorithm")));
     break;
   case CP_TRIANGLE_QUALITY:
-      metrics.push_back("area/max side");
+            metrics.push_back("area/max side");
 			metrics.push_back("inradius/circumradius");
 			metrics.push_back("mean ratio");
+			metrics.push_back("Area");
+			metrics.push_back("Texture Angle Distortion");
+			metrics.push_back("Texture Area Distortion");
 			par.addParam(new RichEnum("Metric", 0, metrics, tr("Metric:"), tr("Choose a metric to compute triangle quality.")));
 			break;
   case CP_DISCRETE_CURVATURE:
@@ -174,18 +158,22 @@ bool ExtraMeshColorizePlugin::applyFilter(QAction *filter, MeshDocument &md, Ric
  MeshModel &m=*(md.mm());
  switch(ID(filter)) {
   case CP_SATURATE_QUALITY:{
+     m.updateDataMask(MeshModel::MM_VERTFACETOPO);
      tri::UpdateQuality<CMeshO>::VertexSaturate(m.cm, par.getFloat("gradientThr"));
      if(par.getBool("updateColor"))
      {
        Histogramf H;
        tri::Stat<CMeshO>::ComputePerVertexQualityHistogram(m.cm,H);
+       m.updateDataMask(MeshModel::MM_VERTCOLOR);
        tri::UpdateColor<CMeshO>::VertexQualityRamp(m.cm,H.Percentile(0.1f),H.Percentile(0.9f));
      }
      Log("Saturated ");
    }
 break;
+  case CP_MAP_VQUALITY_INTO_COLOR:
+     m.updateDataMask(MeshModel::MM_VERTCOLOR);
   case CP_CLAMP_QUALITY:
-  case CP_MAP_VQUALITY_INTO_COLOR:{
+      {
       float RangeMin = par.getFloat("minVal");	
       float RangeMax = par.getFloat("maxVal");		
 			bool usePerc = par.getDynamicFloat("perc")>0;
@@ -217,6 +205,7 @@ break;
       break;
     }
    case CP_MAP_FQUALITY_INTO_COLOR: {
+       m.updateDataMask(MeshModel::MM_FACECOLOR);
        float RangeMin = par.getFloat("minVal");
        float RangeMax = par.getFloat("maxVal");
        float perc = par.getDynamicFloat("perc");
@@ -243,6 +232,10 @@ break;
 
   case CP_DISCRETE_CURVATURE:
     {
+      m.updateDataMask(MeshModel::MM_FACEFACETOPO | MeshModel::MM_VERTCURV);
+      m.updateDataMask(MeshModel::MM_VERTCOLOR | MeshModel::MM_VERTQUALITY);
+      tri::UpdateFlags<CMeshO>::FaceBorderFromFF(m.cm);
+
       if ( tri::Clean<CMeshO>::CountNonManifoldEdgeFF(m.cm) > 0) {
 				errorMessage = "Mesh has some not 2-manifold faces, Curvature computation requires manifoldness"; // text
 				return false; // can't continue, mesh can't be processed
@@ -266,41 +259,97 @@ break;
       tri::Stat<CMeshO>::ComputePerVertexQualityHistogram(m.cm,H);
       tri::UpdateColor<CMeshO>::VertexQualityRamp(m.cm,H.Percentile(0.1f),H.Percentile(0.9f));
       Log( "Curvature Range: %f %f (Used 90 percentile %f %f) ",H.MinV(),H.MaxV(),H.Percentile(0.1f),H.Percentile(0.9f));
-			m.updateDataMask(MeshModel::MM_VERTQUALITY);
-			m.updateDataMask(MeshModel::MM_VERTCOLOR);
     break;
     }  
-  case CP_TRIANGLE_QUALITY:
-    {
-			CMeshO::FaceIterator fi;
-			float min = 0;
-			float max = 1.0;
-			int metric = par.getEnum("Metric");
-			switch(metric){ 
-			case 0: { //area / max edge
-          max = sqrt(3.0f)/2.0f;
-          for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi)
-            if(!(*fi).IsD())
-              (*fi).Q() = vcg::Quality((*fi).P(0), (*fi).P(1),(*fi).P(2));
-        } break;
-			case 1: { //inradius / circumradius
-          for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi)
-            if(!(*fi).IsD())
-              (*fi).Q() = vcg::QualityRadii((*fi).P(0), (*fi).P(1), (*fi).P(2));
-        } break;
-			case 2: { //mean ratio
-          for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi)
-            if(!(*fi).IsD())
-              (*fi).Q() = vcg::QualityMeanRatio((*fi).P(0), (*fi).P(1), (*fi).P(2));
-        } break;
-			default: assert(0);
-			} 
-      tri::UpdateColor<CMeshO>::FaceQualityRamp(m.cm,min,max,false);
-		break;
-    }
+   case CP_TRIANGLE_QUALITY:
+   {
+     m.updateDataMask(MeshModel::MM_FACECOLOR | MeshModel::MM_FACEQUALITY);
+     CMeshO::FaceIterator fi;
+     Distribution<float> distrib;
+     float minV = 0;
+     float maxV = 1.0;
+     int metric = par.getEnum("Metric");
+     if(metric ==4 || metric ==5 )
+     {
+       if(!m.hasDataMask(MeshModel::MM_VERTTEXCOORD) && !m.hasDataMask(MeshModel::MM_WEDGTEXCOORD))
+       {
+         this->errorMessage = "This metric need Texture Coordinate";
+         return false;
+       }
+     }
+     switch(metric){
+
+       case 0: { //area / max edge
+         minV = 0;
+         maxV = sqrt(3.0f)/2.0f;
+         for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = vcg::Quality((*fi).P(0), (*fi).P(1),(*fi).P(2));
+       } break;
+
+       case 1: { //inradius / circumradius
+         for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = vcg::QualityRadii((*fi).P(0), (*fi).P(1), (*fi).P(2));
+       } break;
+
+       case 2: { //mean ratio
+         for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = vcg::QualityMeanRatio((*fi).P(0), (*fi).P(1), (*fi).P(2));
+       } break;
+
+       case 3: { // AREA
+         for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+           (*fi).Q() = vcg::DoubleArea((*fi))*0.5f;
+         tri::Stat<CMeshO>::ComputePerFaceQualityMinMax(m.cm,minV,maxV);
+       } break;
+
+       case 4: { //TEXTURE Angle Distortion
+         if(m.hasDataMask(MeshModel::MM_WEDGTEXCOORD))
+         {
+           for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = Distortion<CMeshO,true>::AngleDistortion(&*fi);
+         } else {
+           for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = Distortion<CMeshO,false>::AngleDistortion(&*fi);
+         }
+         tri::Stat<CMeshO>::ComputePerFaceQualityDistribution(m.cm,distrib);
+         minV = distrib.Percentile(0.05);
+         maxV = distrib.Percentile(0.95);
+       } break;
+
+       case 5: { //TEXTURE Area Distortion
+         float areaScaleVal, edgeScaleVal;
+         if(m.hasDataMask(MeshModel::MM_WEDGTEXCOORD))
+         {
+           Distortion<CMeshO,true>::MeshScalingFactor(m.cm, areaScaleVal,edgeScaleVal);
+           for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = Distortion<CMeshO,true>::AreaDistortion(&*fi,areaScaleVal);
+         } else {
+           Distortion<CMeshO,false>::MeshScalingFactor(m.cm, areaScaleVal,edgeScaleVal);
+           for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD())
+             (*fi).Q() = Distortion<CMeshO,false>::AreaDistortion(&*fi,areaScaleVal);
+
+         }
+         tri::Stat<CMeshO>::ComputePerFaceQualityDistribution(m.cm,distrib);
+         minV = distrib.Percentile(0.05);
+         maxV = distrib.Percentile(0.95);
+       } break;
+
+       default: assert(0);
+     }
+     tri::UpdateColor<CMeshO>::FaceQualityRamp(m.cm,minV,maxV,false);
+     break;
+   }
 
 
-  case CP_RANDOM_FACE:
+  case CP_RANDOM_CONNECTED_COMPONENT:
+   m.updateDataMask(MeshModel::MM_FACEFACETOPO);
+   m.updateDataMask(MeshModel::MM_FACEMARK | MeshModel::MM_FACECOLOR);
+   vcg::tri::UpdateColor<CMeshO>::FaceRandomConnectedComponent(m.cm);
+   break;
+
+ case CP_RANDOM_FACE:
+     m.updateDataMask(MeshModel::MM_FACEFACETOPO);
+     m.updateDataMask(MeshModel::MM_FACEMARK | MeshModel::MM_FACECOLOR);
     vcg::tri::UpdateColor<CMeshO>::MultiFaceRandom(m.cm);
     break;
 
@@ -312,29 +361,33 @@ break;
 		break;
   case CP_FACE_SMOOTH:
 		{
+    m.updateDataMask(MeshModel::MM_FACEFACETOPO);
 		int iteration = par.getInt("iteration");
 		tri::Smooth<CMeshO>::FaceColorLaplacian(m.cm,iteration,false,cb);
 		}
 		break;
   case CP_FACE_TO_VERTEX:
-		 tri::UpdateColor<CMeshO>::VertexFromFace(m.cm);
+     m.updateDataMask(MeshModel::MM_VERTCOLOR);
+     tri::UpdateColor<CMeshO>::VertexFromFace(m.cm);
 		break;
 	 case CP_VERTEX_TO_FACE:
-		 tri::UpdateColor<CMeshO>::FaceFromVertex(m.cm);
+     m.updateDataMask(MeshModel::MM_FACECOLOR);
+     tri::UpdateColor<CMeshO>::FaceFromVertex(m.cm);
 		 break;
   case CP_TEXTURE_TO_VERTEX:
 		{
-			if(!HasPerWedgeTexCoord(m.cm)) break;
+      m.updateDataMask(MeshModel::MM_VERTCOLOR);
+      if(!HasPerWedgeTexCoord(m.cm)) break;
 			CMeshO::FaceIterator fi; 
 			QImage tex(m.cm.textures[0].c_str());
 			for(fi=m.cm.face.begin();fi!=m.cm.face.end();++fi) if(!(*fi).IsD()) 
 			{
 				for (int i=0; i<3; i++)
 				{
-					vcg::Point2f newcoord((*fi).WT(i).P().X()-(int)(*fi).WT(i).P().X(),(*fi).WT(i).P().Y()-(int)(*fi).WT(i).P().Y());
-					QRgb val = tex.pixel(newcoord[0]*tex.width(),(1-newcoord[1])*tex.height()-1);
+          // note the trick for getting only the fractional part of the uv with the correct wrapping (e.g. 1.5 -> 0.5 and -0.3 -> 0.7)
+          vcg::Point2f newcoord((*fi).WT(i).P().X()-floor((*fi).WT(i).P().X()),(*fi).WT(i).P().Y()-floor((*fi).WT(i).P().Y()));
+          QRgb val = tex.pixel(newcoord[0]*tex.width(),(1-newcoord[1])*tex.height()-1);
 					(*fi).V(i)->C().SetRGB(qRed(val),qGreen(val),qBlue(val));
-					
 				}
 			}
 	    }
@@ -362,6 +415,7 @@ MeshFilterInterface::FilterClass ExtraMeshColorizePlugin::getClass(QAction *a){
     return FilterClass(Quality + FaceColoring);
 
   case   CP_RANDOM_FACE:
+  case   CP_RANDOM_CONNECTED_COMPONENT:
   case   CP_FACE_SMOOTH:
   case   CP_VERTEX_TO_FACE:
   case   CP_MAP_FQUALITY_INTO_COLOR:
@@ -377,6 +431,7 @@ int ExtraMeshColorizePlugin::getPreConditions(QAction *a) const{
   switch(ID(a)){
   case CP_TRIANGLE_QUALITY:
   case CP_RANDOM_FACE:
+  case CP_RANDOM_CONNECTED_COMPONENT:
   case CP_DISCRETE_CURVATURE:
   case CP_COLOR_NON_TOPO_COHERENT:
     return MeshModel::MM_FACENUMBER;
@@ -404,6 +459,7 @@ int ExtraMeshColorizePlugin::postCondition( QAction* a ) const{
   case CP_TRIANGLE_QUALITY:
     return MeshModel::MM_FACECOLOR | MeshModel::MM_FACEQUALITY;
   case CP_RANDOM_FACE:
+  case CP_RANDOM_CONNECTED_COMPONENT:
   case CP_COLOR_NON_TOPO_COHERENT:
   case CP_FACE_SMOOTH:
   case CP_VERTEX_TO_FACE:
