@@ -6,10 +6,10 @@
 #include <wrap/io_trimesh/import_ply.h>
 #include <vcg/space/triangle3.h>
 #include <vcg/simplex/face/component_rt.h>
-#include <vcg/complex/trimesh/base.h>
+#include <vcg/complex/complex.h>
 #include "local_parametrization.h"
 #include "uv_grid.h"
-#include <vcg/complex/trimesh/stat.h>
+#include <vcg/complex/algorithms/stat.h>
 #include "param_mesh.h"
 
 ///ABSTRACT MESH THAT MAINTAINS THE WHOLE PARAMETERIZATION
@@ -43,7 +43,8 @@ class AbstractFace    : public vcg::Face  < AbstractUsedTypes,
 	vcg::face::FFAdj,
 	vcg::face::VertexRef,
 	vcg::face::Color4b,
-	vcg::face::BitFlags>
+	vcg::face::BitFlags,
+	vcg::face::Quality3f>
 //vcg::face::Normal3f>
 {};
 
@@ -89,7 +90,7 @@ public:
     }
 
 	CoordType RPos;
-	static const bool Has_Auxiliary(){return false;}
+  static bool Has_Auxiliary(){return false;}
 };
 
 class ParamFace: public vcg::Face <  ParamUsedTypes,
@@ -98,10 +99,7 @@ class ParamFace: public vcg::Face <  ParamUsedTypes,
   vcg::face::Normal3f,
 	vcg::face::WedgeTexCoord2f,
 	vcg::face::Mark,
-	vcg::face::EdgePlane
-  //,
-  //vcg::face::Qualityf // not really needed
-  >
+	vcg::face::Quality3f>
 {
 public:
 	/*template < class LeftV>
@@ -845,14 +843,10 @@ private:
 	void Clamp(vcg::Point2f &UV)
 	{
 		float eps=0.00001f;
-		if ((UV.X()<eps)&&(UV.X()>-eps))
-			UV.X()=0;
-		if ((UV.X()<1+eps)&&(UV.X()>1-eps))
-			UV.X()=1;
-		if ((UV.Y()<eps)&&(UV.Y()>-eps))
-			UV.Y()=0;
-		if ((UV.Y()<1+eps)&&(UV.Y()>1-eps))
-			UV.Y()=1;
+    if (UV.X() < eps) UV.X()=0;
+    if (UV.X()>1-eps) UV.X()=1;
+    if (UV.Y() < eps) UV.Y()=0;
+    if (UV.Y()>1-eps) UV.Y()=1;
 	}
 	
 	
@@ -1219,7 +1213,7 @@ public:
 			vcg::Point2<ScalarType> UV_interp=bary3D.X()*UVd0+bary3D.Y()*UVd1+bary3D.Z()*UVd2;
 			inv_GE1(EdgeIndex,UV_interp,I,UV);
 			Clamp(UV);
-			assert((UV.X()>=0)&&(UV.Y()>=0)&&(UV.X()<=1)&&(UV.Y()<=1)&&(UV.X()+UV.Y()<=1+eps));
+      //assert((UV.X()>=0)&&(UV.Y()>=0)&&(UV.X()<=1)&&(UV.Y()<=1)&&(UV.X()+UV.Y()<=1+eps));
 			assert((I==I0)||(I==I1)||(I==I2));
 			return;
 		}
@@ -1352,7 +1346,7 @@ public:
 		if (LocalIndex==-1)
 			return false;
 
-		GetUV<AbstractMesh>(&star_domain->face[LocalIndex],bary,UVHstar.X(),UVHstar.Y());
+    InterpolateUV<AbstractMesh>(&star_domain->face[LocalIndex],bary,UVHstar.X(),UVHstar.Y());
 		return true;
 	}
 
@@ -1408,7 +1402,7 @@ public:
 		int LocalIndex=diamond_meshes[DiamIndex].Global2Local(I);
 		if(LocalIndex!=-1)
 		{
-			GetUV<AbstractMesh>(&diam_domain->face[LocalIndex],bary,UVDiam.X(),UVDiam.Y());
+      InterpolateUV<AbstractMesh>(&diam_domain->face[LocalIndex],bary,UVDiam.X(),UVDiam.Y());
 			return;
 		}
 		///if ! found seach in the star space
