@@ -30,18 +30,17 @@
 
 #include <common/interfaces.h>
 
-#include <vcg/complex/trimesh/clean.h>
-#include <vcg/complex/trimesh/refine.h>
-#include <vcg/complex/trimesh/refine_loop.h>
-#include <vcg/complex/trimesh/append.h>
-#include <vcg/complex/trimesh/create/advancing_front.h>
-#include <vcg/complex/trimesh/create/marching_cubes.h>
+#include <vcg/complex/algorithms/clean.h>
+#include <vcg/complex/algorithms/refine.h>
+#include <vcg/complex/algorithms/refine_loop.h>
+#include <vcg/complex/append.h>
+#include <vcg/complex/algorithms/create/advancing_front.h>
+#include <vcg/complex/algorithms/create/marching_cubes.h>
 #include "mlsmarchingcube.h"
 
 #include "mlsplugin.h"
 #include "rimls.h"
 #include "apss.h"
-#include "mlsutils.h"
 #include "implicits.h"
 
 
@@ -369,7 +368,7 @@ bool MlsPlugin::applyFilter(QAction* filter, MeshDocument& md, RichParameterSet&
 	if (id == FP_SELECT_SMALL_COMPONENTS)
 	{
 		MeshModel* mesh = md.mm();
-		mesh->updateDataMask(MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACEFLAGBORDER);
+		mesh->updateDataMask(MeshModel::MM_FACEFACETOPO);
 		bool nonClosedOnly = par.getBool("NonClosedOnly");
 		float ratio = par.getFloat("NbFaceRatio");
 		vcg::tri::SmallComponent<CMeshO>::Select(mesh->cm, ratio, nonClosedOnly);
@@ -466,13 +465,13 @@ bool MlsPlugin::applyFilter(QAction* filter, MeshDocument& md, RichParameterSet&
 				//UpdateFaceNormalFromVertex(m.cm);
 				if (k!=0)
 				{
-					mesh->updateDataMask(MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACEFLAGBORDER);
+					mesh->updateDataMask(MeshModel::MM_FACEFACETOPO);
 
 					vcg::tri::UpdateNormals<CMeshO>::PerFace(mesh->cm);
 					vcg::tri::UpdateNormals<CMeshO>::NormalizeFace(mesh->cm);
 					//vcg::RefineE<CMeshO,vcg::MidPoint<CMeshO> >(m.cm, vcg::MidPoint<CMeshO>(), edgePred, false, cb);
 					vcg::tri::RefineOddEvenE<CMeshO, tri::OddPointLoop<CMeshO>, tri::EvenPointLoop<CMeshO> >
-							(mesh->cm, tri::OddPointLoop<CMeshO>(), tri::EvenPointLoop<CMeshO>(), edgePred, selectionOnly, cb);
+							(mesh->cm, tri::OddPointLoop<CMeshO>(mesh->cm), tri::EvenPointLoop<CMeshO>(), edgePred, selectionOnly, cb);
 				}
 				// project all vertices onto the MLS surface
 				for (unsigned int i = 0; i< mesh->cm.vert.size(); i++)
@@ -590,12 +589,12 @@ bool MlsPlugin::applyFilter(QAction* filter, MeshDocument& md, RichParameterSet&
 
 			// extra zero detection and removal
 			{
-				mesh->updateDataMask(MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACEFLAGBORDER);
+				mesh->updateDataMask(MeshModel::MM_FACEFACETOPO );
 				// selection...
 				vcg::tri::SmallComponent<CMeshO>::Select(mesh->cm, 0.1f);
 				// deletion...
 				vcg::tri::SmallComponent<CMeshO>::DeleteFaceVert(mesh->cm);
-				mesh->clearDataMask(MeshModel::MM_FACEFACETOPO | MeshModel::MM_FACEFLAGBORDER);
+				mesh->clearDataMask(MeshModel::MM_FACEFACETOPO);
 			}
 
 			Log( "Marching cubes MLS meshing done.");
