@@ -20,22 +20,33 @@
 * for more details.                                                         *
 *                                                                           *
 ****************************************************************************/
+#include <vcg/math/base.h>
 #include <QtGui>
+#define EIGEN_YES_I_KNOW_SPARSE_MODULE_IS_NOT_STABLE_YET
+#include <eigenlib/Eigen/Sparse>
+#include <eigenlib/Eigen/src/Sparse/SparseMatrix.h>
+#include <eigenlib/Eigen/src/Sparse/DynamicSparseMatrix.h>
+#include <eigenlib/unsupported/Eigen/SparseExtra>
 
-#include <math.h>
 #include <float.h>
 #include <stdlib.h>
 #include "filter_texture.h"
 #include "pushpull.h"
 #include "rastering.h"
+#include <vcg/complex/algorithms/update/texture.h>
+#include<wrap/io_trimesh/export_ply.h>
+#include <vcg/complex/algorithms/parametrization/voronoi_atlas.h>
 
+using namespace vcg;
 
 FilterTexturePlugin::FilterTexturePlugin() 
 { 
-    typeList << FP_UV_TO_COLOR
+    typeList << FP_VORONOI_ATLAS
             << FP_UV_WEDGE_TO_VERTEX
+            << FP_UV_VERTEX_TO_WEDGE
             << FP_BASIC_TRIANGLE_MAPPING
             << FP_SET_TEXTURE
+            << FP_PLANAR_MAPPING
             << FP_COLOR_TO_TEXTURE
             << FP_TRANSFER_TO_TEXTURE
             << FP_TEX_TO_VCOLOR_TRANSFER;
@@ -48,12 +59,14 @@ QString FilterTexturePlugin::filterName(FilterIDType filterId) const
 {
     switch(filterId)
     {
-    case FP_UV_TO_COLOR : return QString("UV to Color");
+    case FP_VORONOI_ATLAS : return QString("Parametrization: Voronoi Atlas");
     case FP_UV_WEDGE_TO_VERTEX : return QString("Convert PerWedge UV into PerVertex UV");
-    case FP_BASIC_TRIANGLE_MAPPING : return QString("Trivial Per-Triangle Parametrization ");
+    case FP_UV_VERTEX_TO_WEDGE : return QString("Convert PerVertex UV into PerWedge UV");
+    case FP_BASIC_TRIANGLE_MAPPING : return QString("Parametrization: Trivial Per-Triangle ");
+    case FP_PLANAR_MAPPING : return QString("Parametrization: Flat Plane ");
     case FP_SET_TEXTURE : return QString("Set Texture");
     case FP_COLOR_TO_TEXTURE : return QString("Vertex Color to Texture");
-    case FP_TRANSFER_TO_TEXTURE : return QString("Transfer Color to Texture (between 2 meshes)");
+    case FP_TRANSFER_TO_TEXTURE : return QString("Transfer Vertex Attributes to Texture (between 2 meshes)");
     case FP_TEX_TO_VCOLOR_TRANSFER : return QString("Texture to Vertex Color (between 2 meshes)"); // TODO Choose a name
     default : assert(0);
     }
@@ -65,13 +78,17 @@ QString FilterTexturePlugin::filterInfo(FilterIDType filterId) const
 {
     switch(filterId)
     {
-    case FP_UV_TO_COLOR :  return QString("Maps the UV Space into a color space, thus colorizing mesh vertices according to UV coords.");
+    case FP_VORONOI_ATLAS :  return QString("Build an atlased parametrization based on a geodesic voronoi partitioning of the surface and parametrizing each region using Harmonic Mapping. For the  parametrization of the disk like voronoi regions the used method is: <br><b>Ulrich Pinkall, Konrad Polthier</b><br>\
+                                            <i>Computing Discrete Minimal Surfaces and Their Conjugates</i> <br>\
+                                            Experimental Mathematics, Vol 2 (1), 1993<br> .");
     case FP_UV_WEDGE_TO_VERTEX : return QString("Converts per Wedge Texture Coordinates to per Vertex Texture Coordinates splitting vertices with not coherent Wedge coordinates.");
+    case FP_UV_VERTEX_TO_WEDGE : return QString("Converts per Vertex Texture Coordinates to per Wedge Texture Coordinates. It does not merge superfluos vertices...");
     case FP_BASIC_TRIANGLE_MAPPING : return QString("Builds a trivial triangle-by-triangle parametrization. <br> Two methods are provided, the first maps maps all triangles into equal sized triangles, while the second one adapt the size of the triangles in texture space to their original size.");
+    case FP_PLANAR_MAPPING : return QString("Builds a trivial flat plane parametrization.");
     case FP_SET_TEXTURE : return QString("Set a texture associated with current mesh parametrization.<br>"
                                          "If the texture provided exists it will be simply associated to the current mesh else a dummy texture will be created and saved in the same directory.");
     case FP_COLOR_TO_TEXTURE : return QString("Fills the specified texture accordingly to per vertex color.");
-    case FP_TRANSFER_TO_TEXTURE : return QString("Transfer texture/vertex color from one mesh to another's texture.");
+    case FP_TRANSFER_TO_TEXTURE : return QString("Transfer texture color, vertex color or normal from one mesh to another's texture. This is generally used to restore detail lost in simplification");
     case FP_TEX_TO_VCOLOR_TRANSFER : return QString("Generates Vertex Color values picking color from another mesh texture.");
     default : assert(0);
     }
@@ -82,9 +99,11 @@ int FilterTexturePlugin::getPreConditions(QAction *a) const
 {
     switch (ID(a))
     {
-    case FP_UV_TO_COLOR : return MeshModel::MM_VERTTEXCOORD;
     case FP_UV_WEDGE_TO_VERTEX : return MeshModel::MM_WEDGTEXCOORD;
-    case FP_BASIC_TRIANGLE_MAPPING : return MeshModel::MM_FACENUMBER;
+    case FP_UV_VERTEX_TO_WEDGE : return MeshModel::MM_VERTTEXCOORD;
+    case FP_BASIC_TRIANGLE_MAPPING :
+    case FP_VORONOI_ATLAS :
+    case FP_PLANAR_MAPPING : return MeshModel::MM_FACENUMBER;
     case FP_SET_TEXTURE : return MeshModel::MM_WEDGTEXCOORD;
     case FP_COLOR_TO_TEXTURE : return MeshModel::MM_VERTCOLOR | MeshModel::MM_WEDGTEXCOORD;
     case FP_TRANSFER_TO_TEXTURE : return MeshModel::MM_NONE;
@@ -98,9 +117,11 @@ int FilterTexturePlugin::getRequirements(QAction *a)
 {
     switch (ID(a))
     {
-    case FP_UV_TO_COLOR :
+    case FP_VORONOI_ATLAS :
     case FP_UV_WEDGE_TO_VERTEX :
+    case FP_UV_VERTEX_TO_WEDGE :
     case FP_BASIC_TRIANGLE_MAPPING :
+    case FP_PLANAR_MAPPING :
     case FP_SET_TEXTURE : return MeshModel::MM_NONE;
     case FP_COLOR_TO_TEXTURE : return MeshModel::MM_FACEFACETOPO;
     case FP_TRANSFER_TO_TEXTURE : return MeshModel::MM_NONE;
@@ -114,8 +135,10 @@ int FilterTexturePlugin::postCondition( QAction *a) const
 {
     switch (ID(a))
     {
-    case FP_UV_TO_COLOR : return MeshModel::MM_VERTCOLOR;
+    case FP_VORONOI_ATLAS : return MeshModel::MM_UNKNOWN;
     case FP_UV_WEDGE_TO_VERTEX : return MeshModel::MM_UNKNOWN;
+    case FP_UV_VERTEX_TO_WEDGE : return MeshModel::MM_WEDGTEXCOORD;
+    case FP_PLANAR_MAPPING : return MeshModel::MM_WEDGTEXCOORD;
     case FP_BASIC_TRIANGLE_MAPPING : return MeshModel::MM_WEDGTEXCOORD;
     case FP_SET_TEXTURE : return MeshModel::MM_UNKNOWN;
     case FP_COLOR_TO_TEXTURE : return MeshModel::MM_UNKNOWN;
@@ -133,9 +156,11 @@ FilterTexturePlugin::FilterClass FilterTexturePlugin::getClass(QAction *a)
 {
     switch(ID(a))
     {
-    case FP_UV_TO_COLOR : return FilterClass(MeshFilterInterface::VertexColoring + MeshFilterInterface::Texture);
+    case FP_VORONOI_ATLAS :
     case FP_UV_WEDGE_TO_VERTEX :
+    case FP_UV_VERTEX_TO_WEDGE :
     case FP_BASIC_TRIANGLE_MAPPING :
+    case FP_PLANAR_MAPPING :
     case FP_SET_TEXTURE :
     case FP_COLOR_TO_TEXTURE :
     case FP_TRANSFER_TO_TEXTURE : return MeshFilterInterface::Texture;
@@ -161,11 +186,16 @@ static QString extractFilenameWOExt(MeshModel* mm)
 void FilterTexturePlugin::initParameterSet(QAction *action, MeshDocument &md, RichParameterSet & parlst) 
 {
     switch(ID(action)) {
-    case FP_UV_TO_COLOR :
-        parlst.addParam(new RichEnum("colorspace", 0, QStringList("Red-Green") << "Hue-Saturation", "Color Space", "The color space used to mapping UV to"));
+    case FP_VORONOI_ATLAS :
+      parlst.addParam(new RichInt("regionNum", 10, "Approx. Region Num", "An estimation of the number of regions that must be generated. Smaller regions could lead to parametrizations with smaller distortion."));
+      parlst.addParam(new RichBool("overlapFlag", false, "Overlap", "If checked the resulting parametrization will be composed by <i>overlapping</i> regions, e.g. the resulting mesh will have duplicated faces: each region will have a ring of ovelapping duplicate faces that will ensure that border regions will be parametrized in the atlas twice. This is quite useful for building mipmap robust atlases"));
         break;
     case FP_UV_WEDGE_TO_VERTEX :
         break;
+    case FP_PLANAR_MAPPING :
+      parlst.addParam(new RichEnum("projectionPlane", 0, QStringList("XY") << "XZ"<<"YZ","Projection plane","Choose the projection plane"));
+      parlst.addParam(new RichBool("aspectRatio", false, "Preserve Ratio", "If checked the resulting parametrization will preserve the original apsect ratio of the model otherwise it will fill up the whole 0..1 uv space"));
+      break;
     case FP_BASIC_TRIANGLE_MAPPING :
         parlst.addParam(new RichInt("sidedim", 0, "Quads per line", "Indicates how many triangles have to be put on each line (every quad contains two triangles)\nLeave 0 for automatic calculation"));
         parlst.addParam(new RichInt("textdim", 1024, "Texture Dimension (px)", "Gives an indication on how big the texture is"));
@@ -196,9 +226,9 @@ void FilterTexturePlugin::initParameterSet(QAction *action, MeshDocument &md, Ri
             parlst.addParam(new RichMesh ("sourceMesh",md.mm(),&md, "Source Mesh",
                                           "The mesh that contains the source data that we want to transfer"));
             parlst.addParam(new RichMesh ("targetMesh",md.mm(),&md, "Target Mesh",
-                                          "The mesh whose texture will be filled according to source mesh texture or vertex color"));
-            parlst.addParam(new RichEnum("data", 0, QStringList("Vertex Color") << "Texture Color", "Color Data Source",
-                                         "Choose to transfer color information from source mesh texture or vertex color"));
+                                          "The mesh whose texture will be filled according to source mesh data"));
+            parlst.addParam(new RichEnum("AttributeEnum", 0, QStringList("Vertex Color")  << "Vertex Normal" << "Vertex Quality"<< "Texture Color", "Color Data Source",
+                                         "Choose what attribute has to be transferred onto the target texture. You can choos bettween Per vertex attributes (clor,normal,quality) or to transfer color information from source mesh texture"));
             parlst.addParam(new RichAbsPerc("upperBound", md.mm()->cm.bbox.Diag()/50.0, 0.0f, md.mm()->cm.bbox.Diag(),
                                             tr("Max Dist Search"), tr("Sample points for which we do not find anything whithin this distance are rejected and not considered for recovering data")));
             parlst.addParam(new RichString("textName", fileName, "Texture file", "The texture file to be created"));
@@ -257,7 +287,7 @@ inline int getLongestEdge(const CMeshO::FaceType & f)
     return res;
 }
 
-typedef vcg::Triangle2<CMeshO::FaceType::TexCoordType::ScalarType> Tri2;
+typedef Triangle2<CMeshO::FaceType::TexCoordType::ScalarType> Tri2;
 
 inline void buildTrianglesCache(std::vector<Tri2> &arr, int maxLevels, float border, float quadSize, int idx=-1)
 {
@@ -309,49 +339,55 @@ T log_2(const T num)
 }
 
 // The Real Core Function doing the actual mesh processing.
-bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichParameterSet &par, vcg::CallBackPos *cb)
+bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichParameterSet &par, CallBackPos *cb)
 {
     MeshModel &m=*(md.mm());
     switch(ID(filter))     {
-    case FP_UV_TO_COLOR : {
-            int vcount = m.cm.vert.size();
-            int colsp = par.getEnum("colorspace");
-            if (!m.hasDataMask(MeshModel::MM_VERTCOLOR))
-                m.updateDataMask(MeshModel::MM_VERTCOLOR);
-            for (int i=0; i<vcount; ++i)
-            {
-                CMeshO::VertexType &v = m.cm.vert[i];
-                if (!v.IsD())
-                {
-                    // 'Normalized' 0..1
-                    float normU, normV;
-                    normU = v.T().U() - (int)v.T().U();
-                    if (normU < 0.) normU += 1.;
-                    normV = v.T().V() - (int)v.T().V();
-                    if (normV < 0.) normV += 1.;
+    case FP_VORONOI_ATLAS : {
+      MeshModel *baseModel=md.mm();
+      baseModel->updateDataMask(MeshModel::MM_FACEFACETOPO);
+      tri::UpdateTopology<CMeshO>::FaceFace(baseModel->cm);
+      int nonManifVertNum=tri::Clean<CMeshO>::CountNonManifoldVertexFF(baseModel->cm,false);
+      int nonManifEdgeNum=tri::Clean<CMeshO>::CountNonManifoldEdgeFF(baseModel->cm,false);
 
-                    switch(colsp) {
-                        // Red-Green color space
-                    case 0 : v.C() = vcg::Color4b((int)floor((normU*255)+0.5), (int)floor((normV*255)+0.5), 0, 255); break;
-                        // Hue-Saturation color space
-                    case 1 : {
-                            vcg::Color4f c;
-                            c.SetHSVColor(normU, normV, 1.0);
-                            v.C().Import(c);
-                        }
-                        break;
-                    default : assert(0);
-                };
-                }
-                cb(i*100/vcount, "Colorizing vertices from UV coordinates ...");
-            }
-        }
-        break;
+      if(nonManifVertNum>0 || nonManifEdgeNum>0)
+      {
+        Log("Mesh is not manifold\n:%i non manifold Vertices\n%i nonmanifold Edges\n",nonManifVertNum,nonManifEdgeNum);
+        this->errorMessage = "Mesh is not manifold. See Log for details";
+        return false;
+      }
+
+      MeshModel *paraModel=md.addNewMesh("","VoroAtlas",false);
+      tri::VoronoiAtlas<CMeshO>::VoronoiAtlasParam pp;
+      pp.sampleNum =par.getInt("regionNum");
+      pp.overlap=par.getBool("overlapFlag");
+
+      paraModel->updateDataMask(MeshModel::MM_WEDGTEXCOORD);
+      // Note that CMesh has ocf texcoord and the append inside VoronoiAtlas class need that.
+      // This is a design bug of the VCGLib...
+      int bitToBeCleared =0;
+      if(!baseModel->hasDataMask(MeshModel::MM_WEDGTEXCOORD)) bitToBeCleared |=MeshModel::MM_WEDGTEXCOORD;
+      if(!baseModel->hasDataMask(MeshModel::MM_VERTTEXCOORD)) bitToBeCleared |=MeshModel::MM_VERTTEXCOORD;
+      baseModel->updateDataMask(MeshModel::MM_WEDGTEXCOORD | MeshModel::MM_VERTTEXCOORD);
+      tri::VoronoiAtlas<CMeshO>::Build(baseModel->cm,paraModel->cm, pp);
+      if(pp.overlap==false)
+        tri::Clean<CMeshO>::RemoveDuplicateVertex(paraModel->cm);
+
+      tri::UpdateNormals<CMeshO>::PerVertexPerFace(paraModel->cm);
+
+      baseModel->clearDataMask(bitToBeCleared);
+      Log("Voronoi Atlas: Completed Processing in %i iterations",pp.vas.iterNum);
+      Log("Asked %i generated %i regions",pp.sampleNum,pp.vas.regionNum);
+      Log("Unwrap Time   %6.3f s", float(pp.vas.unwrapTime) / CLOCKS_PER_SEC);
+      Log("Voronoi Time  %6.3f s", float(pp.vas.voronoiTime) / CLOCKS_PER_SEC);
+      Log("Sampling Time %6.3f s", float(pp.vas.samplingTime) / CLOCKS_PER_SEC);
+      }
+      break;
 
     case FP_UV_WEDGE_TO_VERTEX : {
             int vn = m.cm.vn;
             m.updateDataMask(MeshModel::MM_VERTTEXCOORD);
-            vcg::tri::AttributeSeam::SplitVertex(m.cm, ExtractVertex, CompareVertex);
+            tri::AttributeSeam::SplitVertex(m.cm, ExtractVertex, CompareVertex);
             if (m.cm.vn != vn)
             {
                 m.clearDataMask(MeshModel::MM_FACEFACETOPO);
@@ -360,7 +396,28 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
         }
         break;
 
-    case FP_BASIC_TRIANGLE_MAPPING : {
+    case FP_UV_VERTEX_TO_WEDGE : {
+            m.updateDataMask(MeshModel::MM_WEDGTEXCOORD);
+            tri::UpdateTexture<CMeshO>::WedgeTexFromVertexTex(m.cm);
+        }
+        break;
+
+    case FP_PLANAR_MAPPING : {
+      m.updateDataMask(MeshModel::MM_WEDGTEXCOORD);
+      // Get Parameters
+      Point3f planeVec[3][2] = {
+        {Point3f(1,0,0),Point3f(0,1,0)},  // XY
+        {Point3f(0,0,1),Point3f(1,0,0)},  // XZ
+        {Point3f(0,1,0),Point3f(0,0,1)}   // YZ
+      };
+
+      int sideDim = par.getEnum("projectionPlane");
+      bool aspectRatio = par.getBool("aspectRatio");
+      tri::UpdateTexture<CMeshO>::WedgeTexFromPlane(m.cm, planeVec[sideDim][0], planeVec[sideDim][1], aspectRatio);
+    }
+    break;
+    case FP_BASIC_TRIANGLE_MAPPING :
+    {
             m.updateDataMask(MeshModel::MM_WEDGTEXCOORD);
 
             // Get Parameters
@@ -584,7 +641,7 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
 
             // Check textName and eventually add .png ext
             CheckError(std::max<int>(textName.lastIndexOf("\\"),textName.lastIndexOf("/")) != -1, "Path in Texture file not allowed");
-            if (!textName.endsWith(".png", Qt::CaseInsensitive))
+            if (!textName.endsWith(".png", Qt::CaseInsensitive) && !textName.endsWith(".dds", Qt::CaseInsensitive) )
                 textName.append(".png");
 
             // Creates path to texture file
@@ -657,13 +714,13 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
             img.fill(qRgba(0,0,0,0)); // transparent black
 
             // Compute (texture-space) border edges
-            vcg::tri::UpdateTopology<CMeshO>::FaceFaceFromTexCoord(m.cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceBorderFromFF(m.cm);
+            tri::UpdateTopology<CMeshO>::FaceFaceFromTexCoord(m.cm);
+            tri::UpdateFlags<CMeshO>::FaceBorderFromFF(m.cm);
 
             // Rasterizing triangles
             RasterSampler rs(img);
             rs.InitCallback(cb, m.cm.fn, 0, 80);
-            vcg::tri::SurfaceSampling<CMeshO,RasterSampler>::Texture(m.cm,rs,textW,textH,true);
+            tri::SurfaceSampling<CMeshO,RasterSampler>::Texture(m.cm,rs,textW,textH,true);
 
             // Revert alpha values for border edge pixels to 255
             cb(81, "Cleaning up texture ...");
@@ -679,12 +736,12 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
             if (pp)
             {
                 cb(85, "Filling texture holes...");
-                vcg::PullPush(img, qRgba(0,0,0,0));
+                PullPush(img, qRgba(0,0,0,0));
             }
 
             // Undo topology changes
-            vcg::tri::UpdateTopology<CMeshO>::FaceFace(m.cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceBorderFromFF(m.cm);
+            tri::UpdateTopology<CMeshO>::FaceFace(m.cm);
+            tri::UpdateFlags<CMeshO>::FaceBorderFromFF(m.cm);
 
             // Save texture
             cb(90, "Saving texture ...");
@@ -704,10 +761,14 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
     case FP_TRANSFER_TO_TEXTURE : {
             MeshModel *srcMesh = par.getMesh("sourceMesh");
             MeshModel *trgMesh = par.getMesh("targetMesh");
-            bool colorSampling;
-            switch (par.getEnum("data")) {
-            case 0: colorSampling = true; break;
-            case 1: colorSampling = false; break;
+            bool vertexSampling=false;
+            bool textureSampling=false;
+            int  vertexMode= -1;
+            switch (par.getEnum("AttributeEnum")) {
+            case 0: vertexSampling= true; vertexMode=0; break;  // Color
+            case 1: vertexSampling= true; vertexMode=1; break;  // Normal
+            case 2: vertexSampling= true; vertexMode=2; break;  // Quality
+            case 3: textureSampling = true; break;
             default: assert(0);
             }
             float upperbound = par.getAbsPerc("upperBound"); // maximum distance to stop search
@@ -727,9 +788,11 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
             // Source image (for texture to texture transfer)
             QImage srcImg;
 
-            if (colorSampling) {
-                CheckError(!srcMesh->hasDataMask(MeshModel::MM_VERTCOLOR), "Source mesh doesn't have Per Vertex Color");
-            } else {
+            if (vertexSampling) {
+                if(vertexMode==0) {CheckError(!srcMesh->hasDataMask(MeshModel::MM_VERTCOLOR), "Source mesh doesn't have Per Vertex Color");}
+                if(vertexMode==1) {CheckError(!srcMesh->hasDataMask(MeshModel::MM_VERTNORMAL), "Source mesh doesn't have Per Vertex Normal");}
+                if(vertexMode==2) {CheckError(!srcMesh->hasDataMask(MeshModel::MM_VERTQUALITY), "Source mesh doesn't have Per Vertex Quality");}
+            } else { assert(textureSampling);
                 // Check whether is possible to access source mesh texture
                 CheckError(!srcMesh->hasDataMask(MeshModel::MM_WEDGTEXCOORD), "Source mesh doesn't have Per Wedge Texture Coordinates");
                 CheckError(srcMesh->cm.textures.empty(), "Source mesh doesn't have any associated texture");
@@ -764,22 +827,21 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
 
             // Compute (texture-space) border edges
             trgMesh->updateDataMask(MeshModel::MM_FACEFACETOPO);
-            vcg::tri::UpdateTopology<CMeshO>::FaceFaceFromTexCoord(trgMesh->cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceBorderFromFF(trgMesh->cm);
+            tri::UpdateTopology<CMeshO>::FaceFaceFromTexCoord(trgMesh->cm);
+            tri::UpdateFlags<CMeshO>::FaceBorderFromFF(trgMesh->cm);
 
             // Rasterizing faces
             srcMesh->updateDataMask(MeshModel::MM_FACEMARK);
-            vcg::tri::UpdateNormals<CMeshO>::PerFaceNormalized(srcMesh->cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceProjection(srcMesh->cm);
-            if (colorSampling)
+            tri::UpdateNormals<CMeshO>::PerFaceNormalized(srcMesh->cm);
+            if (vertexSampling)
             {
-                TransferColorSampler sampler(srcMesh->cm, img, upperbound); // color sampling
+                TransferColorSampler sampler(srcMesh->cm, img, upperbound,vertexMode); // color sampling
                 sampler.InitCallback(cb, trgMesh->cm.fn, 0, 80);
-                vcg::tri::SurfaceSampling<CMeshO,TransferColorSampler>::Texture(trgMesh->cm,sampler,img.width(),img.height(),false);
-            } else {
+                tri::SurfaceSampling<CMeshO,TransferColorSampler>::Texture(trgMesh->cm,sampler,img.width(),img.height(),false);
+            } else { assert(textureSampling);
                 TransferColorSampler sampler(srcMesh->cm, img, &srcImg, upperbound); // texture sampling
                 sampler.InitCallback(cb, trgMesh->cm.fn, 0, 80);
-                vcg::tri::SurfaceSampling<CMeshO,TransferColorSampler>::Texture(trgMesh->cm,sampler,img.width(),img.height(),false);
+                tri::SurfaceSampling<CMeshO,TransferColorSampler>::Texture(trgMesh->cm,sampler,img.width(),img.height(),false);
             }
 
             // Revert alpha values from border edge pixel to 255
@@ -796,12 +858,12 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
             if (pp)
             {
                 cb(85, "Filling texture holes...");
-                vcg::PullPush(img, qRgba(0,0,0,0));
+                PullPush(img, qRgba(0,0,0,0));
             }
 
             // Undo topology changes
-            vcg::tri::UpdateTopology<CMeshO>::FaceFace(trgMesh->cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceBorderFromFF(trgMesh->cm);
+            tri::UpdateTopology<CMeshO>::FaceFace(trgMesh->cm);
+            tri::UpdateFlags<CMeshO>::FaceBorderFromFF(trgMesh->cm);
 
             // Save texture
             cb(90, "Saving texture ...");
@@ -841,13 +903,12 @@ bool FilterTexturePlugin::applyFilter(QAction *filter, MeshDocument &md, RichPar
             trgMesh->updateDataMask(MeshModel::MM_VERTCOLOR);
 
             srcMesh->updateDataMask(MeshModel::MM_FACEMARK);
-            vcg::tri::UpdateNormals<CMeshO>::PerFaceNormalized(srcMesh->cm);
-            vcg::tri::UpdateFlags<CMeshO>::FaceProjection(srcMesh->cm);
+            tri::UpdateNormals<CMeshO>::PerFaceNormalized(srcMesh->cm);
 
             // Colorizing vertices
             VertexSampler vs(srcMesh->cm, srcImg, upperbound);
             vs.InitCallback(cb, trgMesh->cm.vn);
-            vcg::tri::SurfaceSampling<CMeshO,VertexSampler>::VertexUniform(trgMesh->cm,vs,trgMesh->cm.vn);
+            tri::SurfaceSampling<CMeshO,VertexSampler>::VertexUniform(trgMesh->cm,vs,trgMesh->cm.vn);
         }
         break;
 
