@@ -32,9 +32,9 @@
 #include <vcg/simplex/face/topology.h>
 #include <vcg/simplex/face/pos.h>
 #include <vcg/simplex/face/jumping_pos.h>
-#include <vcg/complex/trimesh/append.h>
-#include <vcg/complex/trimesh/clean.h>
-#include <vcg/complex/trimesh/hole.h>
+#include <vcg/complex/append.h>
+#include <vcg/complex/algorithms/clean.h>
+#include <vcg/complex/algorithms/hole.h>
 #include <vcg/space/intersection2.h>
 #include <wrap/gl/glu_tesselator.h>
 
@@ -292,11 +292,11 @@ bool FilterZippering::isOnBorder( Point3f point, CMeshO::FacePointer f )  {
 	if ( f == 0 ) return false;	//null face
     
 	//compute barycentric coords
-	float bc[3];
-	InterpolationParameters<CMeshO::FaceType, float>( *f, f->N(), point, bc[0], bc[1], bc[2] );
+  Point3f bc;
+  InterpolationParameters<CMeshO::FaceType, float>( *f, f->N(), point, bc );
 	//search for max and min
-	int min_el = min_element(bc, bc+3) - bc;
-	int max_el = max_element(bc, bc+3) - bc;
+  int min_el = min_element(&bc[0], &bc[0]+3) - &bc[0];
+  int max_el = max_element(&bc[0], &bc[0]+3) - &bc[0];
 	//coords of max el = 1.0f -> check vertex
 	if ( bc[max_el] >= 1.0f - eps ) return isBorderVert( f, max_el );
 	//coords of min_el = 0.0f -> check edge
@@ -1519,10 +1519,8 @@ bool FilterZippering::applyFilter(QAction *filter, MeshDocument &md, RichParamet
 	tri::UnMarkAll(b->cm);
 
 	tri::UpdateNormals<CMeshO>::PerFaceNormalized(a->cm);
-	tri::UpdateFlags<CMeshO>::FaceProjection(a->cm);
 	tri::UpdateNormals<CMeshO>::PerVertexNormalized(a->cm);
 	tri::UpdateNormals<CMeshO>::PerFaceNormalized(b->cm);
-	tri::UpdateFlags<CMeshO>::FaceProjection(b->cm);
 	tri::UpdateNormals<CMeshO>::PerVertexNormalized(b->cm);
 	//fixed eps 
 	eps = 0.00001f;
@@ -1597,7 +1595,6 @@ bool FilterZippering::applyFilter(QAction *filter, MeshDocument &md, RichParamet
 		tri::UpdateTopology<CMeshO>::FaceFace(a->cm);
 		tri::UpdateFlags<CMeshO>::FaceClear(a->cm);
 		tri::UpdateNormals<CMeshO>::PerFaceNormalized(a->cm);   
-		tri::UpdateFlags<CMeshO>::FaceProjection(a->cm);  
 		tri::UpdateNormals<CMeshO>::PerVertexNormalized(a->cm);
 
 		//create grid on mesh A
