@@ -35,9 +35,9 @@ using namespace vcg;
 
 void MeshShaderRenderPlugin::initActionList() {
 
-	QAction * qaNone = new QAction("None", this); 
+	/*QAction * qaNone = new QAction("None", this); 
 	qaNone->setCheckable(false);
-	actionList << qaNone;
+	actionList << qaNone;*/
 
 	QDir shadersDir = QDir(qApp->applicationDirPath());
 #if defined(Q_OS_WIN)
@@ -316,10 +316,16 @@ void MeshShaderRenderPlugin::Init(QAction *a, MeshDocument &/*md*/, RenderMode &
 				
 
 					std::vector<TextureInfo>::iterator tIter = shaders[a->text()].textureInfo.begin();
-					while (tIter != shaders[a->text()].textureInfo.end()) {
+					while (tIter != shaders[a->text()].textureInfo.end()) 
+					{
 						glEnable(tIter->Target);
 						QImage img, imgScaled, imgGL;
-						img.load(tIter->path);
+						bool opened = img.load(tIter->path);
+						if (!opened)
+						{
+							supported = false;
+							return;
+						}
 						// image has to be scaled to a 2^n size. We choose the first 2^N <= picture size.
 						int bestW=pow(2.0,floor(::log(double(img.width() ))/::log(2.0)));
 						int bestH=pow(2.0,floor(::log(double(img.height()))/::log(2.0)));
@@ -457,9 +463,15 @@ void MeshShaderRenderPlugin::Render(QAction *a, MeshDocument &md, RenderMode &rm
 	glGetError();
 	foreach(MeshModel * mp, md.meshList)
 				{
-					if(mp->visible) mp->Render(rm.drawMode,rm.colorMode,rm.textureMode);
+					if(mp->visible) mp->render(rm.drawMode,rm.colorMode,rm.textureMode);
 				}
 	glUseProgramObjectARB(0);
 }
 
+void MeshShaderRenderPlugin::Finalize( QAction*, MeshDocument*, GLArea* )
+{
+	delete sDialog;
+	sDialog = 0;
+}
+
 Q_EXPORT_PLUGIN(MeshShaderRenderPlugin)
