@@ -21,16 +21,11 @@
 *                                                                           *
 ****************************************************************************/
 
-#include <QApplication>
+#include <common/mlapplication.h>
 #include <common/interfaces.h>
 #include <common/pluginmanager.h>
 #include <common/filterscript.h>
 
-#include <wrap/io_trimesh/io_mask.h>
-#include <vcg/complex/trimesh/update/normal.h>
-#include <vcg/complex/trimesh/update/bounding.h>
-#include <vcg/complex/trimesh/clean.h>
-
 class FilterData
 {
 public:
@@ -347,7 +342,7 @@ private:
 int main(int argc, char *argv[])
 {
   FILE *logfp=stdout;
-	QApplication app(argc, argv);  
+	MeshLabApplication app(argc, argv);  
 	MeshLabServer server;
 	MeshDocument meshDocument;
 	QStringList meshNamesIn, meshNamesOut;
