@@ -20,80 +20,7 @@
 * for more details.                                                         *
 *                                                                           *
 ****************************************************************************/
-/****************************************************************************
-  History
 
-$Log: not supported by cvs2svn $
-Revision 1.21  2008/02/04 21:26:45  ganovelli
-added ImportData which imports all local attributes into vertexplus and faceplus.
-A local attribute is everything (N(), C(), Q()....) except pointers to other simplices
-(i.e. FFAdj, VFAdj, VertexRef) which are set to NULL.
-Added some function for const attributes
-
-Revision 1.20  2008/01/28 08:42:51  cignoni
-added assert when writing on empty data members
-
-Revision 1.19  2008/01/19 17:49:05  ganovelli
-missing const  cVF added
-
-Revision 1.18  2007/11/20 09:43:53  ganovelli
-added missing include to color4
-
-Revision 1.17  2007/05/04 16:16:04  ganovelli
-added include to texcoor2
-
-Revision 1.16  2007/03/12 15:42:11  tarini
-Texture coord name change!  "TCoord" and "Texture" are BAD. "TexCoord" is GOOD.
-
-Revision 1.15  2007/03/12 15:37:19  tarini
-Texture coord name change!  "TCoord" and "Texture" are BAD. "TexCoord" is GOOD.
-
-Revision 1.14  2007/02/27 09:32:00  cignoni
-Added constructor to the VFadj component to comply to the allocator needs
-
-Revision 1.13  2007/02/12 19:01:23  ganovelli
-added Name(std:vector<std::string>& n) that fills n with the names of the attribute of the face type
-
-Revision 1.12  2007/01/11 10:22:39  cignoni
-Added intialization of vertexRef to 0.
-
-Revision 1.11  2006/12/06 00:08:57  cignoni
-Added FFp1 and FFp2 shortcuts
-
-Revision 1.10  2006/12/04 11:00:02  ganovelli
-Cambiate Has*Opt in Has*Occ e aggiunti typedef per la compilazione di Occ
-
-Revision 1.9  2006/11/28 22:34:28  cignoni
-Added default constructor with null initialization to adjacency members.
-AddFaces and AddVertices NEED to know if the topology is correctly computed to update it.
-
-Revision 1.8  2006/10/07 09:59:42  cignoni
-Added missing const to EmptyFF
-
-Revision 1.7  2006/01/09 13:58:55  cignoni
-Added Initialization of Color in Vertex and Face Components
-
-Revision 1.6  2005/11/22 15:49:39  cignoni
-removed two spurious computenormal
-
-Revision 1.5  2005/11/21 21:44:47  cignoni
-Moved ComputeNormal and ComputeNormalizedNormal out of the face class (no more a member function!)
-
-Revision 1.4  2005/11/18 15:44:49  cignoni
-Access to constant normal changed from by val to by reference
-
-Revision 1.3  2005/11/16 22:58:17  cignoni
-Added IncrementalMark and WedgeTexCoord
-Standardized name of flags. It is plural becouse each simplex has many flag.
-
-Revision 1.2  2005/11/12 18:43:14  cignoni
-added missing cFFi
-
-Revision 1.1  2005/10/14 15:07:58  cignoni
-First Really Working version
-
-
-****************************************************************************/
 #ifndef __VCG_FACE_PLUS_COMPONENT
 #define __VCG_FACE_PLUS_COMPONENT
 
@@ -110,32 +37,130 @@ Some naming Rules
 All the Components that can be added to a vertex should be defined in the namespace vert:
 
 */
+/*------------------------- EMPTY CORE COMPONENTS -----------------------------------------*/
 
-/*-------------------------- VERTEX ----------------------------------------*/ 
-template <class T> class EmptyVertexRef: public T {
+template <class T> class EmptyCore: public T {
 public:
- // typedef typename T::VertexType VertexType;
- // typedef typename T::CoordType CoordType;
   inline typename T::VertexType *       & V( const int ) 	    {	assert(0);		static typename T::VertexType *vp=0; return vp; }
-  inline typename T::VertexType * const & V( const int ) const {	assert(0);		static typename T::VertexType *vp=0; return vp; }
   inline typename T::VertexType * cV( const int ) const {	assert(0);		static typename T::VertexType *vp=0; return vp;	}
-
-	inline typename T::VertexType *       & FVp( const int i )				{	return this->V(i); }
-	inline typename T::VertexType * const & FVp( const int i) const		{	return this->V(i); }
-	inline typename T::VertexType * cFVp( const int i ) const					{	return this->cV(i); }
-
+  inline typename T::VertexType *       & FVp( const int i )				{	return this->V(i); }
+  inline typename T::VertexType * cFVp( const int i ) const					{	return this->cV(i); }
   inline typename T::CoordType & P( const int ) 	    {	assert(0);		static typename T::CoordType coord(0, 0, 0); return coord;	}
-  inline const typename T::CoordType & P( const int ) const {	assert(0);		static typename T::CoordType coord(0, 0, 0); return coord;	}
   inline const typename T::CoordType &cP( const int ) const	{	assert(0);		static typename T::CoordType coord(0, 0, 0); return coord;	}
-	template <class RightF>
-	void ImportData(const RightF & rightF) {T::ImportData(rightF);}
-	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-  static bool HasVertexRef()   { return false; }
-	static bool HasFVAdjacency()   { return false; }
-	static void Name(std::vector<std::string> & name){T::Name(name);}
 
-};
+  static bool HasVertexRef()   { return false; }
+  static bool HasFVAdjacency()   { return false; }
+
+  typedef typename T::VertexType::NormalType NormalType;
+  NormalType &N() { static NormalType dummy_normal(0, 0, 0);  assert(0); return dummy_normal; }
+  const NormalType &cN() const { static NormalType dummy_normal(0, 0, 0); return dummy_normal; }
+  NormalType &WN(int) { static NormalType dummy_normal(0, 0, 0);  assert(0); return dummy_normal; }
+  const NormalType cWN(int) const { static NormalType dummy_normal(0, 0, 0); return dummy_normal; }
+
+  static bool HasWedgeNormal()   { return false; }
+  static bool HasFaceNormal()   { return false; }
+  static bool HasWedgeNormalOcf()   { return false; }
+  static bool HasFaceNormalOcf()   { return false; }
+
+
+  typedef int WedgeTexCoordType;
+  typedef vcg::TexCoord2<float,1> TexCoordType;
+  TexCoordType &WT(const int) { static TexCoordType dummy_texture;  assert(0); return dummy_texture;}
+  TexCoordType const &cWT(const int) const { static TexCoordType dummy_texture; return dummy_texture;}
+
+  static bool HasWedgeTexCoord()   { return false; }
+  static bool HasWedgeTexCoordOcf()   { return false; }
+
+  int &Flags() { static int dummyflags(0);  assert(0); return dummyflags; }
+  int Flags() const { return 0; }
+  static bool HasFlags()   { return false; }
+  static bool HasFlagsOcf()   { return false; }
+
+  inline void InitIMark()    {  }
+  inline int & IMark()       { assert(0); static int tmp=-1; return tmp;}
+  inline int IMark() const {return 0;}
+
+  typedef int MarkType;
+  typedef float QualityType;
+  typedef Point3f Quality3Type;
+  typedef vcg::Color4b ColorType;
+
+  ColorType &C() { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
+  const ColorType &cC() const { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
+  ColorType &WC(const int) { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
+  const ColorType &cWC(const int) const { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
+  QualityType &Q() { static QualityType dummyQuality(0);  assert(0); return dummyQuality; }
+  const QualityType &cQ() const { static QualityType dummyQuality(0);  assert(0); return dummyQuality; }
+  Quality3Type &Q3() { static Quality3Type dummyQuality3(0,0,0);  assert(0); return dummyQuality3; }
+  const Quality3Type &cQ3() const { static Quality3Type dummyQuality3(0,0,0);  assert(0); return dummyQuality3; }
+
+  static bool HasFaceColor()   { return false; }
+  static bool HasFaceColorOcf() { return false;}
+
+  static bool HasWedgeColor()   { return false; }
+  static bool HasWedgeColorOcf()   { return false; }
+
+  static bool HasFaceQuality()   { return false; }
+  static bool HasFaceQualityOcf() { return false;}
+
+  static bool HasFaceQuality3()   { return false; }
+  static bool HasFaceQuality3Ocf() { return false;}
+
+  static bool HasMark()   { return false; }
+  static bool HasMarkOcf()   { return false; }
+
+  typedef int VFAdjType;
+  typename T::FacePointer       &VFp(const int)       { static typename T::FacePointer fp=0;  assert(0); return fp; }
+  typename T::FacePointer const cVFp(const int) const { static typename T::FacePointer const fp=0; return fp; }
+  typename T::FacePointer       &FFp(const int)       { static typename T::FacePointer fp=0;  assert(0); return fp; }
+  typename T::FacePointer const cFFp(const int) const { static typename T::FacePointer const fp=0; return fp; }
+  typename T::EdgePointer       &FEp(const int)       { static typename T::EdgePointer fp=0;  assert(0); return fp; }
+  typename T::EdgePointer const cFEp(const int) const { static typename T::EdgePointer const fp=0; return fp; }
+  typename T::HEdgePointer       &FHp()       { static typename T::HEdgePointer fp=0;  assert(0); return fp; }
+  typename T::HEdgePointer const cFHp() const { static typename T::HEdgePointer const fp=0; assert(0);return fp; }
+  char &VFi(const int j){(void)j; static char z=0;  assert(0); return z;}
+  char &FFi(const int j){(void)j; static char z=0;  assert(0); return z;}
+  const char &cVFi(const int j){(void)j; static char z=0; return z;}
+  const char &cFFi(const int j) const {(void)j; static char z=0; return z;}
+
+  static bool HasVFAdjacency()   {   return false; }
+  static bool HasFFAdjacency()   {   return false; }
+  static bool HasFEAdjacency()   {   return false; }
+  static bool HasFHAdjacency()   {   return false; }
+
+  static bool HasFFAdjacencyOcf()   {   return false; }
+  static bool HasVFAdjacencyOcf()   {   return false; }
+  static bool HasFEAdjacencyOcf()   {   return false; }
+  static bool HasFHAdjacencyOcf()   {   return false; }
+
+  typedef int CurvatureDirType;
+
+  Point3f &PD1(){static Point3f dummy(0,0,0); return dummy;}
+  Point3f &PD2(){static Point3f dummy(0,0,0); return dummy;}
+  const Point3f &cPD1() const {static Point3f dummy(0,0,0); return dummy;}
+  const Point3f &cPD2()const {static Point3f dummy(0,0,0); return dummy;}
+
+  float &K1(){ static float dummy(0);assert(0);return dummy;}
+  float &K2(){ static float dummy(0);assert(0);return dummy;}
+  const float &cK1()const { static float dummy(0);assert(0);return dummy;}
+  const float &cK2()const { static float dummy(0);assert(0);return dummy;}
+
+  static bool HasCurvatureDir()   { return false; }
+  static bool HasCurvatureDirOcf()   { return false; }
+
+  inline void SetVN(const int & /*n*/) {assert(0);}
+
+  static bool HasPolyInfo()   { return false; }
+
+  template <class RightF>
+  void ImportData(const RightF & rightF) {T::ImportData(rightF);}
+  inline void Alloc(const int & ns){T::Alloc(ns);}
+  inline void Dealloc(){T::Dealloc();}
+  static void Name(std::vector<std::string> & name){T::Name(name);}
+  };
+
+  /*-------------------------- VertexRef ----------------------------------------*/
+
 template <class T> class VertexRef: public T {
 public:
 	VertexRef(){
@@ -159,12 +184,12 @@ public:
 	/** Return the pointer to the ((j+1)%3)-th vertex of the face.
 		@param j Index of the face vertex.
 	 */
-  inline typename T::VertexType *      &  V0( const int j )       { return V(j);}
-  inline typename T::VertexType *      &  V1( const int j )       { return V((j+1)%3);}
-  inline typename T::VertexType *      &  V2( const int j )       { return V((j+2)%3);}
-  inline typename T::VertexType * const   V0( const int j ) const { return V(j);}
-  inline typename T::VertexType * const   V1( const int j ) const { return V((j+1)%3);}
-  inline typename T::VertexType * const   V2( const int j ) const { return V((j+2)%3);}
+  inline typename T::VertexType * & V0( const int j )       { return V(j);}
+  inline typename T::VertexType * & V1( const int j )       { return V((j+1)%3);}
+  inline typename T::VertexType * & V2( const int j )       { return V((j+2)%3);}
+  inline typename T::VertexType *   V0( const int j ) const { return V(j);}
+  inline typename T::VertexType *   V1( const int j ) const { return V((j+1)%3);}
+  inline typename T::VertexType *   V2( const int j ) const { return V((j+2)%3);}
   inline typename T::VertexType *  cV0( const int j ) const { return cV(j);}
   inline typename T::VertexType *  cV1( const int j ) const { return cV((j+1)%3);}
   inline typename T::VertexType *  cV2( const int j ) const { return cV((j+2)%3);}
@@ -201,56 +226,11 @@ public:
   typename T::VertexType *v[3];
 };
 
-
-
-/*-------------------------- NORMAL ----------------------------------------*/ 
-
-template <class T> class EmptyNormal: public T {
-public:
-  //typedef vcg::Point3s NormalType;
-  typedef typename T::VertexType::NormalType NormalType;
-  NormalType &N() { static NormalType dummy_normal(0, 0, 0);  assert(0); return dummy_normal; }
-  const NormalType &cN() const { static NormalType dummy_normal(0, 0, 0); return dummy_normal; }
-  NormalType &WN(int) { static NormalType dummy_normal(0, 0, 0);  assert(0); return dummy_normal; }
-  const NormalType cWN(int) const { static NormalType dummy_normal(0, 0, 0); return dummy_normal; }
-
-	template <class RightF>
-	void ImportData(const RightF & rightF){ T::ImportData(rightF);}
-  static bool HasWedgeNormal()   { return false; }
-  static bool HasFaceNormal()   { return false; }
-  static bool HasWedgeNormalOcc()   { return false; }
-  static bool HasFaceNormalOcc()   { return false; }
-  static bool HasWedgeNormalOcf()   { return false; }
-  static bool HasFaceNormalOcf()   { return false; }
-//  void ComputeNormal() {assert(0);}
-//  void ComputeNormalizedNormal() {assert(0);}
-	static void Name(std::vector<std::string> & name){ T::Name(name);}
-
-};
-template <class T> class NormalFromVert: public T {
-public:
-  typedef typename T::VertexType::NormalType NormalType;
-  NormalType &N() { return _norm; }
-  NormalType &cN() const { return _norm; }
-	template <class RightF>
-	void ImportData(const RightF & rightF){ N() = rightF.cN(); T::ImportData(rightF);}
- 	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-  static bool HasFaceNormal()   { return true; }
-//  void ComputeNormal() {	_norm = vcg::Normal<typename T::FaceType>(*(static_cast<typename T::FaceType *>(this))); }
-//  void ComputeNormalizedNormal() {	_norm = vcg::NormalizedNormal(*this);}  
-	static void Name(std::vector<std::string> & name){name.push_back(std::string("NormalFromVert"));T::Name(name);}
-
-private:
-  NormalType _norm;    
-};
-
+template <class T>
+void ComputeNormal(T &f) {	f.N().Import(vcg::Normal<T>(f)); }
 
 template <class T>
-void ComputeNormal(T &f) {	f.N() = vcg::Normal<T>(f); }
-
-template <class T>
-void ComputeNormalizedNormal(T &f) {	f.N() = vcg::NormalizedNormal<T>(f); }
+void ComputeNormalizedNormal(T &f) {	f.N().Import(vcg::NormalizedNormal<T>(f)); }
 
 template <class A, class T> class NormalAbs: public T {
 public:
@@ -307,14 +287,11 @@ private:
 
 template <class TT> class WedgeRealNormal3s: public WedgeRealNormal<vcg::Point3s, TT> {
 public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeRealNormal2s"));TT::Name(name);}};
-
 template <class TT> class WedgeRealNormal3f: public WedgeRealNormal<vcg::Point3f, TT> {
 public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeRealNormal2f"));TT::Name(name);}};
-
 template <class TT> class WedgeRealNormal3d: public WedgeRealNormal<vcg::Point3d, TT> {
 public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeRealNormal2d"));TT::Name(name);}};
 
-
 template <class T> class Normal3s: public NormalAbs<vcg::Point3s, T> {
 public:static void Name(std::vector<std::string> & name){name.push_back(std::string("Normal3s"));T::Name(name);}
 };
@@ -328,21 +305,6 @@ public: static void Name(std::vector<std::string> & name){name.push_back(std::st
 
 /*-------------------------- TexCoord ----------------------------------------*/ 
 
-template <class T> class EmptyWedgeTexCoord: public T {
-public:
-	typedef int WedgeTexCoordType;
-  typedef vcg::TexCoord2<float,1> TexCoordType;
-  TexCoordType &WT(const int) { static TexCoordType dummy_texture;  assert(0); return dummy_texture;}
-  TexCoordType const &cWT(const int) const { static TexCoordType dummy_texture; return dummy_texture;}
-	template <class RightF>
-	void ImportData(const RightF & rightF){ T::ImportData(rightF);}
-	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-  static bool HasWedgeTexCoord()   { return false; }
-  static bool HasWedgeTexCoordOcc()   { return false; }
-  static void Name(std::vector<std::string> & name){T::Name(name);}
-
-};
 template <class A, class T> class WedgeTexCoord: public T {
 public:
 	typedef int WedgeTexCoordType;
@@ -370,22 +332,7 @@ template <class TT> class WedgeTexCoord2d: public WedgeTexCoord<TexCoord2<double
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeTexCoord2d"));TT::Name(name);}
 };
 
-/*------------------------- FLAGS -----------------------------------------*/ 
-template <class T> class EmptyBitFlags: public T {
-public:
-	/// Return the vector of Flags(), senza effettuare controlli sui bit
-  int &Flags() { static int dummyflags(0);  assert(0); return dummyflags; }
-  int Flags() const { return 0; }
-	template <class RightF>
-	void ImportData(const RightF & rightF){ T::ImportData(rightF);}
-	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-  static bool HasFlags()   { return false; }
-  static bool HasFlagsOcc()   { return false; }
-	static void Name(std::vector<std::string> & name){T::Name(name);}
-
-};
-
+/*------------------------- BitFlags -----------------------------------------*/
 template <class T> class BitFlags:  public T {
 public:
   BitFlags(){_flags=0;}
@@ -404,47 +351,7 @@ private:
   int  _flags;    
 };
 
-/*-------------------------- COLOR ----------------------------------*/ 
-
-template <class T> class EmptyColorMarkQuality: public T {
-public:
-	typedef int MarkType;
-	inline void InitIMark()    {  }
-	inline int & IMark()       { assert(0); static int tmp=-1; return tmp;}
-	inline int IMark() const {return 0;}
-
-	typedef float QualityType;
-	typedef vcg::Color4b ColorType;
-	ColorType &C() { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
-	const ColorType &cC() const { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
-	ColorType &WC(const int) { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
-	const ColorType &cWC(const int) const { static ColorType dumcolor(vcg::Color4b::White);  assert(0); return dumcolor; }
-	QualityType &Q() { static QualityType dummyQuality(0);  assert(0); return dummyQuality; }
-	const QualityType &cQ() const { static QualityType dummyQuality(0);  assert(0); return dummyQuality; }
-  
-	static bool HasFaceColor()   { return false; }
-	static bool HasFaceColorOcc() { return false;}
-	static bool HasFaceColorOcf() { return false;}
-
-	static bool HasWedgeColor()   { return false; }
-	static bool HasWedgeColorOcc()   { return false; }
-	static bool HasWedgeColorOcf()   { return false; }
-
-	static bool HasFaceQuality()   { return false; }
-	static bool HasFaceQualityOcc()   { return false; }
-	static bool HasFaceQualityOcf() { return false;}
-
-	static bool HasMark()   { return false; }
-	static bool HasMarkOcc()   { return false; }
-	static bool HasMarkOcfb()   { return false; }
-  
-	static void Name(std::vector<std::string> & name){T::Name(name);}
-	template <class RightF>
-	void ImportData(const RightF & rightF){ T::ImportData(rightF);}
-	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-
-};
+/*-------------------------- Color Mark Quality  ----------------------------------*/
 template <class A, class T> class Color: public T {
 public:
   typedef A ColorType;
@@ -481,17 +388,14 @@ private:
 template <class T> class WedgeColor4b: public WedgeColor<vcg::Color4b, T> {
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeColor4b"));T::Name(name);}
 };
-
 template <class T> class WedgeColor4f: public WedgeColor<vcg::Color4f, T> {
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeColor4f"));T::Name(name);}
 };
-
 template <class T> class Color4b: public Color<vcg::Color4b, T> { public:
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("Color4b"));T::Name(name);}
 };
 
 /*-------------------------- Quality  ----------------------------------*/ 
-
 template <class A, class T> class Quality: public T {
 public:
   typedef A QualityType;
@@ -517,12 +421,38 @@ public:  static void Name(std::vector<std::string> & name){name.push_back(std::s
 template <class T> class Qualityd: public Quality<double, T> {
 public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("Qualityd"));T::Name(name);}
 };
-/*-------------------------- INCREMENTAL MARK  ----------------------------------------*/ 
 
+/*-------------------------- Quality3  ----------------------------------*/
+template <class A, class T> class Quality3: public T {
+public:
+	typedef vcg::Point3<A> Quality3Type;
+  Quality3Type &Q3() { return _quality; }
+  const Quality3Type &cQ3() const { return _quality; }
+	template <class RightF>
+	void ImportData(const RightF & rightF){ if(RightF::HasFaceQuality3()) Q3() = rightF.cQ3();T::ImportData(rightF);}
+	inline void Alloc(const int & ns){T::Alloc(ns);}
+	inline void Dealloc(){T::Dealloc();}
+  static bool HasFaceQuality3()   { return true; }
+  static bool HasFaceQuality3Occ()   { return true; }
+  static void Name(std::vector<std::string> & name){name.push_back(std::string("Quality3"));T::Name(name);}
+private:
+  Quality3Type _quality;    
+};
+
+template <class T> class Quality3s: public Quality3<short, T> {
+public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("Quality3s"));T::Name(name);}
+};
+template <class T> class Quality3f: public Quality3<float, T> {
+public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("Quality3f"));T::Name(name);}
+};
+template <class T> class Quality3d: public Quality3<double, T> {
+public:  static void Name(std::vector<std::string> & name){name.push_back(std::string("Quality3d"));T::Name(name);}
+};
+
+/*-------------------------- INCREMENTAL MARK  ----------------------------------------*/ 
 template <class T> class Mark: public T {
 public:
   static bool HasMark()      { return true; }
-  static bool HasMarkOcc()   { return true; }
   inline void InitIMark()    { _imark = 0; }
   inline int & IMark()       { return _imark;}
   inline const int & IMark() const {return _imark;}
@@ -534,43 +464,56 @@ public:
 	int _imark;
 };
 
+/*-------------------------- Curvature Direction ----------------------------------*/
+  template <class S>
+  struct CurvatureDirBaseType{
+          typedef Point3<S> VecType;
+          typedef  S   ScalarType;
+          CurvatureDirBaseType () {}
+          Point3<S>max_dir,min_dir; // max and min curvature direction
+          S k1,k2;// max and min curvature values
+  };
 
-/*----------------------------- VFADJ ------------------------------*/ 
-
-
-template <class T> class EmptyAdj: public T {
+template <class A, class TT> class CurvatureDir: public TT {
 public:
-	typedef int VFAdjType;
-  typename T::FacePointer       &VFp(const int)       { static typename T::FacePointer fp=0;  assert(0); return fp; }
-  typename T::FacePointer const cVFp(const int) const { static typename T::FacePointer const fp=0; return fp; }
-  typename T::FacePointer       &FFp(const int)       { static typename T::FacePointer fp=0;  assert(0); return fp; }
-  typename T::FacePointer const cFFp(const int) const { static typename T::FacePointer const fp=0; return fp; }
-  typename T::EdgePointer       &FEp(const int)       { static typename T::EdgePointer fp=0;  assert(0); return fp; }
-  typename T::EdgePointer const cFEp(const int) const { static typename T::EdgePointer const fp=0; return fp; }
-	typename T::HEdgePointer       &FHp()       { static typename T::HEdgePointer fp=0;  assert(0); return fp; }
-	typename T::HEdgePointer const cFHp() const { static typename T::HEdgePointer const fp=0; assert(0);return fp; }
-	char &VFi(const int j){(void)j; static char z=0;  assert(0); return z;};
-  char &FFi(const int j){(void)j; static char z=0;  assert(0); return z;};
-  const char &cVFi(const int j){(void)j; static char z=0; return z;};
-  const char &cFFi(const int j) const {(void)j; static char z=0; return z;};
-	template <class RightF>
-	void ImportData(const RightF & rightF){ T::ImportData(rightF);}
-	inline void Alloc(const int & ns){T::Alloc(ns);}
-	inline void Dealloc(){T::Dealloc();}
-  static bool HasVFAdjacency()   {   return false; }
-  static bool HasFFAdjacency()   {   return false; }
-  static bool HasFEAdjacency()   {   return false; }
-	static bool HasFHAdjacency()   {   return false; }
+  typedef A CurvatureDirType;
+        typedef typename CurvatureDirType::VecType VecType;
+        typedef typename CurvatureDirType::ScalarType ScalarType;
 
-  static bool HasFFAdjacencyOcc()   {   return false; }
-  static bool HasVFAdjacencyOcc()   {   return false; }
-  static bool HasFEAdjacencyOcc()   {   return false; }
-	static bool HasFHAdjacencyOcc()   {   return false; }
+        VecType &PD1(){ return _curv.max_dir;}
+        VecType &PD2(){ return _curv.min_dir;}
+        const VecType &cPD1() const {return _curv.max_dir;}
+        const VecType &cPD2() const {return _curv.min_dir;}
 
-  static void Name(std::vector<std::string> & name){T::Name(name);}
+        ScalarType &K1(){ return _curv.k1;}
+        ScalarType &K2(){ return _curv.k2;}
+        const ScalarType &cK1() const {return _curv.k1;}
+        const ScalarType &cK2()const  {return _curv.k2;}
+        template < class LeftV>
+        void ImportData(const LeftV  & left ) {
+          if(LeftV::HasCurvatureDir()) {
+                PD1() = left.cPD1(); PD2() = left.cPD2();
+                K1()  = left.cK1();  K2()  = left.cK2();
+          }
+          TT::ImportData( left);
+        }
 
+        static bool HasCurvatureDir()   { return true; }
+        static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvatureDir"));TT::Name(name);}
+
+private:
+  CurvatureDirType _curv;
 };
 
+
+template <class T> class CurvatureDirf: public CurvatureDir<CurvatureDirBaseType<float>, T> {
+public:	static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvatureDirf"));T::Name(name);}
+};
+template <class T> class CurvatureDird: public CurvatureDir<CurvatureDirBaseType<double>, T> {
+public:	static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvatureDird"));T::Name(name);}
+};
+
+/*----------------------------- VFADJ ------------------------------*/ 
 template <class T> class VFAdj: public T {
 public:
 	VFAdj(){
@@ -596,7 +539,6 @@ private:
 };
 
 /*----------------------------- FFADJ ------------------------------*/ 
-
 template <class T> class FFAdj: public T {
 public:
 	FFAdj(){
@@ -630,7 +572,6 @@ private:
 
 
 /*----------------------------- FEADJ ------------------------------*/ 
-
 template <class T> class FEAdj: public T {
 public:
 	FEAdj(){
@@ -638,16 +579,16 @@ public:
 		_fep[1]=0;
 		_fep[2]=0;
 	}
-  typename T::FacePointer       &FEp(const int j)        { assert(j>=0 && j<3);  return _fep[j]; }
-  typename T::FacePointer const  FEp(const int j) const  { assert(j>=0 && j<3);  return _fep[j]; }
-  typename T::FacePointer const cFEp(const int j) const  { assert(j>=0 && j<3);  return _fep[j]; }
+  typename T::EdgePointer       &FEp(const int j)        { assert(j>=0 && j<3);  return _fep[j]; }
+  typename T::EdgePointer const  FEp(const int j) const  { assert(j>=0 && j<3);  return _fep[j]; }
+  typename T::EdgePointer const cFEp(const int j) const  { assert(j>=0 && j<3);  return _fep[j]; }
   char        &FEi(const int j)       { return _fei[j]; }
   const char &cFEi(const int j) const { return _fei[j]; }
 
-  typename T::FacePointer        &FEp1( const int j )       { return FEp((j+1)%3);}
-	typename T::FacePointer        &FEp2( const int j )       { return FEp((j+2)%3);}
-	typename T::FacePointer  const  FEp1( const int j ) const { return FEp((j+1)%3);}
-	typename T::FacePointer  const  FEp2( const int j ) const { return FEp((j+2)%3);}
+  typename T::EdgePointer        &FEp1( const int j )       { return FEp((j+1)%3);}
+	typename T::EdgePointer        &FEp2( const int j )       { return FEp((j+2)%3);}
+	typename T::EdgePointer  const  FEp1( const int j ) const { return FEp((j+1)%3);}
+	typename T::EdgePointer  const  FEp2( const int j ) const { return FEp((j+2)%3);}
 
 	template <class RightF>
 	void ImportData(const RightF & rightF){T::ImportData(rightF);}
@@ -657,13 +598,12 @@ public:
   static void Name(std::vector<std::string> & name){name.push_back(std::string("FEAdj"));T::Name(name);}
 
 private:
-  typename T::FacePointer _fep[3] ;    
+  typename T::EdgePointer _fep[3] ;    
   char _fei[3] ;    
 };
 
 
 /*----------------------------- FHADJ ------------------------------*/
-
 template <class T> class FHAdj: public T {
 public:
 	FHAdj(){_fh=0;}
