@@ -234,7 +234,8 @@ bool IsVFAdjacencyEnabled() const {return VFAdjacencyEnabled;}
 void EnableVFAdjacency() {
 	assert(VALUE_TYPE::HasVFAdjacencyOcf());
 	VFAdjacencyEnabled=true;
-	AV.resize((*this).size());
+	VFAdjType zero; zero._fp=0; zero._zp=-1;
+	AV.resize((*this).size(),zero);
 }
 
 void DisableVFAdjacency() {
@@ -357,8 +358,9 @@ public:
 
   static bool HasVFAdjacency()   {   return true; }
   static bool HasVFAdjacencyOcf()   {   return true; }
-	static bool IsVFAdjacencyEnabled(const typename T::VertexType *vp)   {return vp->Base().VFAdjacencyEnabled;}
+    static bool IsVFAdjacencyEnabled(const typename T::VertexType *vp)   {return vp->Base().VFAdjacencyEnabled;}
 
+   static void Name(std::vector<std::string> & name){name.push_back(std::string("VFAdjOcf"));T::Name(name);}
 private:
 };
 
@@ -388,12 +390,12 @@ public:
 	void ImportData(const LeftV & leftV){
 			if((*this).Base().NormalEnabled && leftV.Base().NormalEnabled ) // copy the data only if they are enabled in both vertices
 				N().Import(leftV.cN());
-			T::ImportData(leftV);}
+            T::ImportData(leftV);}
 };
 
-template <class T> class Normal3sOcf: public NormalOcf<vcg::Point3s, T> {};
-template <class T> class Normal3fOcf: public NormalOcf<vcg::Point3f, T> {};
-template <class T> class Normal3dOcf: public NormalOcf<vcg::Point3d, T> {};
+template <class T> class Normal3sOcf: public NormalOcf<vcg::Point3s, T> {public: static void Name(std::vector<std::string> & name){name.push_back(std::string("Normal3sOcf"));T::Name(name);}};
+template <class T> class Normal3fOcf: public NormalOcf<vcg::Point3f, T> {public: static void Name(std::vector<std::string> & name){name.push_back(std::string("Normal3fOcf"));T::Name(name);}};
+template <class T> class Normal3dOcf: public NormalOcf<vcg::Point3d, T> {public: static void Name(std::vector<std::string> & name){name.push_back(std::string("Normal3dOcf"));T::Name(name);}};
 
 ///*-------------------------- COLOR ----------------------------------*/
 
@@ -401,6 +403,7 @@ template <class A, class T> class ColorOcf: public T {
 public:
 	typedef A ColorType;
 	ColorType &C() { assert((*this).Base().ColorEnabled); return (*this).Base().CV[(*this).Index()]; }
+	const ColorType &C() const { return this->cC(); }
 	const ColorType &cC() const { assert((*this).Base().ColorEnabled); return (*this).Base().CV[(*this).Index()]; }
 	template <class LeftV>
 	void ImportData(const LeftV & leftV)
@@ -411,10 +414,12 @@ public:
 		}
 
 	static bool HasColor()   { return true; }
-	static bool HasColorOcf()   { assert(!T::HasColorOcf()); return true; }
+    static bool HasColorOcf()   { assert(!T::HasColorOcf()); return true; }
 };
 
-template <class T> class Color4bOcf: public ColorOcf<vcg::Color4b, T> {};
+template <class T> class Color4bOcf: public ColorOcf<vcg::Color4b, T> {
+    static void Name(std::vector<std::string> & name){name.push_back(std::string("Color4bOcf"));T::Name(name);}
+};
 
 ///*-------------------------- QUALITY  ----------------------------------*/
 
@@ -432,10 +437,12 @@ public:
 			T::ImportData(leftV);
 		}
 	static bool HasQuality()   { return true; }
-	static bool HasQualityOcf()   { assert(!T::HasQualityOcf()); return true; }
+    static bool HasQualityOcf()   { assert(!T::HasQualityOcf()); return true; }
 };
 
-template <class T> class QualityfOcf: public QualityOcf<float, T> {};
+template <class T> class QualityfOcf: public QualityOcf<float, T> {
+public: static void Name(std::vector<std::string> & name){name.push_back(std::string("QualityfOcf"));T::Name(name);}
+};
 
 
 ///*-------------------------- TEXTURE  ----------------------------------*/
@@ -444,6 +451,7 @@ template <class A, class TT> class TexCoordOcf: public TT {
 public:
   typedef A TexCoordType;
   TexCoordType &T() {  assert((*this).Base().TexCoordEnabled); return (*this).Base().TV[(*this).Index()]; }
+  const TexCoordType &T() const { return this->cT(); }
   const TexCoordType &cT() const {  assert((*this).Base().TexCoordEnabled); return (*this).Base().TV[(*this).Index()]; }
 	template < class LeftV>
 	void ImportData(const LeftV & leftV)
@@ -454,10 +462,12 @@ public:
 			TT::ImportData(leftV);
 		}
 	static bool HasTexCoord()   { return true; }
-	static bool HasTexCoordOcf()   { assert(!TT::HasTexCoordOcf()); return true; }
+    static bool HasTexCoordOcf()   { assert(!TT::HasTexCoordOcf()); return true; }
 };
 
-template <class T> class TexCoordfOcf: public TexCoordOcf<TexCoord2<float,1>, T> {};
+template <class T> class TexCoordfOcf: public TexCoordOcf<TexCoord2<float,1>, T> {
+public: static void Name(std::vector<std::string> & name){name.push_back(std::string("TexCoordfOcf"));T::Name(name);}
+};
 
 ///*-------------------------- MARK  ----------------------------------*/
 
@@ -484,7 +494,9 @@ public:
 	}
 	static bool HasMark()   { return true; }
 	static bool HasMarkOcf()   { return true; }
-	inline void InitIMark()    { IMark() = 0; }
+    inline void InitIMark()    { IMark() = 0; }
+    static void Name(std::vector<std::string> & name){name.push_back(std::string("IMark"));T::Name(name);}
+
 };
 
 
@@ -515,13 +527,13 @@ public:
 	static bool IsCurvatureEnabled(const typename TT::VertexType *v)   { return v->Base().CurvatureEnabled; }
 
 	static bool HasCurvatureOcf()   { return true; }
-	static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvatureOcf"));TT::Name(name);}
+
 
 private:
 };
 
-template <class T> class CurvaturefOcf: public CurvatureOcf<float, T> {};
-template <class T> class CurvaturedOcf: public CurvatureOcf<double, T> {};
+template <class T> class CurvaturefOcf: public CurvatureOcf<float, T> {public: static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvaturefOcf"));T::Name(name);} };
+template <class T> class CurvaturedOcf: public CurvatureOcf<double, T> {public: static void Name(std::vector<std::string> & name){name.push_back(std::string("CurvaturedOcf"));T::Name(name);} };
 
 
 ///*-------------------------- CURVATURE DIR ----------------------------------*/
@@ -641,65 +653,55 @@ public:
 
 namespace tri
 {
+template < class VertexType >
+bool VertexVectorHasPerVertexVFAdjacency(const vertex::vector_ocf<VertexType> &fv)
+{
+  if(VertexType::HasVFAdjacencyOcf()) return fv.IsVFAdjacencyEnabled();
+  else return VertexType::HasVFAdjacency();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexRadius(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasRadiusOcf()) return fv.IsRadiusEnabled();
+	else return VertexType::HasRadius();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexQuality(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasQualityOcf()) return fv.IsQualityEnabled();
+	else return VertexType::HasQuality();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexNormal(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasNormalOcf()) return fv.IsNormalEnabled();
+	else return VertexType::HasNormal();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexColor(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasColorOcf()) return fv.IsColorEnabled();
+	else return VertexType::HasColor();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexCurvature(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasCurvatureOcf()) return fv.IsCurvatureEnabled();
+	else return VertexType::HasCurvature();
+}
+template < class VertexType >
+bool VertexVectorHasPerVertexCurvatureDir(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasCurvatureDirOcf()) return fv.IsCurvatureDirEnabled();
+	else return VertexType::HasCurvatureDir();
+}
 
-	template < class, class,class, class> class TriMesh;
-
-	template < class VertexType, class ContainerType0, class Container1, class Container2  >
-		bool HasPerVertexVFAdjacency (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0, Container1, Container2 > & m)
-	{
-		if(VertexType::HasVFAdjacencyOcf()) return m.vert.IsVFAdjacencyEnabled();
-		else return VertexType::HasVFAdjacency();
-	}
-
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2 >
-	bool HasPerVertexRadius (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2 > & m)
-	{
-		if(VertexType::HasRadiusOcf()) return m.vert.IsRadiusEnabled();
-		else return VertexType::HasRadius();
-	}
-	
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2>
-		bool HasPerVertexQuality (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2> & m)
-	{
-		if(VertexType::HasQualityOcf()) return m.vert.IsQualityEnabled();
-		else return VertexType::HasQuality();
-	}
-	
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2 >
-		bool HasPerVertexTexCoord (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2> & m)
-	{
-		if(VertexType::HasTexCoordOcf()) return m.vert.IsTexCoordEnabled();
-		else return VertexType::HasTexCoord();
-	}
-
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2 >
-		bool HasPerVertexNormal (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2> & m)
-	{
-		if(VertexType::HasNormalOcf()) return m.vert.IsNormalEnabled();
-		else return VertexType::HasNormal();
-	}
-
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2>
-		bool HasPerVertexColor (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2> & m)
-	{
-		if(VertexType::HasColorOcf()) return m.vert.IsColorEnabled();
-		else return VertexType::HasColor();
-	}
-
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2 >
-		bool HasPerVertexCurvature (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2 > & m)
-	{
-		if(VertexType::HasCurvatureOcf()) return m.vert.IsCurvatureEnabled();
-		else return VertexType::HasCurvature();
-	}
-
-	template < class VertexType, class ContainerType0, class ContainerType1 ,class ContainerType2>
-		bool HasPerVertexCurvatureDir (const TriMesh < vertex::vector_ocf< VertexType > , ContainerType0,   ContainerType1,  ContainerType2 > & m)
-	{
-		if(VertexType::HasCurvatureDirOcf()) return m.vert.IsCurvatureDirEnabled();
-		else return VertexType::HasCurvatureDir();
-	}
-
+template < class VertexType >
+bool VertexVectorHasPerVertexTexCoord(const vertex::vector_ocf<VertexType> &fv)
+{
+	if(VertexType::HasTexCoordOcf()) return fv.IsTexCoordEnabled();
+	else return VertexType::HasTexCoord();
+}
 }
 }// end namespace vcg
 #endif
