@@ -20,122 +20,6 @@
  * for more details.                                                         *
  *                                                                           *
  ****************************************************************************/
-/****************************************************************************
-  History
-
-$Log: not supported by cvs2svn $
-Revision 1.33  2007/06/07 15:16:39  fiorin
-Added IntersectionSphereTriangle
-
-Revision 1.32  2007/05/29 14:33:29  fiorin
-Added IntersectionSegmentSphere
-
-Revision 1.31  2007/04/16 09:08:15  cignoni
-commented out non compiling intersectionSpherePlane
-
-Revision 1.30  2007/04/10 22:26:47  pietroni
-IntersectionPlanePlane first parameter is a const
-
-Revision 1.29  2007/04/04 23:19:40  pietroni
-- Changed name of intersection function between plane and triangle from Intersection to IntersectionPlaneTriangle.
-- Added Intersection_Plane_Sphere function.
-
-Revision 1.28  2007/02/21 02:40:52  m_di_benedetto
-Added const qualifier to bbox parameter in Intersection_Triangle_Box().
-
-Revision 1.27  2006/10/25 16:04:32  pietroni
-added intersection control between bounding boxes for intersection between segment and triangle function
-
-Revision 1.26  2006/09/14 08:39:07  ganovelli
-Intersection_sphere_sphere added
-
-Revision 1.25  2006/06/06 14:35:31  zifnab1974
-Changes for compilation on linux AMD64. Some remarks: Linux filenames are case-sensitive. _fileno and _filelength do not exist on linux
-
-Revision 1.24  2006/06/01 08:38:02  pietroni
-Added functions:
-
-- Intersection_Segment_Triangle
-- IntersectionPlaneBox
-- Intersection_Triangle_Box
-
-Revision 1.23  2006/03/29 07:53:36  cignoni
-Missing ';' (thx Maarten)
-
-Revision 1.22  2006/03/20 14:42:49  pietroni
-IntersectionSegmentPlane and IntersectionSegmentBox functions Added
-
-Revision 1.21  2006/01/20 16:35:51  pietroni
-added IntersectionSegmentBox function
-
-Revision 1.20  2005/10/03 16:07:50  ponchio
-Changed order of functions intersection_line_box and
-intersectuion_ray_box
-
-Revision 1.19  2005/09/30 13:11:39  pietroni
-corrected 1 compiling error on Ray_Box_Intersection function
-
-Revision 1.18  2005/09/29 15:30:10  pietroni
-Added function RayBoxIntersection, renamed intersection line box from "Intersection" to "Intersection_Line_Box"
-
-Revision 1.17  2005/09/29 11:48:00  m_di_benedetto
-Added functor RayTriangleIntersectionFunctor.
-
-Revision 1.16  2005/09/28 19:40:55  m_di_benedetto
-Added intersection for ray-triangle (with Ray3 type).
-
-Revision 1.15  2005/06/29 15:28:31  callieri
-changed intersection names to more specific to avoid ambiguity
-
-Revision 1.14  2005/03/15 11:22:39  ganovelli
-added intersection between tow planes (porting from old vcg lib)
-
-Revision 1.13  2005/01/26 10:03:08  spinelli
-aggiunta intersect  ray-box
-
-Revision 1.12  2004/10/13 12:45:51  cignoni
-Better Doxygen documentation
-
-Revision 1.11  2004/09/09 14:41:32  ponchio
-forgotten typename SEGMENTTYPE::...
-
-Revision 1.10  2004/08/09 09:48:43  pietroni
-correcter .dir to .Direction and .ori in .Origin()
-
-Revision 1.9  2004/08/04 20:55:02  pietroni
-added rey triangle intersections funtions
-
-Revision 1.8  2004/07/11 22:08:04  cignoni
-Added a cast to remove a warning
-
-Revision 1.7  2004/05/14 03:14:29  ponchio
-Fixed some minor bugs
-
-Revision 1.6  2004/05/13 23:43:54  ponchio
-minor bug
-
-Revision 1.5  2004/05/05 08:21:55  cignoni
-syntax errors in inersection plane line.
-
-Revision 1.4  2004/05/04 02:37:58  ganovelli
-Triangle3<T> replaced by TRIANGLE
-Segment<T> replaced by EDGETYPE
-
-Revision 1.3  2004/04/29 10:48:44  ganovelli
-error in plane segment corrected
-
-Revision 1.2  2004/04/26 12:34:50  ganovelli
-plane line
-plane segment
-triangle triangle added
-
-Revision 1.1  2004/04/21 14:22:27  cignoni
-Initial Commit
-
-
-****************************************************************************/
-
-
 
 #ifndef __VCGLIB_INTERSECTION_3
 #define __VCGLIB_INTERSECTION_3
@@ -333,10 +217,9 @@ namespace vcg {
 		return penetration_detected;
 	}; //end of IntersectionSphereTriangle
 
-
   /// intersection between line and plane
   template<class T>
-    inline bool IntersectionLinePlane( const Plane3<T> & pl, const Line3<T> & li, Point3<T> & po){
+    inline bool IntersectionPlaneLine( const Plane3<T> & pl, const Line3<T> & li, Point3<T> & po){
     const T epsilon = T(1e-8);
 
     T k = pl.Direction().dot(li.Direction());						// Compute 'k' factor
@@ -346,14 +229,28 @@ namespace vcg {
     po = li.Origin() + li.Direction()*r;
     return true;
   }
-	
+    /// intersection between line and plane
+    template<class T>
+    inline bool IntersectionLinePlane(const Line3<T> & li, const Plane3<T> & pl, Point3<T> & po){
+
+      return IntersectionPlaneLine(pl,li,po);
+    }
+
    /// intersection between segment and plane
   template<class T>
     inline bool IntersectionPlaneSegment( const Plane3<T> & pl, const Segment3<T> & s, Point3<T> & p0){
 		T p1_proj = s.P1()*pl.Direction()-pl.Offset();
 		T p0_proj = s.P0()*pl.Direction()-pl.Offset();
-		if ( (p1_proj>0)-(p0_proj<0)) return false;
-		p0 =  s.P0() + (s.P1()-s.P0()) * fabs(p0_proj/(p1_proj-p0_proj));
+    if ( (p1_proj>0)-(p0_proj<0)) return false;
+
+    if(p0_proj == p1_proj) return false;
+
+    // check that we perform the computation in a way that is independent with v0 v1 swaps
+    if(p0_proj < p1_proj)
+           p0 =  s.P0() + (s.P1()-s.P0()) * fabs(p0_proj/(p1_proj-p0_proj));
+    if(p0_proj > p1_proj)
+           p0 =  s.P1() + (s.P0()-s.P1()) * fabs(p1_proj/(p0_proj-p1_proj));
+
 		return true;
   }
 
@@ -377,6 +274,8 @@ namespace vcg {
 
   /// intersection between plane and triangle 
   // not optimal: uses plane-segment intersection (and the fact the two or none edges can be intersected)
+  // its use is rather dangerous because it can return inconsistent stuff on degenerate cases.
+  // added assert to underline this danger.
   template<typename TRIANGLETYPE> 
     inline bool IntersectionPlaneTriangle( const Plane3<typename TRIANGLETYPE::ScalarType> & pl, 
 			      const  TRIANGLETYPE & tr, 
@@ -388,15 +287,18 @@ namespace vcg {
           return true;
         else
         {
-          IntersectionPlaneSegment(pl,Segment3<T>(tr.P(1),tr.P(2)),sg.P1());
-          return true;
+          if(IntersectionPlaneSegment(pl,Segment3<T>(tr.P(1),tr.P(2)),sg.P1()))
+            return true;
+          else assert(0);
+              return true;
         }
       }
       else
       {
         if(IntersectionPlaneSegment(pl,Segment3<T>(tr.P(1),tr.P(2)),sg.P0()))
         {
-          IntersectionPlaneSegment(pl,Segment3<T>(tr.P(0),tr.P(2)),sg.P1());
+          if(IntersectionPlaneSegment(pl,Segment3<T>(tr.P(0),tr.P(2)),sg.P1()))return true;
+          assert(0);
           return true;
         }
       }
@@ -713,66 +615,13 @@ template<class ScalarType>
 bool IntersectionPlaneBox(const vcg::Plane3<ScalarType> &pl,
 							vcg::Box3<ScalarType> &bbox)
 {
-	typedef typename vcg::Segment3<ScalarType> SegmentType;
-	typedef typename vcg::Point3<ScalarType> CoordType;
-	SegmentType diag[4];
-	
-	CoordType intersection;
-	//find the 4 diagonals
-	diag[0]=SegmentType(bbox.P(0),bbox.P(7));
-	diag[1]=SegmentType(bbox.P(1),bbox.P(6));
-	diag[2]=SegmentType(bbox.P(2),bbox.P(5));
-	diag[3]=SegmentType(bbox.P(3),bbox.P(4));
-	ScalarType a,b,dist;
-	for (int i=0;i<3;i++)
-		//call intersection of segment and plane
-    if (vcg::IntersectionPlaneSegment(pl,diag[i],intersection))
-			return true;
-	return false;
+	ScalarType dist,dist1;
+	if(bbox.IsNull()) return false; // intersection with a  null bbox is empty
+	dist = Distance(pl,bbox.P(0)) ;
+	for (int i=1;i<8;i++)  if(  Distance(pl,bbox.P(i))*dist<0) return true;
+	return true;
 }
 
-///if exists return the center and ardius of circle
-///that is the intersectionk between the sphere and 
-//the plane
-template <class ScalarType>
-bool IntersectionPlaneSphere(const Plane3<ScalarType> &pl,
-							   const Sphere3<ScalarType> &sphere,
-							   Point3<ScalarType> &center,
-							   ScalarType &radius)
-{
-	/* ///set the origin on the center of the sphere
-	vcg::Plane3<ScalarType> pl1;
-	vcg::Point3<ScalarType> p_plane=pl.Direction()*pl.Offset();
-	vcg::Point3<ScalarType> p_plane=p_plane-sphere.Center();
-
-	///set again the plane
-	pl1.Set(p_plane,pl.Direction());
-
-	///test d must be positive
-	d=pl1.Offset();
-	vcg::Point3<ScalarType> n=pl1.Direction();
-	///invert d if is <0
-	if (d<0)
-	{
-		n=-n;
-		d=-d;
-	}
-	///no intersection occour
-	if (d>r)
-		return false;
-	else
-	{
-		///calculate center and translate in back
-		center=n*d;
-		center+=sphere.Center();
-		radius=math::Sqrt((r*r)-(d*d));
-	}
-	 */
-	 assert(0);
-	 return true;
-}
-
-
 template<class ScalarType>
 bool IntersectionTriangleBox(const vcg::Box3<ScalarType>   &bbox,
 							   const vcg::Point3<ScalarType> &p0,
@@ -781,7 +630,6 @@ bool IntersectionTriangleBox(const vcg::Box3<ScalarType>   &bbox,
 {
 	typedef typename vcg::Point3<ScalarType> CoordType;
 	CoordType intersection;
-
 	/// control bounding box collision
 	vcg::Box3<ScalarType> test;
 	test.SetNull();
