@@ -172,6 +172,7 @@ typedef typename SaveMeshType::FaceType FaceType;
 typedef typename SaveMeshType::FacePointer FacePointer;
 typedef typename SaveMeshType::VertexIterator VertexIterator;
 typedef typename SaveMeshType::FaceIterator FaceIterator;
+typedef typename SaveMeshType::EdgeIterator EdgeIterator;
 
 static int Save(SaveMeshType &m, const char * filename, bool binary=true)
 {
@@ -361,7 +362,14 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
 
 	for(i=0;i<pi.fdn;i++)
 			fprintf(fpout,"property %s %s\n",pi.FaceData[i].stotypename(),pi.FaceData[i].propname);
-
+	// Saving of edges
+	if(m.en>0)
+	  fprintf(fpout,
+			  "element edge %d\n"
+			  "property int vertex1\n"
+			  "property int vertex2\n"
+			  ,m.en
+			  );
 	fprintf(fpout, "end_header\n"	);
 
 		// Salvataggio camera
@@ -438,8 +446,8 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
 	for(j=0,vi=m.vert.begin();vi!=m.vert.end();++vi){
 		vp=&(*vi);
 		indices[vi] = j;
-
-    if(cb && ((j%1000)==0))(*cb)( (100*j)/(m.vn+m.fn), "Saving Vertices");
+	//((m.vn+m.fn) != 0) all vertices and faces have been marked as deleted but the are still in the vert/face vectors  
+    if(cb && ((j%1000)==0) && ((m.vn+m.fn) != 0) )(*cb)( (100*j)/(m.vn+m.fn), "Saving Vertices");
 
     if( !HasPerVertexFlags(m) || !vp->IsD() )
 		{
@@ -533,6 +541,7 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
 			j++;
 		}
 	}
+	/*vcg::tri::*/
 	// this assert triggers when the vn != number of vertexes in vert that are not deleted.
   assert(j==m.vn); 
 
@@ -545,7 +554,9 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
 	int fcnt=0;
 	for(j=0,fi=m.face.begin();fi!=m.face.end();++fi)
 		{
-		  if(cb && ((j%1000)==0) ) (*cb)( 100*(m.vn+j)/(m.vn+m.fn), "Saving Vertices");
+			//((m.vn+m.fn) != 0) all vertices and faces have been marked as deleted but the are still in the vert/face vectors  
+		  if(cb && ((j%1000)==0) && ((m.vn+m.fn) != 0)) 
+			  (*cb)( 100*(m.vn+j)/(m.vn+m.fn), "Saving Vertices");
 
 			fp=&(*fi);
 			if( ! fp->IsD() )
@@ -561,7 +572,7 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
           if(HasPerFaceFlags(m)&&( pi.mask & Mask::IOM_FACEFLAGS) )
 						fwrite(&(fp->Flags()),sizeof(int),1,fpout);
 
-          if( HasPerVertexTexCoord(m) && (pi.mask & Mask::IOM_WEDGTEXCOORD) ) // you can save VT as WT if you really want it...
+          if( HasPerVertexTexCoord(m) && (pi.mask & Mask::IOM_VERTTEXCOORD) )
 					{
 						fwrite(&b6,sizeof(char),1,fpout);
 						float t[6];
@@ -703,6 +714,24 @@ static int Save(SaveMeshType &m,  const char * filename, bool binary, PlyInfo &p
 			}
 		}
 	assert(fcnt==m.fn);
+	int eauxvv[2];
+	int ecnt=0;
+	for(EdgeIterator ei=m.edge.begin();ei!=m.edge.end();++ei)
+		{
+			if( ! ei->IsD() )
+			{
+				++ecnt;
+				if(binary)
+				{
+						eauxvv[0]=indices[ei->cV(0)];
+						eauxvv[1]=indices[ei->cV(1)];
+						fwrite(vv,sizeof(int),2,fpout);
+				}
+				else // ***** ASCII *****
+				  fprintf(fpout,"%d %d \n", indices[ei->cV(0)],	indices[ei->cV(1)]);
+			}
+	}
+	assert(ecnt==m.en);
 	fclose(fpout); 
 	return 0;
 }
@@ -760,6 +789,7 @@ static const char *ErrorMsg(int error)
 	  capability |= vcg::tri::io::Mask::IOM_WEDGTEXMULTI ;
     capability |= vcg::tri::io::Mask::IOM_WEDGNORMAL   ;
 		capability |= vcg::tri::io::Mask::IOM_CAMERA   ;
+		capability |= vcg::tri::io::Mask::IOM_BITPOLYGONAL;
 	  return capability;
   }
 
